@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -557,8 +551,8 @@
 ; WARNING: Do not replace (cadr form) by fn below.  These can differ!  Fn is
 ; passed to output-type-for-declare-form in order to get its 'stobjs-out, but
 ; (cadr form) can be the *1* function for fn.  The mistaken placement of fn
-; below caused a factor of 4 slowdown in GCL in the first lemma5 in
-; books/unicode/utf8-decode.lisp, because the proclaim for function
+; below caused a factor of 4 slowdown in GCL in the first lemma5 in community
+; book books/unicode/utf8-decode.lisp, because the proclaim for function
 ; utf8-combine4-guard was overwritten by a subsequent weaker proclaimed type
 ; that was supposed to be generated for the *1* function, but instead was
 ; generated for utf8-combine4-guard.
@@ -776,7 +770,9 @@ notation causes an error and (b) the use of ,. is not permitted."
                (backquote-lst (cdr l))))
         ((and (consp (car l))
               (eq (caar l) *comma-atsign*))
-         (list 'append (cadr (car l)) (backquote-lst (cdr l))))
+         (cond ((null (cdr l))
+                (cadr (car l)))
+               (t (list 'append (cadr (car l)) (backquote-lst (cdr l))))))
         (t
          (list 'cons
                (backquote (car l))
@@ -881,6 +877,8 @@ notation causes an error and (b) the use of ,. is not permitted."
 
 (defvar *inhibit-sharp-comma-warning* nil)
 
+(defvar *inside-sharp-u-read* nil)
+
 (defun sharp-comma-read (stream char n)
   (or *inhibit-sharp-comma-warning*
       (format t
@@ -927,20 +925,50 @@ notation causes an error and (b) the use of ,. is not permitted."
 ; Thanks to Pascal J. Bourguignon for suggesting this approach.
   
   (declare (ignore char n))
-  (let* ((package-name (read stream))
+  (let* ((package-name (read stream t nil t))
          (package-string (cond ((symbolp package-name)
                                 (symbol-name package-name))
                                ((stringp package-name)
                                 package-name)
                                (t nil)))
-         (*package* (if (assoc package-string
-                               (qfuncall known-package-alist *the-live-state*)
-                               :test 'equal)
-                        (qfuncall find-package-fast package-string)
+         (*package* (cond
+                     (*read-suppress* *package*)
+                     ((assoc package-string
+                             (qfuncall known-package-alist *the-live-state*)
+                             :test 'equal)
+                      (qfuncall find-package-fast package-string))
+                     (t
                       (error "There is no package named ~S that is known to ~
                               ACL2 in this context."
-                             package-name))))
-    (read stream)))
+                             package-name)))))
+    (read stream t nil t)))
+
+(defun sharp-u-read (stream char n)
+  (declare (ignore char n))
+  (let* ((*inside-sharp-u-read*
+          (or (not *inside-sharp-u-read*)
+              (error "Recursive attempt to read a sharp-u (#u)~%expression ~
+                      while inside a sharp-u expression.  This is not~%~
+                      allowed.")))
+         (x (read stream t nil t)))
+    (cond
+     ((numberp x) x)
+     ((not (symbolp x))
+      (error "Failure to read #u expression:~%~
+              #u was not followed by a symbol."))
+     (t (let* ((name (symbol-name x))
+               (c (and (not (equal name ""))
+                       (char name 0))))
+          (cond ((member c '(#\B #\O #\X))
+                 (values ; seems necessary in GCL to return a single value
+                  (read-from-string
+                   (concatenate 'string "#" (remove #\_ name)))))
+                (t (let ((n (read-from-string (remove #\_ name))))
+                     (cond ((numberp n) n)
+                           (*read-suppress* nil)
+                           (t (error "Failure to read #u expression:~%~
+                                      Result ~s is not a numeral."
+                                     n)))))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            SUPPORT FOR #@
@@ -1173,7 +1201,7 @@ notation causes an error and (b) the use of ,. is not permitted."
   (let ((fn
          #+gcl       'si::getenv
          #+allegro   'sys::getenv
-         #+lispworks 'cl::getenv
+         #+lispworks 'hcl::getenv
          #+ccl       'ccl::getenv
          #+sbcl      'sb-ext::posix-getenv
          #+clisp     'ext:getenv))
@@ -1335,6 +1363,33 @@ notation causes an error and (b) the use of ,. is not permitted."
   #-gcl ; presumably CLtL2
   (ignore-errors (user-homedir-pathname)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;             SUPPORT FOR SERIALIZE INTEGRATION INTO ACL2
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+;; The following definitions might seem to belong serialize-raw.lisp, and
+;; that's where they lived when we only built ACL2(h) with CCL.  But Jared
+;; Davis noticed that SBCL 1.0.46 didn't let him add undefined functions into
+;; the readtable.  Note also that it doesn't seem sufficient to give the
+;; function symbols temporary definitions and redefine them later: the
+;; readtable still uses the old functions.  So to solve this, we move the
+;; functions over from serialize-raw.lisp to here.
+
+(declaim (ftype (function (t t t) (values t))
+                ser-decode-from-stream))
+
+(defun ser-cons-reader-macro (stream subchar arg)
+  (declare (ignorable subchar arg))
+  ;; This is the reader macro for #z.  When it is called the #z part has
+  ;; already been read, so we just want to read the serialized object.
+  (ser-decode-from-stream nil :never stream))
+
+(defun ser-hons-reader-macro (stream subchar arg)
+  (declare (ignorable subchar arg))
+  ;; This is the reader macro for #Z.  When it is called the #Z part has
+  ;; already been read, so we just want to read the serialized object.
+  (ser-decode-from-stream nil :smart stream))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            MISCELLANY
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1348,25 +1403,80 @@ notation causes an error and (b) the use of ,. is not permitted."
 
 (defmacro special-form-or-op-p (name)
 
-; This odd macro deals with the problem that the notion of special operator
-; is defined differently in different lisps.
+; At one time we thought that special-operator-p does not work in all Lisps.
+; But if that was true, it no longer seems to be the case in October 2011.
 
-; Even though cmu lisp purports to be CLTL2, in fact it does not
-; contain a defun for special-form-p and does contain a defun of
-; special-operator-p.
-
-; SBCL makes no claim to being CLTL2.  Unfortunately acl2.lisp helpfully
-; pushes :CLTL2 to *FEATURES* anyway (for all :ANSI-CL implementations).
-
-  #+(and (not DRAFT-ANSI-CL-2) CLTL2
-         (not cmu) (not sbcl) (not clisp) (not ccl))
-  `(common-lisp::special-form-p ,name)
-  #+ccl
-  `(common-lisp::special-operator-p ,name)
-  #+(and (not DRAFT-ANSI-CL-2) (not CLTL2)
-         (not cmu) (not sbcl) (not clisp) (not ccl))
-  `(lisp::special-form-p ,name)
-  #+(or (and DRAFT-ANSI-CL-2 (not ccl)) cmu sbcl clisp)
   `(special-operator-p ,name))
 
 (defvar *startup-package-name* "ACL2")
+
+(defmacro save-def (def-form)
+
+; For each definition (save-def (defun name formals ... body)), where defun
+; could be replaced by other macros that take the same arguments (like
+; defun-one-output or defn1), this macro executes the definition and for the
+; hons version, also saves (name formals ... body) as the 'acl2-saved-def
+; property of name.  We use this property to obtain the raw Lisp definition of
+; name for memoize-fn, for Lisps like SBCL where function-lambda-expression
+; doesn't seem to work for us.
+
+; This macro is intended only for raw Lisp definitions.  For definitions in the
+; loop, we expect that cltl-def-from-name will give us the definition.
+
+  #+hons
+  (let* ((def (cdr def-form))
+         (name (car def)))
+    `(progn ,def-form
+            (setf (get ',name 'acl2-saved-def)
+                  ',def-form)))
+  #-hons
+  def-form)
+
+(defun our-function-lambda-expression (sym)
+
+; This is intended only for #+hons; otherwise it reduces to (mv
+; (function-lambda-expression (symbol-function sym)) nil).
+
+  #-cltl2
+  (declare (ignore sym))
+  #-cltl2
+  (mv nil nil)
+  #+cltl2
+  (let ((temp (get sym 'acl2-saved-def)))
+    (cond (temp (values temp t))
+          (t (let* ((fn (symbol-function sym))
+                    (lam (and fn (function-lambda-expression fn))))
+               (cond (lam (values lam nil))
+                     (t (values nil nil))))))))
+
+; [Comment from Jared]: We probably should work toward getting rid of
+; defg/defv's in favor of regular parameters...
+
+(defmacro defg (&rest r)
+
+  "DEFG is a short name for DEFPARAMETER.  However, in Clozure Common
+  Lisp (CCL) its use includes two promises: (1) never to locally bind
+  the variable, e.g., with LET or LAMBDA, and (2) never to call
+  MAKUNBOUND on the variable.  CCL uses fewer machine instructions to
+  reference such a variable than an ordinary special, which may have a
+  per-thread binding that needs to be locked up."
+
+  #-Clozure
+  `(defparameter ,@r)
+  #+Clozure
+  `(ccl::defstatic ,@r))
+
+(defmacro defv (&rest r)
+
+  "DEFV is a short name for DEFVAR.  However, in Clozure Common Lisp
+  (CCL) its use includes two promises: (1) never to locally bind the
+  variable, e.g., with LET or LAMBDA, and (2) never to call MAKUNBOUND
+  on the variable.  CCL uses fewer machine instructions to reference
+  such a variable than an ordinary special, which may have a
+  per-thread binding that needs to be locked up.  Unlike for DEFVAR,
+  the second argument of DEFV is not optional."
+
+  #-Clozure
+  `(defparameter ,@r)
+  #+Clozure
+  `(ccl::defstaticvar ,@r))
