@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -26,36 +20,142 @@
 
 ; This file cannot be compiled because it changes packages in the middle.
 
-; We restrict the hons version to CCL (starting with v3-6), because of
-; CCL-specific changes made to hons-raw.lisp.  With some work this restriction
-; could probably be relaxed, but it's not clear that it's worth the effort.
-#+(and hons (not ccl))
-
-(error "It is illegal to build the experimental HONS version
-of ACL2 in this Common Lisp.  For that version, the
-underlying Common Lisp must be CCL (Clozure Common Lisp,
-formerly OpenMCL).  CCL runs on many platforms, including
-both 32-bit and 64-bit Linux.  See the ACL2 installation
-instructions for how to obtain CCL.")
-
 ; Allow taking advantage of threads in SBCL, CCL, and Lispworks (where we may
-; want to build a parallel version, which needs this to take place).
-#+(or (and sbcl sb-thread) ccl lispworks)
+; want to build a parallel version, which needs this to take place).  At the
+; time that we add (perhaps once again) support for HONS in other lisps besides
+; CCL (August, 2012), there has been code developed that depends on mv-let
+; being the same as multiple-value-bind; see for example community book
+; books/centaur/aig/bddify.lisp, in particular the raw Lisp definition of
+; count-branches-to, specifically the use of b* in the labels definition of
+; lookup.  So we also use multiple-value-bind for mv-let when building the HONS
+; version.
+#+(or (and sbcl sb-thread) ccl lispworks hons)
 (push :acl2-mv-as-values *features*)
 
-; Essay on Parallelism and Parallelism Warts
+; Essay on Parallelism, Parallelism Warts, Parallelism Blemishes, Parallelism
+; No-fixes, Parallelism Hazards, and #+ACL2-PAR notes.
 
-; These sources incorporate code for an experimental extension for parallelism,
-; initiated in David Rager's dissertation work.  That extension may be produced
-; by setting the :acl2-par feature, for example using make (see :DOC
-; compiling-acl2p).  This work is evolving, but we have taken great pains to
-; preserve the functionality of ACL2 without parallelism.  As the parallelism
-; extension is still under active development as of this writing (May, 2011),
-; we use the phrase "Parallelism wart:" to annotate comments about known issues
-; for the #+acl2-par build of ACL2.  These range from minor comment suggestions
-; to deeper questions concerning the correctness of the code.  Rager intends to
-; address most of these parallelism warts by the time his dissertation work is
-; complete.
+; These sources incorporate code for an experimental extension for parallelism
+; contributed by David Rager during his master's and Ph.D. dissertation work.
+; That extension may be built by setting the :acl2-par feature, for example
+; using make (see :DOC compiling-acl2p).  The incorporation of code supporting
+; parallelism has been carried out while taking great pains to preserve the
+; functionality of the ACL2 system proper (i.e., without the experimental
+; extension for parallelism).
+
+; At the lowest level, parallel computation is carried out by Lisp threads,
+; which provide Lisp-level abstractions of OS threads.  A thread can be running
+; or blocked (e.g., waiting for a semaphore signal).  Threads can create other
+; threads.  We manage the creation and use of threads to reflect the resources
+; we have available, for example the number of available cpu cores according to
+; our own tracking.
+
+; The implementation of the multi-threading primitives -- futures, spec-mv-let,
+; plet, pargs, pand, and por -- has a dependency structure shown as follows.
+; For example, multi-threading primitives are at the base of everything,
+; futures and plet/pargs/pand/por are built on top of these primitives, and so
+; forth, as indicated by the indentations.
+
+; multi-threading primitives (semaphores, locks, condition variables, etc)
+;   futures
+;     spec-mv-let
+;       waterfall parallelism
+;       user-level program parallelism (no examples as of April 2012)
+;   plet, pargs, pand, por
+;     user-level program parallelism (e.g., community book
+;                                     books/parallel/fibonacci.lisp)
+
+; This dependency structure roughly correlates to the following file structure.
+
+; #+acl2-par
+; multi-threading-raw.lisp (defines multi-threading primitives)
+;   parallel-raw.lisp  (provides raw Lisp defs. of plet, pargs, pand, and por)
+;     futures-raw.lisp (defines futures and uses some helper functions
+;                       from parallel-raw.lisp)
+;       parallel.lisp
+; #-acl2-par
+; parallel.lisp
+
+; One might wonder how we use threads to execute pieces of parallelism work
+; (where "pieces of parallelism work" can mean either futures, bindings of
+; plet, arguments of calls surrounded by pargs, or arguments given to pand or
+; por).  For futures, the story is as follows.
+
+; (1) A primitive adds a piece of parallelism work to one of the two
+;     parallelism queues (either *future-array* or *work-queue*, as
+;     appropriate).
+;
+; (2) The primitive checks to see if there are enough threads already in
+;     existence to process that piece of parallelism work.  If so, the
+;     primitive returns and execution continues until the result of the
+;     parallelized computation is needed.  If not, the primitive creates one to
+;     many "worker threads" to consume pieces of parallelism work.
+;
+; (3) The primitive may eventually need the result from the piece of
+;     parallelism work.  In this case, it will read the value from the piece of
+;     parallelism work (once it is available) and use it as appropriate.  In
+;     the case that the primitive does not need the value (as can happen with a
+;     pand/por or a spec-mv-let, when the speculative computation is determined
+;     to be useless), the primitive will abort (or early terminate) the
+;     parallel execution of the piece of parallelism work.
+;
+; When a worker thread is created, it performs the following sequence of
+; steps.
+;
+; (A) Waits until there is a piece of parallelism work to consume.  A worker
+;     thread will wait between 10 and 120 seconds before "giving up", unwinding
+;     itself, and freeing itself as a resource for the operating system to
+;     collect.  The reader might be tempted to assume that there would
+;     immediately be work, but this is not guaranteed to be the case (because,
+;     for efficiency reasons, we typically create a handful more threads than
+;     are needed).
+;
+; (B) Waits until there is an idle CPU core available, as determined by our
+;     resource management using the multi-threading primitives available to us
+;     in the Lisp (as opposed to trying to tell the operating system how to
+;     schedule our threads).  There is no timeout associated with this wait.
+;
+; (C) Making it to (C) requires that the thread first made it through (A) and
+;     then also made it through (B).  Thus, the thread has both a piece of
+;     parallelism work and a CPU core.  At this point, the thread executes that
+;     piece of parallelism work.
+;
+; (D) Perhaps the piece of parallelism work itself encounters a parallelism
+;     primitive and decides to further parallelize execution.  In this case,
+;     the worker thread will do (1), (2), and (3), as explained above.
+;
+; (E) At this point, the worker thread has finished executing the piece of
+;     parallelism work and stores the result of that execution in the
+;     appropriate place (e.g., for futures, it stores the execution result in
+;     the "value" slot of the future).
+;
+; (F) After performing some cleanup, the thread goes back to (A).
+
+; We use the phrase "Parallelism wart:" to label comments about known issues
+; for the #+acl2-par build of ACL2 that we would like to fix, time permitting.
+; We also use the phrase "Parallelism blemish:" to identify known issues for
+; the #+acl2-par build of ACL2 that we intend to fix only if led to do so by
+; user complaints.  Finally, we use the phrase "Parallelism no-fix:" to label
+; comments about known issues for the #+acl2-par build of ACL2 that we do not
+; intend to fix, though we could reclassify these if there are sufficiently
+; strong user complaints.  Searching through the parallism warts, blemishes,
+; and no-fixes could be useful when a user reports a bug in #+acl2-par that
+; cannot be replicated in #-acl2-par.
+
+; Parallelism hazards are unrelated to parallelism warts, blemishes, and
+; no-fixes.  Parallelism hazards are macros or functions that are known to be
+; theoretically unsafe when performing multi-threaded execution.  We originally
+; did not expect users to encounter parallelism hazards (because we should have
+; programmed such that the hazards never occur).  However, in practice, these
+; parallelism hazards are somewhat common and we have disabled the automatic
+; warning that occurs everytime a hazard occurs.  Once we re-enable that
+; warning, in the event that users encounter a parallelism hazard, they will be
+; asked to report the associated warning to the ACL2 maintainers.  For example,
+; if state-global-let* is called while executing concurrently, we want to know
+; about it and develop a work-around.  See *possible-parallelism-hazards* and
+; warn-about-parallelism-hazard for more information.
+
+; #+ACL2-PAR notes contain documentation that only applies to #+acl2-par.
 
 ; In an effort to avoid code duplication, we created a definition scheme that
 ; supports defining both serial and parallel versions of a function with one
@@ -80,10 +180,10 @@ instructions for how to obtain CCL.")
 
 ; Only allow the feature :acl2-par in environments that support
 ; multi-threading.  Keep this in sync with the error message about CCL,
-; Lispworks, and SBCL in set-parallel-evaluation-fn and with :doc
+; Lispworks, and SBCL in set-parallel-execution-fn and with :doc
 ; compiling-acl2p.  If we add support for non-ANSI GCL, consider providing a
 ; call of reset-parallelism-variables analogous to the one generated by setting
-; *reset-parallelism-variables* in our-abort.
+
 #+(and acl2-par (not ccl) (not (and sbcl sb-thread)) (not lispworks))
 (error "It is currently illegal to build the parallel version of ACL2 in this
 Common Lisp.  See source file acl2-init.lisp for this error message,
@@ -93,17 +193,37 @@ implementations.")
 #+akcl
 (setq si:*notify-gbc* t)
 
-; Dave Greve reported a problem: the saved_acl2 script in CLISP had characters
-; that, contrary to expectation, were not being interpreter as newlines.  The
-; CLISP folks explained that "CUSTOM:*DEFAULT-FILE-ENCODING* defaults to :DOS
-; on cygwin, so #\Newline is printed as '\r\n' (CRLF)."  We expect that the
-; following setting will fix the problem; Dave tried an experiment for us that
-; seemed to validate this expectation.
-#+clisp
-(setq CUSTOM:*DEFAULT-FILE-ENCODING* :unix)
+; The following has been superseded; see the section on reading characters from
+; files in acl2.lisp.
+;  ; Dave Greve reported a problem: the saved_acl2 script in CLISP had characters
+;  ; that, contrary to expectation, were not being interpreter as newlines.  The
+;  ; CLISP folks explained that "CUSTOM:*DEFAULT-FILE-ENCODING* defaults to :DOS
+;  ; on cygwin, so #\Newline is printed as '\r\n' (CRLF)."  We expect that the
+;  ; following setting will fix the problem; Dave tried an experiment for us that
+;  ; seemed to validate this expectation.
+;  #+clisp
+;  (setq CUSTOM:*DEFAULT-FILE-ENCODING* :unix)
 
-#+lispworks
-(setq system::*stack-overflow-behaviour* nil) ; could be :warn
+#+(and lispworks (not acl2-par))
+(setq system::*stack-overflow-behaviour*
+
+; The following could reasonably be nil or :warn.  David Rager did some
+; experiments suggesting that ACL2(p) regressions (as of July 2011) may be more
+; robust with the :warn setting.  However, that setting causes warnings during
+; the build, and it's easy to imagine that ACL2 users would also see that
+; cryptic warning -- very un-ACL2-like!  So we use nil rather than :warn here.
+
+      nil)
+
+#+(and lispworks acl2-par)
+(setq system:*stack-overflow-behaviour* 
+
+; Since a setting of nil is at least sometimes (if not always) ignored when
+; safety is set to 0 (according to an email communication between David Rager
+; and Martin Simmons), we choose to use the warn setting for the #+acl2-par
+; build.
+
+      :warn)
 
 ; We have observed a significant speedup with Allegro CL when turning off
 ; its cross-referencing capability.  Here are the times before and after
@@ -125,6 +245,15 @@ implementations.")
 
 (load "acl2.lisp")
 
+; We allow ACL2(h) code to take advantage of Ansi CL features.  It's
+; conceivable that we don't need this restriction (which only applies to GCL),
+; but it doesn't currently seem worth the trouble to figure that out.
+#+(and hons (not cltl2))
+(progn
+  (format t "~%ERROR: It is illegal to build the experimental HONS version
+       of ACL2 in this non-ANSI Common Lisp.~%~%")
+  (acl2::exit-lisp))
+
 ; Fix a bug in SBCL 1.0.49 (https://bugs.launchpad.net/bugs/795705), thanks to
 ; patch provided by Nikodemus Siivola.
 #+sbcl
@@ -158,6 +287,19 @@ implementations.")
 
 (in-package "ACL2")
 
+(defconstant *current-acl2-world-key*
+
+; *Current-acl2-world-key* is the property used for the current-acl2-world
+; world.  We formerly used a defvar, but there seemed to be no reason to use a
+; special variable, which Gary Byers has pointed out takes about 5 instructions
+; to read in CCL in order to check if a thread-local binding is dynamically in
+; effect.  So we tried using a defconstant.  Unfortunately, that trick failed
+; in Allegro CL; even if we use a boundp test to guard the defconstant, when we
+; used make-symbol to create the value; the build failed.  So the value is now
+; an interned symbol.
+
+  'acl2_invisible::*CURRENT-ACL2-WORLD-KEY*)
+
 #+(and gcl hons)
 (when (not (gcl-version->= 2 7 0))
   (format t
@@ -182,7 +324,7 @@ implementations.")
 
 (defun system-call (string arguments)
   #+akcl
-  (lisp::system
+  (si::system
    (let ((result string))
      (dolist
       (x arguments)
@@ -496,7 +638,8 @@ implementations.")
                                    "memoize-raw.lisp"
                                    "multi-threading-raw.lisp"
                                    "futures-raw.lisp"
-                                   "parallel-raw.lisp")))))
+                                   "parallel-raw.lisp"
+                                   "serialize-raw.lisp")))))
 
 (defvar *saved-build-date-lst*)
 (defvar *saved-mode*)
@@ -529,10 +672,10 @@ implementations.")
   (concatenate
    'string
    "~% ~a built ~a.~
-    ~% Copyright (C) 2011  University of Texas at Austin"
+    ~% Copyright (C) 2012, Regents of the University of Texas"
    "~% ACL2 comes with ABSOLUTELY NO WARRANTY.  This is free software and you~
     ~% are welcome to redistribute it under certain conditions.  For details,~
-    ~% see the GNU General Public License.~%"
+    ~% see the LICENSE file distributed with ACL2.~%"
 
 ; The following is empty if acl2-startup-info.txt begins with the symbol
 ; :release.
@@ -886,6 +1029,11 @@ implementations.")
   (if *acl2-default-restart-complete*
       (return-from acl2-default-restart nil))
 
+  (#+cltl2
+   common-lisp-user::acl2-set-character-encoding
+   #-cltl2
+   user::acl2-set-character-encoding)
+
   #+ccl
   (progn
 
@@ -908,32 +1056,6 @@ implementations.")
   (maybe-load-acl2-init)
   (eval `(in-package ,*startup-package-name*))
 
-; Remark for #+acl2-par.  Here (for 64-bit Lispworks) and in parallel-raw.lisp
-; (for CCL and SBCL), we set the gc-threshold to a high number.  If the Lisps
-; support it, this threshold could be based off the actual memory in the
-; system, but we just leave it at 2 gigabytes for now.  We peform this setting
-; of the threshold for Lispworks inside the restart function, because Lispworks
-; doesn't save the GC configuration as part of the Lisp image.
-
-; Parallelism wart: the 1 gigabyte threshold may cause problems for machines
-; with less than 1 gigabytes of free RAM.
-
-  #+(and acl2-par (and lispworks lispworks-64bit))
-  (progn
-
-; Calling set-gen-num-gc-threshold sets the GC threshold for the given
-; generation of garbage.
-
-    (system:set-gen-num-gc-threshold 0 (expt 2 20))
-    (system:set-gen-num-gc-threshold 1 (expt 2 27))
-    (system:set-gen-num-gc-threshold 2 (expt 2 28))
-
-; This call to set-blocking-gen-num accomplishes two things: (1) It sets the
-; third generation as the "final" generation -- nothing can be promoted to
-; generation four or higher.  (2) It sets the GC threshold for generation 3.
-
-    (system:set-blocking-gen-num 3 :gc-threshold (expt 2 30)))
-   
 ; The following two lines follow the recommendation in Allegro CL's
 ; documentation file doc/delivery.htm.
 
@@ -984,21 +1106,11 @@ implementations.")
 ; ;; No live processes except internal servers - stopping multiprocessing
 
 ; So we have decided not to call :multiprocessing t, and also not to call
-; (mp:initialize-multiprocessing).  LispWorks 6.0 seems to work fine for ACL2,
-; so it seems that we need not think further about
-; mp:initialize-multiprocessing until perhaps LispWorks is supported with
-; #+acl2-par.
-
-; Parallelism wart: it's unclear whether calling stop-multiprocessing really
-; works.  We are currently seeing an error in ACL2(p) on Lispworks, where
-; cons's are reported as misaligned when including books and also when
-; executing proofs in parallel.  This error didn't occur initially, and it
-; started surfacing about the time that we started calling
-; stop-multiprocessing.
+; (mp:initialize-multiprocessing) in the #-acl2-par case.
 
   #+acl2-par
   (when mp::*multiprocessing*
-    (send-die-to-all-except-initial-threads)
+    (send-die-to-worker-threads)
     (mp::stop-multiprocessing)
     (gc$))
   #+acl2-par
@@ -1144,6 +1256,53 @@ implementations.")
   (setq *acl2-default-restart-complete* nil)
   (save-acl2-in-cmulisp-aux sysout-name sysout-name))
 
+#+sbcl
+(defvar *sbcl-dynamic-space-size*
+
+; The user is welcome to set this value, either by setting this variable before
+; saving an ACL2 image, or by editing the resulting script (e.g., saved_acl2 or
+; saved_acl2h).  Here we explain the defaults that we provide for this
+; variable.
+
+; We observed during development of Version_5.0 that --dynamic-space-size 2000
+; is necessary in order to complete an ACL2(h) regression with SBCL 1.0.55 on a
+; Mac OS 10.6 laptop; otherwise Lisp dies during a GC when certifying community
+; book books/centaur/tutorial/intro.lisp, even with (clear-memoize-tables)
+; executed at the start of acl2-compile-file and with (gc$ :full t) executed
+; there as well, and also at the start of write-expansion-file and immediately
+; before and after include-book-fn in certify-book-fn.  We believe that it has
+; been necessary to use such a --dynamic-space-size setting even to build ACL2
+; (not only ACL2(h)) with SBCL on some platforms, so we decided to use this
+; option for ACL2, not just ACL2(h).
+
+; But in December 2012 we found that 2000 is not sufficient using SBCL 1.0.49
+; on our 64-bit linux system.  Our first such failure was in certifying
+; community book
+; books/models/y86/y86-two-level-abs/common/x86-state-concrete.lisp in ACL2(h).
+; We tried increasing the --dynamic-space-size to 4000, but that was not
+; sufficient for community book
+; books/models/y86/y86-basic/common/x86-state.lisp; in fact, 8000 also was not
+; sufficient for that book.  Fortunately, 16000 was sufficient.
+
+; These are unusual books, in that they allocate an array of size 2^32.
+; Therefore we only increase the value to 16000 under #+hons; after all, the
+; ACL2 regression (as opposed to ACL2(h)) does not certify the y86 books in
+; ACL2.  If --dynamic-space-size 16000 causes a problem for some ACL2(h) users,
+; a simple solution will be for them to edit saved_acl2 or for them to build
+; ACL2 after defining this variable to be smaller than 16000 (though some
+; community book certifications may fail under under books/models/y86/, which
+; are done by default for ACL2(h)).
+
+; On 32-bit systems, 16000 may be too large.  We tried it on a 32-bit Linux
+; system and got an error upon starting ACL2: "--dynamic-space-size argument is
+; out of range: 16000".  So we revert to our earlier value of 2000 for such
+; systems, even if we are doing an ACL2(h) build.  (The y86 books will likely
+; fail in this case, but we expect ACL2(h) users will generally be on 64-bit
+; systems.)
+
+  #+(and x86-64 hons) 16000
+  #-(and x86-64 hons) 2000)
+
 #+sbcl
 (defun save-acl2-in-sbcl-aux (sysout-name core-name)
   (declaim (optimize (sb-ext:inhibit-warnings 3)))
@@ -1186,8 +1345,19 @@ implementations.")
                        "export SBCL_HOME=~s"
                        contrib-dir)
              "")))
-        "~s --core ~s~a --eval '(acl2::sbcl-restart)'"
+
+; We have observed with SBCL 1.0.49 that "make HTML" fails on our 64-bit linux
+; system unless we start sbcl with --control-stack-size 4 [or larger].  The
+; default, according to http://www.sbcl.org/manual/, is 2.  The problem seems
+; to be stack overflow from fmt0, which is not tail recursive.
+
+; See *sbcl-dynamic-space-size* for an explanation of the --dynamic-space-size
+; setting below.
+
+        "~s --dynamic-space-size ~s --control-stack-size 4 --core ~s~a ~
+         --eval '(acl2::sbcl-restart)'"
         prog
+        *sbcl-dynamic-space-size*
         eventual-sysout-core
 
 ; We have found (August 2008) that SBCL 1.0.13 on Windows does not like
@@ -1340,18 +1510,19 @@ implementations.")
      (str sysout-name :direction :output)
      (write-exec-file str
                       nil
-                      "~s -i ~s -p ACL2 -M ~s -m ~dMB $*~%"
+                      "~s -i ~s -p ACL2 -M ~s -m ~dMB -E ISO-8859-1 $*~%"
                       (or (ext:getenv "LISP") "clisp")
                       (rc-filename save-dir)
                       eventual-sysout-mem
 
 ; Here we choose a value of 10 for the -m argument.  We have found that without
-; setting -m, we get a stack overflow in books/unicode/read-utf8.lisp at
-; (verify-guards read-utf8-fast ...) when running on CLISP 2.34 on Linux.
-; CLISP documentation at http://clisp.cons.org/clisp.html#opt-memsize says that
-; it is "common to specify 10 MB" for the value of -m; since that suffices to
-; eliminate the stack overflow mentioned above, we use that value.  Note that
-; we use ~dMB instead of ~sMB because the with-standard-io-syntax wrapper in
+; setting -m, we get a stack overflow in community book
+; books/unicode/read-utf8.lisp at (verify-guards read-utf8-fast ...) when
+; running on CLISP 2.34 on Linux.  CLISP documentation at
+; http://clisp.cons.org/clisp.html#opt-memsize says that it is "common to
+; specify 10 MB" for the value of -m; since that suffices to eliminate the
+; stack overflow mentioned above, we use that value.  Note that we use ~dMB
+; instead of ~sMB because the with-standard-io-syntax wrapper in
 ; write-exec-file seems to put a decimal point after the number when using ~s,
 ; and CLISP complains about that when starting up.
 
@@ -1433,7 +1604,17 @@ implementations.")
                                      "export CCL_DEFAULT_DIRECTORY=~s"
                                      default-dir)
                            "")))
-                      "~s -I ~s -e \"(acl2::acl2-default-restart)\" $*~%"
+
+; It is probably important to use -e just below instead of :toplevel-function,
+; at least for #+hons.  Jared Davis and Sol Swords have told us that it seems
+; that with :toplevel-function one gets a new "toplevel" thread at start-up,
+; which "plays badly with the thread-local hash tables that make up the hons
+; space".
+
+; See the section on "reading characters from files" in file acl2.lisp for an
+; explanation of the -K argument below.
+
+                      "~s -I ~s -K ISO-8859-1 -e \"(acl2::acl2-default-restart)\" $*~%"
                       ccl-program
                       core-name))
     (chmod-executable sysout-name)
