@@ -1,23 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
-; (C) 1997 Computational Logic, Inc.  See the documentation topic
-; NOTE-2-0.
+; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -88,7 +81,7 @@
 ;     stream reaches eof; our other lisps all exit to the operating system.
 ;     This means all the batch jobs we submit via make have to be arranged to
 ;     exit from cmulisp before eof.  This required changing the top-level
-;     makefile and the makefiles of all the books.  I generally put a
+;     makefile and the makefiles of all the community books.  I generally put a
 ;     `:q #+cmu (lisp::quit)' at the end of each workxxx.
 ;     These could be replaced simply by `:q (acl2::exit-lisp)'.
 
@@ -122,7 +115,7 @@
 ;     them, catching errors and producing a HIDE term if we can't eval them,
 ;     we survived the certification of many books before we finally got to a
 ;     proof that couldn't be done without running one of those functions.
-;     The proof, in the bdd books, required evaling (nth -1 nil), which
+;     The proof, in the bdd community books, required evaling (nth -1 nil), which
 ;     according to the axioms is nil but which we couldn't deduce because
 ;     ACL2_*1*_COMMON-LISP::NTH was undefined.
 
@@ -178,8 +171,10 @@
 
 ; Consider using (safety 3) if there is a problem with LispWorks.  It enabled
 ; us to see a stack overflow involving collect-assumptions in the proof of
-; bitn-lam0 from books/rtl/rel2/support/lop3.lisp.  (See save-acl2-in-lispworks
-; for how we have eliminated that stack overflow.)
+; bitn-lam0 from community book books/rtl/rel2/support/lop3.lisp.  The safety
+; setting of 3 helped in this example, because, when the safety is set to 0,
+; the stack is not always automatically extended upon overflow (despite setting
+; system::*stack-overflow-behaviour* to nil in acl2-init.lisp).
 
 ; Safety 1 for CCL has avoided the kernel debugger, e.g. for compiled calls
 ; of car on a non-nil atom.  The following results for CCL show why we have
@@ -210,7 +205,8 @@
 ; 10013.573u 566.983s 33:33.80 525.4%   0+0k 0+0io 0pf+0w
 
 ; Safety 2:
-; [Note: books/clause-processors/SULFA/books/sat-tests/test-incremental.lisp
+; [Note: community book
+; books/clause-processors/SULFA/books/sat-tests/test-incremental.lisp
 ; ran out of space, saving perhaps a minute]
 ; 15637.669u 511.811s 52:02.78 517.1%   0+0k 0+0io 0pf+0w
 
@@ -230,6 +226,12 @@
                            0))
                      )))
 
+; Uncomment the following form if you want extra checks to be done during the
+; use of ACL2, the only effect (other than performance) being to cause errors
+; when those checks fail.
+
+; (push :acl2-extra-checks *features*)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                               FILES
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -245,19 +247,162 @@
 ; Common Lisp for which ACL2 is a verification system.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;                          LISP BUGS AND QUIRKS
+;                       READING CHARACTERS FROM FILES
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-; Patch from Gary Byers for OpenMCL 1.0 (CCL).  He says (4/13/07):
-;   Sometime over the last year or so, code was introduced into ACL2 that
-;   triggered a bug in OpenMCL 1.0 (namely, the initialization of binding
-;   of LOWER-BOUND in the THE-FIXNUM! macro in basis.lisp treats the expression
-;   (1+ UPPER-BOUND) as a fixnum (it isn't ...) and winds up negating the
-;   address of a bignum.
-#+(and ccl coral) ; the symbol :CORAL was on *FEATURES* in 1.0 and earlier
-(ccl:advise ccl::grovel-numeric-form
-            (let* ((ccl::*nx-form-type* t)) (:do-it))
-            :when :around)
+; Files may be viewed as sequences of bytes.  But Lisp can interpret these byte
+; sequences as sequences of characters, depending on so-called character
+; encodings.
+
+; For example, through about March 2012 the default character encoding in CCL
+; was ISO-8859-1, sometimes known as LATIN-1.  When this changed to UTF-8 a
+; failure was reported when attempting to certify community book
+; books/workshops/2006/cowles-gamboa-euclid/Euclid/fld-u-poly/coe-fld.lisp,
+; apparently because of the following line, where the next-to-last character is
+; actually an accented `o' sometimes known as
+; #\LATIN_SMALL_LETTER_O_WITH_ACUTE, having code 243.  (The CLISP build fails
+; if we use that character here, because at this point we have not yet made the
+; adjustments to the character encoding that are done below!)
+
+  ;;; Inverso de la primera operacion
+
+; The accented `o' above is encoded as a single byte in LATIN-1 but as two
+; bytes in UTF-8.
+
+; Jared Davis then suggested that we explicitly specify ISO-8859-1, which might
+; be slightly more efficient for reading files.  Note that GCL (non-Ansi, circa
+; 2010 but probably later) only supports ISO-8859-1 as far as we can tell.  We
+; follow Jared's suggestion below.  It seems that character encoding for
+; reading from files is controlled differently from character encoding for ACL2
+; reading from the terminal.  Jared also suggested setting the terminal
+; encoding to ISO-8859-1 as well, and showed us how to do that in CCL.  We have
+; been unable to figure out how to do that in some other host Lisps, but since
+; files are typically shared between users and (of course) ACL2 reading from
+; the terminal is not, the encoding for the terminal seems less important than
+; for files.
+
+; Even for files, we assert there is no soundness issue, in the sense that for
+; maximum trust we expect each user to certify all books from scratch in a
+; single environment.  But in practice, users share books (in particular,
+; via the community books); so it is desirable to enforce uniform character
+; encodings for files.
+
+; The use of latin-1 could in principle cause problems for those whose default
+; Emacs buffer encoding is utf-8.  That's in fact the case for us at UT CS, but
+; not on one of our Mac laptops as of this writing (April 2012), probably
+; because environment variable LANG is en_US.UTF-8 at UT CS.  But ACL2 users
+; don't often save Lisp files with nonstandard characters.  If they have
+; occasion to do so, they can evaluate
+
+; (setq save-buffer-coding-system 'iso-8859-1)
+
+; in Emacs buffers before saving into files.  This will happen automatically
+; for users who load file emacs/emacs-acl2.el into their emacs sessions.
+
+; At any rate, it seems best to standardize file encodings and leave it to
+; individuals to cope with editing issues.
+
+; Without further ado, we set the default encoding for files.  Below, we make
+; some attempt to do so for the terminal.  We wrap all this into a function,
+; because we have found that for sbcl, upon restart we lose the effect of the
+; assignment below.  It seems safest then to do these same assignments at
+; startup; see the call of the following function in acl2-default-restart.
+
+(defun acl2-set-character-encoding ()
+
+; We set the character encoding (see discussion above).
+
+  #+allegro
+; Alternatively, we could set the locale on the command line using -locale C:
+; see http://www.franz.com/support/documentation/6.1/doc/iacl.htm#locales-1
+; Note that (setq *default-external-format* :ISO-8859-1) is obsolete:
+; http://www.franz.com/support/documentation/6.1/doc/iacl.htm#older-ef-compatibility-2
+  (setq *locale* (find-locale "C"))
+
+  #+ccl
+  (setq ccl:*default-file-character-encoding* :iso-8859-1)
+
+; #+clisp
+; Set using -E ISO-8859-1 command-line option from save-exec.
+; Note that the setting below for custom:*default-file-encoding* works for
+; files, but we were unable to do the analogous thing successfully for
+; custom:*terminal-encoding*, even when restricting that assignment to
+; (interactive-stream-p *terminal-io*).
+
+  #+cmu
+  (setq *default-external-format* :iso-8859-1)
+
+; #+gcl -- already iso-8859-1, it seems, and nothing we can do to change
+;          the encoding anyhow
+
+  #+lispworks
+; This the default on our linux system, at least on both 32- and 64-bit,
+; version 6.1.0.
+  (setq stream::*default-external-format* '(:LATIN-1 :EOL-STYLE :LF))
+
+  #+sbcl
+  (setq sb-impl::*default-external-format* :iso-8859-1)
+
+; ;;;
+; We have made only limited attempts to set the character encoding at the
+; terminal, as discussed above.
+; ;;;
+
+; #+allegro
+; Handled by *locale* setting above.  Formerly the following was later in
+; this file; now we include it only as a comment.
+;  #+(and allegro allegro-version>= (version>= 6 0))
+;  (setf (stream-external-format *terminal-io*)
+;        (excl::find-external-format
+;         #+unix :latin1-base
+;         #-unix :latin1))
+
+; #+ccl
+; For terminal, set using -K ISO-8859-1 command-line option from save-exec.
+
+; #+cmucl -- Probably no setting is necessary.
+
+; #+clisp -- Set using command-line option (see above).
+
+; #+gcl -- There seems to be nothing we can do.
+
+; #+lispworks -- No support seems to be available.
+
+; #+sbcl
+; We found that "unsetenv LANG" results in (stream-external-format
+; *terminal-io*) being ascii at the terminal instead of utf-8; or, just start
+; up sbcl like this to get ascii:
+
+;   LANG=C ; XTERM_LOCALE=C ; sbcl
+
+; But we don't know how to get latin-1.  We even tried each of these, but got
+; ascii:
+
+;   LANG=en_US.LATIN-1 ; XTERM_LOCALE=en_US.LATIN-1
+;   LANG=en_US.ISO-8859-1 ; XTERM_LOCALE=en_US.ISO-8859-1
+
+  )
+
+; Here, we invoke the function defined above, so that a suitable character
+; encoding is used during the build, not only when starting up a built image
+; (which is why we call acl2-set-character-encoding in acl2-default-restart).
+(acl2-set-character-encoding)
+
+; We also do the following for clisp, since we set the encoding on the command
+; line (see comment above) but we want to be able to read our own source files
+; during the build.  See the comment in (defdoc character-encoding ...) in
+; axioms.lisp.
+#+clisp
+(setq custom:*default-file-encoding*
+      (ext:make-encoding :charset 'charset:iso-8859-1
+; The following is a bit of a guess, but seems consistent with what we used to
+; do in acl2-init.lisp; see the commented-out call there that sets
+; custom:*default-file-encoding*.
+                         :line-terminator :unix))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;                          LISP BUGS AND QUIRKS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; See acl2-fns.lisp for a fix to user-homedir-pathname for some versions of
 ; GCL.
@@ -286,13 +431,6 @@
               (function (lambda (x y)
                           (equal (symbol-name x) (symbol-name y))))))
 
-; Speed up I/O in Allegro 6 and beyond:
-#+(and allegro allegro-version>= (version>= 6 0))
-(setf (stream-external-format *terminal-io*)
-      (excl::find-external-format
-       #+unix :latin1-base
-       #-unix :latin1))
-
 ; Turn off automatic declaration of special variables, in particular since we
 ; do not want state declared special; see the comment above
 ; (eval '(setq state *the-live-state*)) in load-acl2.
@@ -322,6 +460,107 @@
 #+ccl
 (setq ccl::*record-source-file* nil)
 
+; The following avoids errors from extra right parentheses, but we leave it
+; commented out since it doesn't seem important enough to merit messing around
+; at this low level, and for just one Lisp.
+; #+ccl
+; (setq ccl::*ignore-extra-close-parenthesis* t)
+
+; We have tried to build under ECL (Embeddable Common-Lisp), and with some
+; modifications, we made progress -- except there appears (as of Sept. 2011) to
+; be no good way for us to save an executable image.  Specifically, it appears
+; that c:build-program not will suffice for saving state (properties etc.) --
+; it's just for saving specified .o files.
+
+; Here we document steps to take towards possibly porting to ECL in the future.
+
+; If state-global-let* expansion causes an error due to excessive code blow-up,
+; then consider replacing its definition by placing the following after
+; state-free-global-let*.  HOWEVER, first think about whether this is right; we
+; haven't thought through the effect of mixing a stack of let*-bindings of
+; state global variables with the acl2-unwind-protect mechanism.  Also,
+; comments are omitted here; be sure to restore them.
+
+;;; (defmacro state-global-let*-logical (bindings body)
+;;;   (declare (xargs :guard (and (state-global-let*-bindings-p bindings)
+;;;                               (no-duplicatesp-equal (strip-cars bindings)))))
+;;; 
+;;;   `(let ((state-global-let*-cleanup-lst
+;;;           (list ,@(state-global-let*-get-globals bindings))))
+;;;      ,@(and (null bindings)
+;;;             '((declare (ignore state-global-let*-cleanup-lst))))
+;;;      (acl2-unwind-protect
+;;;       "state-global-let*"
+;;;       (pprogn ,@(state-global-let*-put-globals bindings)
+;;;               (check-vars-not-free (state-global-let*-cleanup-lst) ,body))
+;;;       (pprogn
+;;;        ,@(state-global-let*-cleanup bindings 0)
+;;;        state)
+;;;       (pprogn
+;;;        ,@(state-global-let*-cleanup bindings 0)
+;;;        state))))
+;;; 
+;;; #-acl2-loop-only
+;;; (defmacro enforce-live-state-p (form)
+;;; 
+;;; ; Note that STATE is intended to be lexically bound at the point where this
+;;; ; macro is called.
+;;; 
+;;;   `(progn (when (not (live-state-p state)) ; (er hard! ...)
+;;;             (let ((*hard-error-returns-nilp* nil))
+;;;               (illegal 'enforce-live-state-p
+;;;                        "The state was expected to be live in the context of ~
+;;;                         an evaluation of the form:~|~x0"
+;;;                        (list (cons #\0 ',form)))))
+;;;           ,form))
+;;; 
+;;; (defmacro state-global-let* (bindings body)
+;;;   (cond #-acl2-loop-only
+;;;         ((and (symbol-doublet-listp bindings)
+;;;               (not (assoc-eq 'acl2-raw-mode-p bindings)))
+;;; 
+;;; ; The test above guarantees that we merely have bindings of state globals.  A
+;;; ; triple requires cleanup using a setter function.  Also we avoid giving this
+;;; ; simple treatment to 'acl2-raw-mode-p because the semantics of
+;;; ; state-global-let* are to call f-put-global, which has side effects in the
+;;; ; case of 'acl2-raw-mode-p.
+;;; 
+;;;          `(enforce-live-state-p
+;;;            (warn-about-parallelism-hazard
+;;;             '(state-global-let* ,bindings ,body)
+;;;             (state-free-global-let* ,bindings ,body))))
+;;;         (t `(state-global-let*-logical ,bindings ,body))))
+
+; Also, place the following forms in file acl2-fns.lisp, just above qfuncall
+; (but there may be better ways to do this).
+
+;;; ; The following is in acl2.lisp, but seems to be needed here as well.
+;;; #+ecl
+;;; (ext:package-lock "COMMON-LISP" nil)
+;;; 
+;;; Similarly in acl2.lisp, just before handling of package-lock on
+;;; "COMMON-LISP" for clisp:
+;;; 
+;;; #+ecl
+;;; (ext:package-lock "COMMON-LISP" nil)
+
+; Finally, consider these additional notes.
+
+;;; We need (require "cmp") if we're to use c:build-program.
+
+;;; Special-form-or-op-p: treat ecl like sbcl.
+
+;;; System-call: treat ecl like akcl (actually replace #+akcl by #+(or akcl
+;;; ecl)).
+
+;;; Initialize-state-globals: treat ecl just like lispworks.
+
+;;; Where we have the binding (compiler:*suppress-compiler-notes* t) for akcl,
+;;; perhaps include the binding (*compile-verbose* t) for ecl.
+
+;;; Modify exit-lisp to treat ecl like akcl, except using ext::quit instead of
+;;; lisp::bye.
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                              PACKAGES
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -410,10 +649,16 @@
       (make-package inv :use nil)))
 
 ; LispWorks has a package named "DEF", and that name conflicts with an ACL2
-; package of that name introduced in books/coi/.  So we rename it here.
+; package of that name introduced in community book books/coi/.  We deal with
+; that issue here.  Thanks to Martin Simmons for providing this code in place
+; of the original code, which instead invoked (rename-package "DEF"
+; "DEF-FROM-LW-RENAMED").
 #+lispworks
 (when (find-package "DEF")
-  (rename-package "DEF" "DEF-FROM-LW-RENAMED"))
+  (unless (equal (package-name "DEF") "DSPEC")
+    (error "Expected LispWorks DEF package to be called DSPEC"))
+  (rename-package "DSPEC" "DSPEC"
+                  (remove "DEF" (package-nicknames "DSPEC") :test 'equal)))
 
 ; The value of the constant *the-live-state* is actually just a
 ; symbol, but that symbol is the unique representative of the one
@@ -583,10 +828,11 @@
 
   '(
     #+acl2-par "multi-threading-raw"
+    #+hons "serialize-raw"
     "axioms"
-    "memoize" ; but only get special under-the-hood treatment with #+hons
-    "hons"    ; but only get special under-the-hood treatment with #+hons
-    "boot-strap-pass-2"
+    "memoize"   ; but only get special under-the-hood treatment with #+hons
+    "hons"      ; but only get special under-the-hood treatment with #+hons
+    "serialize" ; but only get special under-the-hood treatment with #+hons
     "basis"
     "parallel" ; but only get special under-the-hood treatment with #+acl2-par
     #+acl2-par "futures-raw"
@@ -599,6 +845,7 @@
     "type-set-b"
     "linear-b"
     "non-linear"
+    "tau"
     "rewrite"
     "simplify"
     "bdd"
@@ -616,6 +863,7 @@
     "tutorial"
     "interface-raw"
     "defpkgs"
+    "boot-strap-pass-2" ; at the end so that it is compiled last
     )
   "*acl2-files* is the list of all the files necessary to build
 ACL2 from scratch.")
@@ -671,7 +919,7 @@ ACL2 from scratch.")
    (setq acl2::*copy-of-acl2-version*
 ;  Keep this in sync with the value of acl2-version in *initial-global-table*.
          (concatenate 'string
-                      "ACL2 Version 4.3"
+                      "ACL2 Version 6.0"
                       #+non-standard-analysis
                       "(r)"
                       #+(and mcl (not ccl))
@@ -684,6 +932,16 @@ ACL2 from scratch.")
 
 (in-package "ACL2")
 
+(defparameter *compiled-file-extension*
+
+; Note that for the community books, files books/Makefile-generic,
+; books/Makefile-subdirs, and books/Makefile-psubdirs all deal with compiled
+; file extensions.  Thanks to Gary Byers for suggested the following approach
+; for #+ansi-cl, which however appears to work for all Lisps supported as of
+; early 2006.
+
+  (pathname-type (compile-file-pathname "foo.lisp")))
+
 (defmacro initialize-state-globals ()
   (let ((acl2-compiler-enabled-var
          #+cltl2 'common-lisp-user::*acl2-compiler-enabled*
@@ -708,7 +966,11 @@ ACL2 from scratch.")
                                             ,acl2-compiler-enabled-var))
                                  ,acl2-compiler-enabled-var)
                                 (t
-                                 #+(or ccl sbcl) ; these compile on the fly
+
+; Warning: Keep the following "compile on the fly" readtime conditional in sync
+; with the one in acl2-compile-file.
+
+                                 #+(or ccl sbcl)
                                  :books
                                  #-(or ccl sbcl)
                                  t))
@@ -725,9 +987,14 @@ ACL2 from scratch.")
                 #+lispworks :lispworks
                 #-(or gcl ccl sbcl allegro clisp cmu lispworks)
                 (error
-                 "The underlying host Lisp appears not to support ACL2. ~%~
+                 "Error detected in initialize-state-globals: ~%~
+                  The underlying host Lisp appears not to support ACL2. ~%~
                   Contact the ACL2 implementors to request such support."))
              *the-live-state*)
+            (f-put-global
+             'compiled-file-extension
+             ,*compiled-file-extension*
+             *the-live-state*)
             #+unix
             (f-put-global 'tmp-dir "/tmp" *the-live-state*)
             #-acl2-mv-as-values
@@ -843,6 +1110,42 @@ ACL2 from scratch.")
 ; instead of the "LISP BUGS AND QUIRKS" section because we want to define this
 ; in the ACL2 package.
 
+(defmacro with-redefinition-suppressed (&rest forms)
+
+; This macro is equivalent to progn for SBCL and CMUCL, where we typically
+; suppress more warnings anyhow; see with-warnings-suppressed.
+
+  `(let (#+lispworks
+         (compiler::*redefinition-action* :QUIET)
+         #+allegro
+         (excl:*redefinition-warnings* nil)
+         #+clisp
+         (custom::*suppress-check-redefinition*
+
+; This binding may not suffice during the build.  Below is a comment that is
+; probably old as of this writing (Oct. 9, 2011).
+
+; Unfortunately, the above binding seems to be ignored when in the scope of
+; with-compilation-unit, so we get redefinition warnings from clisp during the
+; build.  Here is an example that exhibits that behavior CLISP 2.44.1 (even
+; without ACL2); and the same problem occurs if we switch the order of
+; with-compilation-unit and the binding of *suppress-check-redefinition*.
+
+;   (common-lisp::with-compilation-unit
+;    ()
+;    (let ((custom::*suppress-check-redefinition* t))
+;      (dolist (name '("foo" "bar"))
+;        (let ((source (make-pathname :name name :type "lisp")))
+;          (load source)
+;          (progn
+;            (compile-file source)
+;            (load (make-pathname :name name :type "fas")))))))
+
+          t)
+         #+ccl
+         (ccl::*compiler-warn-on-duplicate-definitions* nil))
+     ,@forms))
+
 (defmacro with-warnings-suppressed (&rest forms)
 
 ; See also the handler-bind call in init.lisp.  Thanks to Juho Snellman for his
@@ -887,39 +1190,8 @@ ACL2 from scratch.")
                        (invoke-restart 'muffle-warning))))
            ,@forms))
 
-  #+lispworks
-  `(let ((compiler::*redefinition-action* :QUIET))
-     ,@forms)
-
-  #+allegro
-  `(let ((excl:*redefinition-warnings* nil)) ,@forms)
-
-  #+clisp
-  `(let ((custom::*suppress-check-redefinition* t))
-
-; Unfortunately, the above binding seems to be ignored when in the scope of
-; with-compilation-unity, so we get redefinition warnings from clisp during the
-; build.  Here is an example that exhibits that behavior CLISP 2.44.1 (even
-; without ACL2); and the same problem occurs if we switch the order of
-; with-compilation-unit and the binding of *suppress-check-redefinition*.
-
-;   (common-lisp::with-compilation-unit
-;    ()
-;    (let ((custom::*suppress-check-redefinition* t))
-;      (dolist (name '("foo" "bar"))
-;        (let ((source (make-pathname :name name :type "lisp")))
-;          (load source)
-;          (progn
-;            (compile-file source)
-;            (load (make-pathname :name name :type "fas")))))))
-
-     ,@forms)
-
-  #+ccl
-  `(let ((ccl::*compiler-warn-on-duplicate-definitions* nil)) ,@forms)
-
-  #-(or cmucl sbcl lispworks allegro clisp ccl)
-  (if (cdr forms) `(progn ,@forms) (car forms)))
+  #-(or sbcl cmu)
+  `(with-redefinition-suppressed ,@forms))
 
 (defmacro with-more-warnings-suppressed (&rest forms)
 
@@ -1024,22 +1296,21 @@ ACL2 from scratch.")
 
 (defvar *lisp-extension* "lisp")
 
-(defparameter *compiled-file-extension*
-
-; Note that books/Makefile-generic, books/Makefile-subdirs, and
-; books/Makefile-psubdirs all deal with compiled file extensions.
-; Thanks to Gary Byers for suggested the following approach for #+ansi-cl,
-; which however appears to work for all Lisps supported as of early 2006.
-
-  (pathname-type (compile-file-pathname "foo.lisp")))
-
 #+sbcl ; turn off compiler notes (noisy)
 (declaim (sb-ext:muffle-conditions sb-ext:compiler-note))
 
+; The Allegro CL documentation tells us that the following variable is
+; "Initially true only if speed is 3 and safety is 0", and that it allows the
+; compiler to assume that the sum of two declared fixnums is a fixnum, etc.  We
+; hope that by setting this variable to nil we avoid seeing occasional checksum
+; errors during include-book when certifying the regression suite.
 #+allegro
-(eval (read-from-string "
-  (SETQ COMPILER::DECLARED-FIXNUMS-REMAIN-FIXNUMS-SWITCH
-        #'(LAMBDA (X Y Z #+(VERSION>= 4 1) D) NIL)) "))
+(setf comp:declared-fixnums-remain-fixnums-switch nil)
+
+; Previous value for the above:
+; (eval (read-from-string "
+;   (SETQ COMPILER::DECLARED-FIXNUMS-REMAIN-FIXNUMS-SWITCH
+;         #'(LAMBDA (X Y Z #+(VERSION>= 4 1) D) NIL)) "))
 
 ; The following appears to allow tail recursion elimination for functions
 ; locally defined using LABELS.  This is important for efficiency since we
@@ -1158,9 +1429,35 @@ which is saved just in case it's needed later.")
    #\!
    #'sharp-bang-read))
 
+(defun define-sharp-u ()
+  (set-dispatch-macro-character
+   #\#
+   #\u
+   #'sharp-u-read))
+
+(defvar *old-character-reader*
+  (get-dispatch-macro-character #\# #\\))
+
 (defun modify-acl2-readtable (do-all-changes)
   (let ((*readtable* *acl2-readtable*))
 
+; Thanks to Jared Davis for contributing the code for #\Y and #\Z (see
+; serialize-raw.lisp).  Note that p. 531 of CLtL2 specifies that #\Y and #\Z
+; are available to us.  Keep these two settings in sync with
+; *reckless-acl2-readtable*.
+
+    #+hons ; SBCL requires #+hons (same restriction as ser-hons-reader-macro)
+    (set-dispatch-macro-character
+     #\#
+     #\Z
+     'ser-hons-reader-macro)
+
+    #+hons ; SBCL requires #+hons (same restriction as ser-cons-reader-macro)
+    (set-dispatch-macro-character
+     #\#
+     #\Y
+     'ser-cons-reader-macro)
+
 ; Backquote
 
     (set-macro-character
@@ -1192,7 +1489,8 @@ which is saved just in case it's needed later.")
       (define-sharp-dot)
       (define-sharp-comma)
       (define-sharp-atsign)
-      (define-sharp-bang))
+      (define-sharp-bang)
+      (define-sharp-u))
 
 ;  Keep control of character reader.  However, we do not need to keep such
 ;  control when reading in a .fas file for CLISP, and in fact, the set-theory
@@ -1222,6 +1520,7 @@ which is saved just in case it's needed later.")
           (define-sharp-comma)
           (define-sharp-atsign)
           (define-sharp-bang)
+          (define-sharp-u)
           (set-dispatch-macro-character
            #\#
            #\\
@@ -1238,6 +1537,34 @@ which is saved just in case it's needed later.")
     (set-dispatch-macro-character #\#
                                   #\=
                                   #'reckless-sharp-equal-read)
+    (set-dispatch-macro-character
+     #\#
+     #\\
+     *old-character-reader*)
+
+; Thanks to Jared Davis for contributing the code for #\Z and #\z (see
+; serialize-raw.lisp).  But is this readtable even necessary now that
+; compact-printing has been replaced by the serialize reader?  Quite possibly
+; it is indeed needed for #-hons, for (as of Version_4.3) the binding of
+; *readtable* to *reckless-acl2-readtable* in a call of compile-file in
+; acl2-compile-file, and for the calls of with-reckless-read in
+; load-compiled-book and include-book-raw.  But we could quite possibly omit
+; the #+hons code below.  At any rate, it seems harmless enough to continue to
+; use this function and to include the #+hons below.  Keep these two settings
+; in sync with modify-acl2-readtable.
+
+    #+hons ; SBCL requires #+hons (same restriction as ser-hons-reader-macro)
+    (set-dispatch-macro-character
+     #\#
+     #\Z
+     'ser-hons-reader-macro)
+
+    #+hons ; SBCL requires #+hons (same restriction as ser-cons-reader-macro)
+    (set-dispatch-macro-character
+     #\#
+     #\Y
+     'ser-cons-reader-macro)
+
     *readtable*))
 
 (defvar *load-compiled-verbose* nil)
@@ -1753,6 +2080,19 @@ which is saved just in case it's needed later.")
 
 (defun exit-lisp (&optional (status '0 status-p))
 
+; Parallelism blemish: In ACL2(p), LispWorks 6.0.1 hasn't always successfully
+; exited when exit-lisp was called.  The call below of stop-multiprocessing is
+; an attempt to improve the chance of a succesful exit.  In practice, the call
+; does not fix the problem.  However, we leave it for now since we don't think
+; it can hurt.  If exit-lisp starts working reliably without the following
+; calls to send-die-to-worker-threads and stop-multiprocessing, they should be
+; removed.
+
+  #+(and acl2-par lispworks)
+  (when mp::*multiprocessing*
+    (send-die-to-worker-threads)
+    (mp::stop-multiprocessing))
+
 ; The status (an integer) will be returned as the exit status (shell variable
 ; $?).  We avoid passing the status argument when it is 0, in case Windows or
 ; other operating systems get confused by it.  However, this function is a
@@ -1779,7 +2119,7 @@ which is saved just in case it's needed later.")
   #+clisp
   (if status-p (user::exit status) (user::exit))
   #+lispworks ; Version 4.2.0; older versions have used bye
-  (if status-p (lisp::quit :status status) (lisp::quit))
+  (if status-p (lispworks:quit :status status) (lispworks:quit))
   #+akcl
   (if status-p (lisp::bye status) (lisp::bye))
   #+lucid
@@ -1918,7 +2258,7 @@ which is saved just in case it's needed later.")
 
 ; The following constant was originally in translate.lisp, but CLISP warned
 ; that it was being redefined.  This seems to be the same problem as mentioned
-; above for Allegro above, so we define it here.
+; above for Allegro, so we define it here.
 
 (defconstant *big-n-special-object* '(nil . nil))
 
@@ -1959,6 +2299,39 @@ which is saved just in case it's needed later.")
       (incf i))
     ar))
 
+; The following constants were originally in memoize-raw.lisp, but CMUCL caused
+; a redefinition error.  This may be the same problem as mentioned above for
+; Allegro.
+#+hons
+(progn
+; locals used in memoize-on and memoize-off
+
+(defconstant *mo-f* (make-symbol "F"))
+(defconstant *mo-h* (make-symbol "H"))
+(defconstant *mo-o* (make-symbol "O"))
+
+
+; locals used in functions generated by memoize-fn
+
+(defconstant *mf-old-caller* (make-symbol "OLD-CALLER"))
+(defconstant *mf-start-hons* (make-symbol "START-HONS"))
+(defconstant *mf-start-pons* (make-symbol "START-PONS"))
+(defconstant *mf-start-bytes* (make-symbol "START-BYTES"))
+(defconstant *mf-ans* (make-symbol "ANS"))
+(defconstant *mf-ans-p* (make-symbol "ANS-P"))
+(defconstant *mf-ma* (make-symbol "MA"))
+(defconstant *mf-args* (make-symbol "ARGS"))
+(defconstant *mf-2mmf* (make-symbol "MF-2MMF"))
+(defconstant *mf-2mmf-fnn* (make-symbol "MF-2MMF-FNN"))
+(defconstant *mf-count-loc* (make-symbol "MF-COUNT-LOC"))
+
+(defconstant *mf-cl-error-msg*
+  "~%; Redefining a function in the COMMON-LISP package ~
+   is forbidden.")
+
+(defconstant *attached-fn-temp* (make-symbol "ATTACHED-FN-TEMP"))
+)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            STATISTICS
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
