@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -84,6 +78,9 @@
 ;;; Attachment: too-many-ifs-post-rewrite and too-many-ifs-pre-rewrite
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+#+acl2-loop-only
+; The above readtime conditional avoids a CLISP warning, and lets the defproxy
+; for print-clause-id-okp provide the raw Lisp definition.
 (encapsulate
  ((too-many-ifs-post-rewrite (args val) t
                              :guard (and (pseudo-term-listp args)
@@ -91,15 +88,15 @@
  (local (defun too-many-ifs-post-rewrite (args val)
           (list args val))))
 
-; The following events are derived from the original version of
+; The following events are derived from the original version of community book
 ; books/system/too-many-ifs.lisp.  But here we provide a proof that does not
 ; depend on books.  Our approach was to take the proof in the above book,
 ; eliminate the unnecessary use of an arithmetic book, expand away all uses of
 ; macros and make-events, avoid use of (theory 'minimal-theory) since that
 ; theory didn't yet exist (where these events were originally placed), and
 ; apply some additional hand-editing in order (for example) to remove hints
-; depending on the tools/flag book.  We have left original events from the book
-; as comments.
+; depending on the tools/flag community book.  We have left original events
+; from the book as comments.
 
 (encapsulate
  ()
@@ -482,6 +479,9 @@
 
 ; Complete too-many-ifs-pre-rewrite.
 
+#+acl2-loop-only
+; The above readtime conditional avoids a CLISP warning, and lets the defproxy
+; for print-clause-id-okp provide the raw Lisp definition.
 (encapsulate
   ((too-many-ifs-pre-rewrite (args counts) t
                              :guard
@@ -615,6 +615,9 @@
                                        (fn-count-1 nil (cadr lit) 0
                                                    0)))))))))
 
+#+acl2-loop-only
+; The above readtime conditional avoids a CLISP warning, and lets the defproxy
+; for print-clause-id-okp provide the raw Lisp definition.
 (encapsulate
  ((ancestors-check (lit ancestors tokens) (mv t t)
                    :guard (and (pseudo-termp lit)
@@ -651,10 +654,12 @@
 
 (verify-termination-boot-strap hons-copy-with-state) ; and guards
 
+(verify-termination-boot-strap identity-with-state) ; and guards
+
 (defattach (acl2x-expansion-alist
 ; User-modifiable; see comment in the defstub introducing
 ; acl2x-expansion-alist.
-            hons-copy-with-state))
+            identity-with-state))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Attachments: rw-cache utilities
@@ -702,6 +707,9 @@
 
 (verify-termination-boot-strap clause-id-p) ; and guards
 
+#+acl2-loop-only
+; The above readtime conditional avoids a CLISP warning, and lets the defproxy
+; for print-clause-id-okp provide the raw Lisp definition.
 (encapsulate
  (((print-clause-id-okp *) => * :formals (cl-id) :guard (clause-id-p cl-id)))
  (local (defun print-clause-id-okp (x)
@@ -715,6 +723,9 @@
 ;;; Attachments: oncep-tp
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+#+acl2-loop-only
+; The above readtime conditional avoids a CLISP warning, and lets the defproxy
+; for print-clause-id-okp provide the raw Lisp definition.
 (encapsulate
  (((oncep-tp * *) => *
    :formals (rune wrld)
@@ -735,52 +746,126 @@
 (defattach oncep-tp oncep-tp-builtin)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Attachments: waterfall-parallelism and waterfall-printin
+; verify-termination and guard verification:
+; string-for-tilde-@-clause-id-phrase and some subsidiary functions
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(encapsulate
- ((waterfall-parallelism () t))
- (local (defun waterfall-parallelism () nil))
- (defthm waterfall-parallelism-constraint
-   (member-eq (waterfall-parallelism)
-              *waterfall-parallelism-values*)))
+; David Rager proved termination and guards for
+; string-for-tilde-@-clause-id-phrase, with a proof that included community
+; books unicode/explode-atom and unicode/explode-nonnegative-integer.  Here, we
+; rework that proof a bit to avoid those dependencies.  Note that this proof
+; depends on d-pos-listp, whose termination and guard verification are
+; performed above.
+
+; We proved true-listp-explode-nonnegative-integer here, but then found it was
+; already proved locally in axioms.lisp.  So we made that defthm non-local (and
+; strengthened it to its current form).
+
+(verify-termination-boot-strap chars-for-tilde-@-clause-id-phrase/periods)
+
+(verify-termination-boot-strap chars-for-tilde-@-clause-id-phrase/primes)
+
+(defthm pos-listp-forward-to-integer-listp
+  (implies (pos-listp x)
+           (integer-listp x))
+  :rule-classes :forward-chaining)
+
+(verify-termination-boot-strap chars-for-tilde-@-clause-id-phrase)
+
+(defthm true-listp-chars-for-tilde-@-clause-id-phrase/periods
+  (true-listp (chars-for-tilde-@-clause-id-phrase/periods lst))
+  :rule-classes :type-prescription)
+
+(defthm true-listp-explode-atom
+  (true-listp (explode-atom n print-base))
+  :rule-classes :type-prescription)
 
 (encapsulate
- ((waterfall-printing () t))
- (logic)
- (local (defun waterfall-printing () :full))
- (defthm waterfall-printing-constraint
-   (member-eq (waterfall-printing)
-              *waterfall-printing-values*)))
+ ()
 
-(defun make-symbol-constants-fn (prefix lst)
-  (declare (xargs :guard (and (symbolp prefix)
-                              (symbol-listp lst))))
-  (cond ((endp lst) nil)
-        (t (cons `(defun ,(symbol-constant-fn prefix (car lst)) ()
-                    (declare (xargs :guard t :mode :logic))
-                    ,(car lst))
-                 (make-symbol-constants-fn prefix (cdr lst))))))
+; The following local events create perfectly good rewrite rules, but we avoid
+; the possibility of namespace clashes for existing books by making them local
+; as we add them after Version_4.3.
 
-(defmacro make-waterfall-parallelism-constants (prefix)
-  (cons 'progn (make-symbol-constants-fn prefix
-                                         *waterfall-parallelism-values*)))
+ (local
+  (defthm character-listp-explode-nonnegative-integer
+    (implies
+     (character-listp ans)
+     (character-listp (explode-nonnegative-integer n print-base ans)))))
 
-(make-waterfall-parallelism-constants waterfall-parallelism)
+ (local
+  (defthm character-listp-explode-atom
+    (character-listp (explode-atom n print-base))
+    :hints ; need to disable this local lemma from axioms.lisp
+    (("Goal" :in-theory (disable character-listp-cdr)))))
 
-(defmacro make-waterfall-printing-constants (prefix)
-  (cons 'progn (make-symbol-constants-fn prefix
-                                         *waterfall-printing-values*)))
+ (local
+  (defthm character-listp-chars-for-tilde-@-clause-id-phrase/periods
+    (character-listp (chars-for-tilde-@-clause-id-phrase/periods lst))))
 
-(make-waterfall-printing-constants waterfall-printing)
+ (verify-termination-boot-strap string-for-tilde-@-clause-id-phrase))
 
-; We execute defattach events without :skip-checks.  But we avoid using
-; set-waterfall-parallelism, because that causes an invocation of
-; set-waterfall-printing, which is an error when #-acl2-par.
-#+acl2-loop-only
-(progn
-  (defattach (waterfall-parallelism waterfall-parallelism-nil))
-  (defattach (waterfall-printing waterfall-printing-full)))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+; verify-termination and guard verification:
+; strict-merge-symbol-<, strict-merge-sort-symbol-<, strict-symbol-<-sortedp,
+; and sort-symbol-listp
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(verify-termination-boot-strap strict-merge-symbol-<
+                               (declare (xargs :measure
+                                               (+ (len l1) (len l2)))))
+
+(encapsulate
+ ()
+
+ (local
+  (defthm len-strict-merge-symbol-<
+    (<= (len (strict-merge-symbol-< l1 l2 acc))
+        (+ (len l1) (len l2) (len acc)))
+    :rule-classes :linear))
+
+ (local
+  (defthm len-evens
+    (equal (len l)
+           (+ (len (evens l))
+              (len (odds l))))
+    :rule-classes :linear))
+
+ (local
+  (defthm symbol-listp-evens
+    (implies (symbol-listp x)
+             (symbol-listp (evens x)))
+    :hints (("Goal" :induct (evens x)))))
+
+ (local
+  (defthm symbol-listp-odds
+    (implies (symbol-listp x)
+             (symbol-listp (odds x)))))
+
+ (local
+  (defthm symbol-listp-strict-merge-symbol-<
+    (implies (and (symbol-listp l1)
+                  (symbol-listp l2)
+                  (symbol-listp acc))
+             (symbol-listp (strict-merge-symbol-< l1 l2 acc)))))
+
+ (verify-termination-boot-strap strict-merge-sort-symbol-<
+                                (declare (xargs :measure (len l)
+                                                :verify-guards nil)))
+
+ (defthm symbol-listp-strict-merge-sort-symbol-<
+; This lemma is non-local because it is needed for "make proofs", for
+; guard-verification for new-verify-guards-fns1.
+   (implies (symbol-listp x)
+            (symbol-listp (strict-merge-sort-symbol-< x))))
+
+ (verify-guards strict-merge-sort-symbol-<)
+
+ (verify-termination-boot-strap strict-symbol-<-sortedp) ; and guards
+
+ (verify-termination-boot-strap sort-symbol-listp) ; and guards
+
+ )
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Theories
@@ -839,6 +924,15 @@
 
   :cited-by theory-functions")
 
+; See the Essay on the Status of the Tau System During and After Bootstrapping
+; in axioms.lisp where we discuss choices (1.a), (1.b), (2.a) and (2.b)
+; related to the status of the tau system.  Here is where we implement
+; (2.a).
+
+(in-theory (if (cadr *tau-status-boot-strap-settings*)          ; (2.a)
+               (enable (:executable-counterpart tau-system))
+               (disable (:executable-counterpart tau-system))))
+
 (deftheory ground-zero (current-theory :here)
 
 ; WARNING: Keep this near the end of *acl2-pass-2-files* in order for
@@ -863,6 +957,378 @@
 
   :cited-by theory-functions")
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; meta-extract support
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(verify-termination-boot-strap formals) ; and guards
+(verify-termination-boot-strap constraint-info) ; and guards
+
+(defund meta-extract-formula (name state)
+
+; This function supports meta-extract-global-fact.  It needs to be executable
+; and in :logic mode (hence, as required by the ACL2 build process,
+; guard-verified), since it may be called by meta functions.
+
+; While this function can be viewed as a version of formula, it applies only to
+; symbols (not runes), it is in :logic mode, and there are a few other
+; differences as well.  The present function requires name to be a symbol and
+; only returns a normalp=nil form of body.  (Otherwise, in order to put body in
+; :logic mode, body would need to be guard-verified, which would probably take
+; considerable effort.)
+
+  (declare (xargs :stobjs state
+                  :guard (symbolp name)))
+  (let ((wrld (w state)))
+    (or (getprop name 'theorem nil 'current-acl2-world wrld)
+        (mv-let (flg prop)
+                (constraint-info name wrld)
+                (cond ((eq prop *unknown-constraints*)
+                       *t*)
+                      (flg (ec-call (conjoin prop)))
+                      (t prop))))))
+
+(verify-termination-boot-strap type-set-quote)
+(verify-guards type-set-quote)
+
+(defun typespec-check (ts x)
+  (declare (xargs :guard (integerp ts)))
+  (if (bad-atom x)
+      (< ts 0) ; bad-atom type intersects every complement type
+
+; We would like to write
+;   (ts-subsetp (type-set-quote x) ts)
+; here, but for that we need a stronger guard than (integerp ts), and we prefer
+; to keep this simple.
+
+    (not (eql 0 (logand (type-set-quote x) ts)))))
+
+(defun meta-extract-rw+-term (term alist equiv rhs state)
+
+; This function supports the function meta-extract-contextual-fact.  Neither of
+; these functions is intended to be executed.
+
+; Meta-extract-rw+-term creates (logically) a term claiming that term under
+; alist is equiv to rhs, where equiv=nil represents 'equal and equiv=t
+; represents 'iff.  If equiv is not t, nil, or an equivalence relation, then
+; *t* is returned.
+
+; Note that this function does not support the use of a geneqv for the equiv
+; argument.
+
+  (declare (xargs :mode :program ; becomes :logic with system-verify-guards
+                  :stobjs state
+                  :guard (and (symbol-alistp alist)
+                              (pseudo-term-listp (strip-cdrs alist))
+                              (pseudo-termp term))))
+  (non-exec
+   (let ((lhs (sublis-var alist term)))
+     (case equiv
+       ((nil) `(equal ,lhs ,rhs))
+       ((t)   `(iff ,lhs ,rhs))
+       (otherwise
+        (if (symbolp equiv)
+            (if (equivalence-relationp equiv (w state))
+                `(,equiv ,lhs ,rhs)
+; else bad equivalence relation
+              *t*)
+          *t*))))))
+
+(defun meta-extract-contextual-fact (obj mfc state)
+
+; This function is not intended to be executed.
+
+; This function may be evaled in the hypothesis of a meta rule, because we know
+; it always produces a term that evaluates to non-nil under the mfc where the
+; metafunction is called, using the specific alist A for which we're proving
+; (evl x a) = (evl (metafn x) a).  The terms it produces reflect the
+; correctness of certain prover operations -- currently, accessing type-alist
+; and typeset information, rewriting, and linear arithmetic.
+
+; This function avoids forcing and does not return a tag-tree.
+
+  (declare (xargs :mode :program ; becomes :logic with system-verify-guards
+                  :stobjs state))
+  (non-exec
+   (case-match obj
+     ((':typeset term . &) ; mfc-ts produces correct results
+      `(typespec-check
+        ',(mfc-ts term mfc state :forcep nil :ttreep nil)
+        ,term))
+     ((':rw+ term alist obj equiv . &) ; MFC-RW+ result is equiv to term/alist.
+      (meta-extract-rw+-term term alist equiv
+                             (mfc-rw+ term alist obj equiv mfc state
+                                      :forcep nil :ttreep nil)
+                             state))
+     ((':rw term obj equiv . &) ; as for :rw+, with alist of nil
+      (meta-extract-rw+-term term nil equiv
+                             (mfc-rw term obj equiv mfc state
+                                     :forcep nil :ttreep nil)
+                             state))
+     ((':ap term . &) ; Can linear arithmetic can falsify term?
+      (if (mfc-ap term mfc state
+                  :forcep nil :ttreep nil)
+          `(not ,term)
+        *t*))
+     ((':relieve-hyp hyp alist rune target bkptr . &) ; hyp/alist proved?
+      (if (mfc-relieve-hyp hyp alist rune target bkptr mfc state
+                           :forcep nil :ttreep nil)
+          (sublis-var alist hyp)
+        *t*))
+     (& *t*))))
+
+(defun rewrite-rule-term (x)
+
+; This function is not intended to be executed.  It turns a rewrite-rule record
+; into a term.
+
+  (declare (xargs :guard t))
+  (non-exec
+   (if (eq (access rewrite-rule x :subclass) 'meta)
+       *t*
+     `(implies ,(conjoin (access rewrite-rule x :hyps))
+               (,(access rewrite-rule x :equiv)
+                ,(access rewrite-rule x :lhs)
+                ,(access rewrite-rule x :rhs))))))
+
+(defun meta-extract-global-fact (obj state)
+
+; This function is not intended to be executed.
+
+; This function may be evaled in the hypothesis of a meta rule, because we know
+; it always produces a term that evaluates to non-nil for any alist.  The terms
+; it produces reflect the correctness of certain facts stored in the world.
+
+  (declare (xargs :mode :program ; becomes :logic with system-verify-guards
+                  :stobjs state))
+  (non-exec
+   (case-match obj
+     ((':formula name)
+      (meta-extract-formula name state))
+     ((':lemma fn n)
+      (let* ((lemmas (getprop fn 'lemmas nil 'current-acl2-world (w state)))
+             (rule (nth n lemmas)))
+        ;; This assumes that the LEMMAS property of any symbol in the ACL2 world
+        ;; is a list of rewrite-rule records which reflect known facts.
+        (if (< (nfix n) (len lemmas))
+            (rewrite-rule-term rule)
+          *t*))) ;; fn doesn't exist or n too big
+     (& *t*))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Support for system-verify-guards
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+; This section supports a mechanism for users to extend the set of
+; guard-verified functions.  They do so in community books under books/system/,
+; which are checked when building with feature :acl2-devel.  But normal builds
+; will not set that feature, and will simply trust that functions marked in
+; *system-verify-guards-alist* can be guard-verified.
+
+; A flaw in our approach is that user-supplied guard verifications may depend
+; on package axioms.  Thus, we view such verifications as strong hints, rather
+; than as ironclad guarantees that the functions can be guard-verified in
+; definitional (or even conservative) extensions of the ground-zero theory.  We
+; consider this sufficient, as the event that some package axiom will cause
+; such bogus marking as guard-verified seems much less likely than the event
+; that our system has other serious bugs!
+
+(verify-termination-boot-strap safe-access-command-tuple-form) ; and guards
+
+(defun pair-fns-with-measured-subsets (fns wrld acc)
+  (declare (xargs :guard (and (symbol-listp fns)
+                              (plist-worldp wrld)
+                              (true-listp acc))))
+  (cond ((endp fns) (reverse acc))
+        (t (pair-fns-with-measured-subsets
+            (cdr fns)
+            wrld
+            (cons (let* ((fn (car fns))
+                         (justification (getprop fn 'justification nil
+                                                 'current-acl2-world wrld))
+                         (ms (and (consp justification) ; for guard
+                                  (consp (cdr justification)) ; for guard
+                                  (access justification justification :subset))))
+                    (cons fn ms))
+                  acc)))))
+
+(defun new-verify-guards-fns1 (wrld installed-wrld acc)
+  (declare (xargs :guard (and (plist-worldp wrld)
+                              (plist-worldp installed-wrld)
+                              (symbol-listp acc))))
+  (cond ((or (endp wrld)
+             (and (eq (caar wrld) 'command-landmark)
+                  (eq (cadar wrld) 'global-value)
+                  (equal (safe-access-command-tuple-form (cddar wrld))
+                         '(exit-boot-strap-mode))))
+         (pair-fns-with-measured-subsets
+          (strict-merge-sort-symbol-< acc)
+          installed-wrld
+          nil))
+        ((and (eq (cadar wrld) 'symbol-class)
+              (eq (cddar wrld) :COMMON-LISP-COMPLIANT)
+              (getprop (caar wrld) 'predefined nil 'current-acl2-world
+                       installed-wrld))
+         (new-verify-guards-fns1 (cdr wrld)
+                                 installed-wrld
+                                 (cons (caar wrld) acc)))
+        (t (new-verify-guards-fns1 (cdr wrld) installed-wrld acc))))
+
+(defun new-verify-guards-fns (state)
+
+; It is important for performance that this function be guard-verified, because
+; it is called inside an assert-event form in chk-new-verified-guards, which
+; causes evaluation to be in safe-mode and would cause evaluation of
+; plist-worldp on behalf of guard-checking for new-verify-guards-fns1.
+
+  (declare (xargs :stobjs state))
+  (let ((wrld (w state)))
+    (new-verify-guards-fns1 wrld wrld nil)))
+
+(defconst *system-verify-guards-alist*
+
+; Each cdr was produced by evaluating
+; (new-verify-guards-fns (w state) (w state) nil)
+; after including the book indicated in the car, e.g.,
+; (include-book "system/top" :dir :system).
+; Each entry is of the form (fn . measured-subset).
+
+  '(("system/top"
+     (CONS-TERM)
+     (CONS-TERM1)
+     (CONS-TERM1-MV2)
+     (DUMB-NEGATE-LIT)
+     (FETCH-DCL-FIELD)
+     (FETCH-DCL-FIELDS LST)
+     (FETCH-DCL-FIELDS1 LST)
+     (FETCH-DCL-FIELDS2 KWD-LIST)
+     (META-EXTRACT-CONTEXTUAL-FACT)
+     (META-EXTRACT-GLOBAL-FACT)
+     (META-EXTRACT-RW+-TERM)
+     (MISSING-FMT-ALIST-CHARS)
+     (MISSING-FMT-ALIST-CHARS1 CHAR-TO-TILDE-S-STRING-ALIST)
+     (PLAUSIBLE-DCLSP LST)
+     (PLAUSIBLE-DCLSP1 LST)
+     (QUOTE-LISTP L)
+     (STRIP-DCLS LST)
+     (STRIP-DCLS1 LST)
+     (STRIP-KEYWORD-LIST LST)
+     (SUBCOR-VAR FORM)
+     (SUBCOR-VAR-LST FORMS)
+     (SUBCOR-VAR1 VARS)
+     (SUBLIS-VAR)
+     (SUBLIS-VAR-LST)
+     (SUBLIS-VAR1 FORM)
+     (SUBLIS-VAR1-LST L)
+     (SUBST-EXPR)
+     (SUBST-EXPR-ERROR)
+     (SUBST-EXPR1 TERM)
+     (SUBST-EXPR1-LST ARGS)
+     (SUBST-VAR FORM)
+     (SUBST-VAR-LST L))))
+
+(defconst *len-system-verify-guards-alist*
+  (length *system-verify-guards-alist*))
+
+(defmacro chk-new-verified-guards (n)
+  (cond
+   ((or (not (natp n))
+        (> n *len-system-verify-guards-alist*))
+    `(er soft 'chk-new-verified-guards
+         "The index ~x0 is not a valid index for *system-verify-guards-alist*."
+         ',n))
+   ((eql n *len-system-verify-guards-alist*)
+    '(value-triple :CHK-NEW-VERIFIED-GUARDS-COMPLETE))
+   (t
+    (let* ((pair (nth n *system-verify-guards-alist*)) 
+           (user-book-name (car pair))
+           (fns (cdr pair)))
+      `(progn (include-book ,user-book-name
+                            :DIR :SYSTEM
+                            :UNCERTIFIED-OKP nil
+                            :DEFAXIOMS-OKP nil
+                            :SKIP-PROOFS-OKP nil
+                            :TTAGS nil)
+              (assert-event
+               (equal ',fns
+                      (new-verify-guards-fns state))
+               :msg (msg "ERROR: The set of newly guard-verified functions ~
+                          from the ACL2 community book ~x0 does not match the ~
+                          expected set from the constant ~
+                          *system-verify-guards-alist*.~|~%From the ~
+                          book:~|~X13~|~%Expected from ~
+                          *system-verify-guards-alist*:~|~X23~|"
+                         ',user-book-name
+                         (new-verify-guards-fns state)
+                         ',fns
+                         nil))
+              (value-triple :CHK-NEW-VERIFIED-GUARDS-SUCCESS))))))
+
+(defun system-verify-guards-fn-1 (fns-alist acc)
+  (declare (xargs :guard (symbol-alistp fns-alist)))
+  (cond ((endp fns-alist) acc)
+        (t (system-verify-guards-fn-1
+            (cdr fns-alist)
+            (cons `(skip-proofs (verify-termination-boot-strap ; and guards
+                                 ,(caar fns-alist)
+                                 ,@(let ((ms (cdar fns-alist)))
+                                     (and ms
+                                          `((declare (xargs :measure
+                                                            (:? ,@ms))))))))
+                  acc)))))
+
+(defun cons-absolute-event-numbers (fns-alist wrld acc)
+  (declare (xargs :guard (and (symbol-alistp fns-alist)
+                              (plist-worldp wrld)
+                              (alistp acc))))
+  (cond ((endp fns-alist) acc)
+        (t (cons-absolute-event-numbers
+            (cdr fns-alist)
+            wrld
+            (acons (or (getprop (caar fns-alist) 'absolute-event-number nil
+                                'current-acl2-world wrld)
+                       (er hard? 'cons-absolute-event-numbers
+                           "The 'absolute-event-number property is missing ~
+                            for ~x0."
+                           (caar fns-alist)))
+                   (car fns-alist)
+                   acc)))))
+
+(defun sort->-absolute-event-number (fns-alist wrld)
+  (declare (xargs :mode :program)) ; because of merge-sort-car->
+  (strip-cdrs (merge-sort-car->
+               (cons-absolute-event-numbers fns-alist wrld nil))))
+
+(defun system-verify-guards-fn (alist wrld acc)
+  (declare (xargs :mode :program)) ; because of sort->-absolute-event-number
+  (cond ((endp alist) acc)
+        (t (system-verify-guards-fn
+            (cdr alist)
+            wrld
+            (system-verify-guards-fn-1
+             (sort->-absolute-event-number (cdar alist) wrld)
+             acc)))))
+
+(defmacro system-verify-guards ()
+  `(make-event
+    (let ((events (system-verify-guards-fn *system-verify-guards-alist*
+                                           (w state)
+                                           nil)))
+      (list* 'encapsulate
+             ()
+             '(set-verify-guards-eagerness 2)
+             events))))
+
+; Normally we go ahead and trust *system-verify-guards-alist*, installing
+; guard-verified functions with the following form.  But when feature
+; :acl2-devel is set, then we do not do so, as we instead intend to run
+; (chk-new-verified-guards i) for each i less than the length of
+; *system-verify-guards-alist*, in order to check that the effect of
+; system-verify-guards is sound.
+#+(and acl2-loop-only ; Note that make-event can't be called here in raw Lisp.
+       (not acl2-devel))
+(system-verify-guards)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; End
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
