@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -162,7 +156,7 @@
 
 (defconst *acl2-exports*
 
-; See books/misc/check-acl2-exports.lisp for a list of symbols,
+; See community book books/misc/check-acl2-exports.lisp for a list of symbols,
 ; *acl2-exports-exclusions*, deliberately excluded from this list.
 
 ; Let's keep this list sorted (more efficient for defpkg when users choose to
@@ -190,13 +184,13 @@
         32-BIT-INTEGERP-FORWARD-TO-INTEGERP
         < <-ON-OTHERS <= = > >= ?-FN @ A! ABORT!
         ABS ACCUMULATED-PERSISTENCE ACL2-COUNT
-        ACL2-INPUT-CHANNEL-PACKAGE ACL2-NUMBERP
+        ACL2-INPUT-CHANNEL-PACKAGE ACL2-NUMBER-LISTP ACL2-NUMBERP
         ACL2-ORACLE ACL2-OUTPUT-CHANNEL-PACKAGE
         ACL2-PACKAGE ACONS ACTIVE-RUNEP
         ADD-BINOP ADD-CUSTOM-KEYWORD-HINT
         ADD-DEFAULT-HINTS
         ADD-DEFAULT-HINTS! ADD-INCLUDE-BOOK-DIR
-        ADD-INVISIBLE-FNS ADD-MACRO-ALIAS
+        ADD-INVISIBLE-FNS ADD-MACRO-ALIAS ADD-MACRO-FN
         ADD-MATCH-FREE-OVERRIDE ADD-NTH-ALIAS
         ADD-OVERRIDE-HINTS ADD-OVERRIDE-HINTS!
         ADD-PAIR ADD-PAIR-PRESERVES-ALL-BOUNDP
@@ -224,7 +218,7 @@
         BACKCHAIN-LIMIT BIG-CLOCK-ENTRY
         BIG-CLOCK-NEGATIVE-P BINARY-*
         BINARY-+ BINARY-APPEND BIND-FREE
-        BINOP-TABLE BIT BOOLE$ BOOLEAN-LISTP
+        BIT BOOLE$ BOOLEAN-LISTP
         BOOLEAN-LISTP-CONS BOOLEAN-LISTP-FORWARD
         BOOLEAN-LISTP-FORWARD-TO-SYMBOL-LISTP
         BOOLEANP BOOLEANP-CHARACTERP
@@ -267,6 +261,7 @@
         CODE-CHAR-TYPE COERCE COERCE-INVERSE-1
         COERCE-INVERSE-2 COERCE-OBJECT-TO-STATE
         COERCE-STATE-TO-OBJECT
+        COMMUNITY-BOOKS
         COMMUTATIVITY-OF-*
         COMMUTATIVITY-OF-+ COMP COMPLETION-OF-*
         COMPLETION-OF-+ COMPLETION-OF-<
@@ -295,7 +290,8 @@
         CONSP-ASSOC-EQUAL COROLLARY
         CPU-CORE-COUNT CURRENT-PACKAGE
         CURRENT-THEORY CW CW! CW-GSTACK DECLARE
-        DECREMENT-BIG-CLOCK DEFABBREV DEFATTACH
+        DECREMENT-BIG-CLOCK DEFABBREV DEFABSSTOBJ
+        DEFABSSTOBJ-MISSING-EVENTS DEFATTACH
         DEFAULT DEFAULT-*-1 DEFAULT-*-2
         DEFAULT-+-1 DEFAULT-+-2 DEFAULT-<-1
         DEFAULT-<-2 DEFAULT-BACKCHAIN-LIMIT
@@ -308,8 +304,8 @@
         DEFAULT-DENOMINATOR
         DEFAULT-HINTS DEFAULT-IMAGPART
         DEFAULT-MEASURE-FUNCTION
-        DEFAULT-NUMERATOR DEFAULT-PRINT-PROMPT
-        DEFAULT-REALPART DEFAULT-RULER-EXTENDERS
+        DEFAULT-NUMERATOR DEFAULT-TOTAL-PARALLELISM-WORK-LIMIT
+        DEFAULT-PRINT-PROMPT DEFAULT-REALPART DEFAULT-RULER-EXTENDERS
         DEFAULT-SYMBOL-NAME
         DEFAULT-SYMBOL-PACKAGE-NAME
         DEFAULT-UNARY-/ DEFAULT-UNARY-MINUS
@@ -320,17 +316,17 @@
         DEFINE-PC-ATOMIC-MACRO DEFINE-PC-HELP
         DEFINE-PC-MACRO DEFINE-PC-META
         DEFINE-TRUSTED-CLAUSE-PROCESSOR
-        DEFLABEL DEFMACRO DEFMACRO-LAST DEFN DEFPKG
-        DEFPROXY DEFREFINEMENT DEFSTOBJ DEFSTUB DEFTHEORY
-        DEFTHM DEFTHMD DEFTTAG DEFUN DEFUN-NX
-        DEFUN-SK DEFUND DEFUND-NX DEFUNS
+        DEFLABEL DEFLOCK DEFMACRO DEFMACRO-LAST DEFN DEFND DEFPKG
+        DEFPROXY DEFREFINEMENT DEFSTOBJ DEFSTUB DEFTHEORY DEFTHEORY-STATIC
+        DEFTHM DEFTHMD DEFTTAG DEFUN DEFUN-INLINE DEFUN-NOTINLINE DEFUN-NX
+        DEFUN-SK DEFUND DEFUND-INLINE DEFUND-NOTINLINE DEFUND-NX DEFUNS
         DELETE-ASSOC DELETE-ASSOC-EQ DELETE-ASSOC-EQUAL
         DELETE-INCLUDE-BOOK-DIR
         DENOMINATOR DIGIT-CHAR-P DIGIT-TO-CHAR
         DIMENSIONS DISABLE DISABLE-FORCING
         DISABLE-IMMEDIATE-FORCE-MODEP
-        DISABLEDP DISTRIBUTIVITY
-        DOC DOC! DOCS DOUBLE-REWRITE
+        DISABLEDP DISASSEMBLE$ DISTRIBUTIVITY
+        DMR-START DMR-STOP DOC DOC! DOCS DOUBLE-REWRITE
         DUPLICATES E/D E0-ORD-< E0-ORDINALP
         EC-CALL EIGHTH ELIMINATE-DESTRUCTORS
         ELIMINATE-IRRELEVANCE
@@ -350,7 +346,8 @@
         EXPT-TYPE-PRESCRIPTION-NON-ZERO-BASE
         EXTEND-32-BIT-INTEGER-STACK
         EXTEND-T-STACK
-        EXTEND-WORLD EXTRA-INFO FERTILIZE FC-REPORT
+        EXTEND-WORLD EXTRA-INFO F-GET-GLOBAL F-PUT-GLOBAL
+        FAST-ALIST-FREE-ON-EXIT FERTILIZE FC-REPORT
         FGETPROP FIFTH FILE-CLOCK FILE-CLOCK-P
         FILE-CLOCK-P-FORWARD-TO-INTEGERP
         FIRST FIRST-N-AC
@@ -359,9 +356,9 @@
         FMS-TO-STRING FMS!-TO-STRING FMT-TO-STRING FMT!-TO-STRING
         FMT1-TO-STRING FMT1!-TO-STRING
         FORCE FOURTH FUNCTION-SYMBOLP
-        FUNCTION-THEORY GAG-MODE GC$ GENERALIZE
+        FUNCTION-THEORY GAG-MODE GC$ GC-VERBOSE GENERALIZE GET-COMMAND-SEQUENCE
         GET-GLOBAL GET-OUTPUT-STREAM-STRING$ GET-TIMER GET-WORMHOLE-STATUS
-        GETENV$ GETPROP GETPROP-DEFAULT GETPROPS
+        GCS GETENV$ GETPROP GETPROP-DEFAULT GETPROPS
         GETPROPS1 GLOBAL-TABLE GLOBAL-TABLE-CARS
         GLOBAL-TABLE-CARS1 GLOBAL-VAL
         GOOD-ATOM-LISTP GOOD-BYE GROUND-ZERO
@@ -417,7 +414,7 @@
         MAIN-TIMER MAIN-TIMER-TYPE-PRESCRIPTION
         MAKE-CHARACTER-LIST
         MAKE-CHARACTER-LIST-MAKE-CHARACTER-LIST
-        MAKE-EVENT
+        MAKE-EVENT MAKE-FAST-ALIST
         MAKE-FMT-BINDINGS MAKE-INPUT-CHANNEL
         MAKE-LIST MAKE-LIST-AC MAKE-MV-NTHS
         MAKE-ORD MAKE-OUTPUT-CHANNEL
@@ -431,10 +428,10 @@
         MORE MORE! MORE-DOC MUST-BE-EQUAL
         MUTUAL-RECURSION MUTUAL-RECURSION-GUARDP
         MV MV? MV-LET MV?-LET MV-LIST
-        MV-NTH NATP NEEDS-SLASHES NEWLINE
+        MV-NTH NAT-LISTP NATP NEEDS-SLASHES NEWLINE
         NFIX NIL NIL-IS-NOT-CIRCULAR NINTH
         NO-DUPLICATESP NO-DUPLICATESP-EQ NO-DUPLICATESP-EQUAL
-        NONNEGATIVE-INTEGER-QUOTIENT
+        NON-EXEC NONNEGATIVE-INTEGER-QUOTIENT
         NONNEGATIVE-PRODUCT NONZERO-IMAGPART
         NOT NQTHM-TO-ACL2 NTH NTH-0-CONS
         NTH-0-READ-RUN-TIME-TYPE-PRESCRIPTION
@@ -458,7 +455,9 @@
         OPEN-OUTPUT-CHANNEL-P
         OPEN-OUTPUT-CHANNEL-P1
         OPEN-OUTPUT-CHANNELS OPEN-TRACE-FILE
-        OR OR-MACRO ORDERED-SYMBOL-ALISTP
+        OR OR-MACRO
+        ORACLE-FUNCALL ORACLE-APPLY ORACLE-APPLY-RAW
+        ORDERED-SYMBOL-ALISTP
         ORDERED-SYMBOL-ALISTP-ADD-PAIR
         ORDERED-SYMBOL-ALISTP-ADD-PAIR-FORWARD
         ORDERED-SYMBOL-ALISTP-FORWARD-TO-SYMBOL-ALISTP
@@ -476,13 +475,13 @@
         POSITIVE POSP POWER-EVAL PPROGN PR
         PR! PREPROCESS PRIN1$ PRIN1-WITH-SLASHES
         PRIN1-WITH-SLASHES1 PRINC$ PRINT-GV
-        PRINT-OBJECT$ PRINT-RATIONAL-AS-DECIMAL
-        PRINT-TIMER PROFILE
+        PRINT-OBJECT$ PRINT-OBJECT$-SER PRINT-RATIONAL-AS-DECIMAL
+        PRINT-SUMMARY-USER PRINT-TIMER PROFILE
         PROG2$ PROGN PROGN! PROGN$ PROGRAM
         PROOF-TREE PROOFS-CO PROPER-CONSP
         PROPS PROVE PSEUDO-TERM-LISTP
         PSEUDO-TERM-LISTP-FORWARD-TO-TRUE-LISTP
-        PSEUDO-TERMP PSO PSO! PSOG PSTACK
+        PSEUDO-TERMP PSO PSO! PSOF PSOG PSTACK
         PUFF PUFF* PUSH-TIMER PUSH-UNTOUCHABLE
         PUT-ASSOC PUT-ASSOC-EQ PUT-ASSOC-EQL
         PUT-ASSOC-EQUAL PUT-GLOBAL PUTPROP
@@ -515,24 +514,29 @@
         REALPART-COMPLEX REALPART-IMAGPART-ELIM
         REBUILD REDEF REDEF!
         REDEF+ REDEF- REDO-FLAT REM REMOVE
+        REGENERATE-TAU-DATABASE
         REMOVE-BINOP REMOVE-CUSTOM-KEYWORD-HINT
         REMOVE-DEFAULT-HINTS
         REMOVE-DEFAULT-HINTS!
         REMOVE-DUPLICATES REMOVE-DUPLICATES-EQ REMOVE-DUPLICATES-EQL
         REMOVE-DUPLICATES-EQUAL
         REMOVE-EQ REMOVE-EQUAL
-        REMOVE-INVISIBLE-FNS REMOVE-MACRO-ALIAS
+        REMOVE-INVISIBLE-FNS REMOVE-MACRO-ALIAS REMOVE-MACRO-FN
         REMOVE-NTH-ALIAS REMOVE-OVERRIDE-HINTS
         REMOVE-OVERRIDE-HINTS!
         REMOVE-RAW-ARITY REMOVE-UNTOUCHABLE
         REMOVE1 REMOVE1-EQ REMOVE1-EQUAL RESET-FC-REPORTING
         RESET-KILL-RING RESET-LD-SPECIALS
         RESET-PREHISTORY RESET-PRINT-CONTROL
-        RESIZE-LIST REST RETRACT-WORLD
+        RESIZE-LIST REST RESTORE-MEMOIZATION-SETTINGS RETRACT-WORLD
         RETRIEVE RETURN-LAST RETURN-LAST-TABLE
         REVAPPEND REVERSE REWRITE-STACK-LIMIT
-        RFIX ROUND SATISFIES SAVE-EXEC
-        SEARCH SECOND SET-BACKCHAIN-LIMIT
+        RFIX ROUND SATISFIES
+        SAVE-AND-CLEAR-MEMOIZATION-SETTINGS SAVE-EXEC
+        SEARCH SECOND
+        SERIALIZE-READ SERIALIZE-WRITE SET-ABSSTOBJ-DEBUG
+        SET-ACCUMULATED-PERSISTENCE
+        SET-BACKCHAIN-LIMIT
         SET-BODY SET-BOGUS-DEFUN-HINTS-OK
         SET-BOGUS-MUTUAL-RECURSION-OK
         SET-CASE-SPLIT-LIMITATIONS
@@ -566,24 +570,26 @@
         SET-MEASURE-FUNCTION SET-NON-LINEAR
         SET-NON-LINEARP SET-NU-REWRITER-MODE
         SET-OVERRIDE-HINTS SET-OVERRIDE-HINTS!
-        SET-PARALLEL-EVALUATION
+        SET-PARALLEL-EXECUTION
         SET-PRINT-BASE SET-PRINT-CASE
         SET-PRINT-CIRCLE SET-PRINT-CLAUSE-IDS
         SET-PRINT-ESCAPE SET-PRINT-LENGTH
         SET-PRINT-LEVEL SET-PRINT-LINES
         SET-PRINT-RADIX SET-PRINT-READABLY
-        SET-PRINT-RIGHT-MARGIN SET-RAW-MODE
+        SET-PRINT-RIGHT-MARGIN SET-PROVER-STEP-LIMIT SET-RAW-MODE
         SET-RAW-MODE-ON! SET-RAW-PROOF-FORMAT
         SET-REWRITE-STACK-LIMIT
         SET-RULER-EXTENDERS
         SET-RW-CACHE-STATE SET-RW-CACHE-STATE!
-        SET-SAVED-OUTPUT SET-STATE-OK
-        SET-PROVER-STEP-LIMIT
-        SET-TAINTED-OK SET-TAINTED-OKP
-        SET-TIMER SET-TRACE-EVISC-TUPLE
-        SET-VERIFY-GUARDS-EAGERNESS
+        SET-SAVED-OUTPUT SET-SERIALIZE-CHARACTER SET-STATE-OK
+        SET-TAU-AUTO-MODE SET-TIMER
+        SET-TOTAL-PARALLELISM-WORK-LIMIT
+        SET-TOTAL-PARALLELISM-WORK-LIMIT-ERROR
+        SET-TRACE-EVISC-TUPLE SET-VERIFY-GUARDS-EAGERNESS
         SET-W SET-WELL-FOUNDED-RELATION
         SET-WATERFALL-PARALLELISM
+        SET-WATERFALL-PARALLELISM-HACKS-ENABLED
+        SET-WATERFALL-PARALLELISM-HACKS-ENABLED!
         SET-WATERFALL-PRINTING
         SET-WORMHOLE-DATA
         SET-WORMHOLE-ENTRY-CODE SET-WRITE-ACL2X SETENV$ SEVENTH
@@ -602,7 +608,7 @@
         STANDARD-OI STANDARD-STRING-ALISTP
         STANDARD-STRING-ALISTP-FORWARD-TO-ALISTP
         START-PROOF-TREE
-        STATE STATE-GLOBAL-LET*-CLEANUP
+        STATE STATE-GLOBAL-LET* STATE-GLOBAL-LET*-CLEANUP
         STATE-GLOBAL-LET*-GET-GLOBALS
         STATE-GLOBAL-LET*-PUT-GLOBALS STATE-P
         STATE-P-IMPLIES-AND-FORWARD-TO-STATE-P1
@@ -639,10 +645,11 @@
         SYMBOLP-INTERN-IN-PACKAGE-OF-SYMBOL
         SYNP SYNTAXP SYS-CALL SYS-CALL-STATUS
         T T-STACK T-STACK-LENGTH T-STACK-LENGTH1
-        TABLE TABLE-ALIST TAKE TENTH
-        TERM-ORDER THE THE-ERROR THE-FIXNUM
+        TABLE TABLE-ALIST TAKE
+        TAU-DATA TAU-DATABASE TAU-STATUS TAU-SYSTEM
+        TENTH TERM-ORDER THE THE-ERROR THE-FIXNUM
         THE-FIXNUM! THEORY THEORY-INVARIANT
-        THIRD THM TIME$ TIMER-ALISTP
+        THIRD THM TIME$ TIME-TRACKER TIME-TRACKER-TAU TIMER-ALISTP
         TIMER-ALISTP-FORWARD-TO-TRUE-LIST-LISTP-AND-SYMBOL-ALISTP
         TOGGLE-PC-MACRO
         TOP-LEVEL TRACE! TRACE$ TRANS
@@ -659,7 +666,7 @@
         UNARY-FUNCTION-SYMBOL-LISTP UNICITY-OF-0
         UNICITY-OF-1 UNION$ UNION-EQ UNION-EQUAL
         UNION-THEORIES UNIVERSAL-THEORY
-        UNMONITOR UNSAVE UNSIGNED-BYTE UNTRACE$
+        UNMONITOR UNSAVE UNSIGNED-BYTE UNTRACE$ UNTRANS-TABLE
         UNTRANSLATE UPDATE-32-BIT-INTEGER-STACK
         UPDATE-ACL2-ORACLE
         UPDATE-ACL2-ORACLE-PRESERVES-STATE-P1
@@ -679,11 +686,13 @@
         VERIFY-GUARDS VERIFY-GUARDS-FORMULA
         VERIFY-TERMINATION W WALKABOUT
         WARNING! WATERFALL-PARALLELISM WATERFALL-PRINTING 
-        WET WITH-GUARD-CHECKING
+        WET WITH-FAST-ALIST WITH-GUARD-CHECKING
         WITH-LIVE-STATE WITH-LOCAL-STATE WITH-LOCAL-STOBJ WITH-OUTPUT
+        WITH-OUTPUT-LOCK
         WITH-PROVER-STEP-LIMIT WITH-PROVER-STEP-LIMIT!
-        WITH-PROVER-TIME-LIMIT WITHOUT-EVISC
-        WORLD WORMHOLE WORMHOLE-DATA
+        WITH-PROVER-TIME-LIMIT WITH-SERIALIZE-CHARACTER
+        WITH-STOLEN-ALIST WITHOUT-EVISC
+        WOF WORLD WORMHOLE WORMHOLE-DATA
         WORMHOLE-ENTRY-CODE WORMHOLE-EVAL
         WORMHOLE-P WORMHOLE-STATUSP
         WORMHOLE1 WRITABLE-FILE-LISTP
