@@ -1,13 +1,12 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 was produced by modifying ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTES-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of Version 2 of the GNU General Public License as
+; published by the Free Software Foundation.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -367,7 +366,7 @@ an html file.  See :DOC markup")
   (pprogn
    (princ$ "<br><br><br><a href=\"" channel state)
    (princ$ html-file channel state)
-   (princ$ "\"><img src=\"llogo.gif\"></a>" channel state)
+   (princ$ "\"><img src=\"acl2-logo-62-41.gif\"></a>" channel state)
    (cond ((null index-file) ; then channel is to the index file
           state)
          (t (pprogn
