@@ -1,13 +1,12 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTES-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of Version 2 of the GNU General Public License as
+; published by the Free Software Foundation.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -74,7 +73,9 @@
   ;      "control-h f compare-windows", for more info).
   ; "control-t q" is like "control-t w" above, but ignores whitespace (and case
   ;      too, with a positive prefix argument).
-  ; Lisp mode comes up with auto-fill mode on, right margin set at column 79.
+  ; Lisp mode comes up with auto-fill mode on, right margin set at column 79,
+  ;      and causes latin-1 (i.e., iso-8859-1) encoding (see :doc
+  ;      character-encoding) to be used when saving a buffer to a file.
   ;      If X Windows is being run, then font-lock-mode is also turned on,
   ;      which causes Emacs to color text in .lisp files.  If you don't want
   ;      colors in .lisp files, put this in your .emacs file after the load of
@@ -88,12 +89,23 @@
   ; "control-t control-f" buries the current buffer (puts it on the bottom of
   ;      the buffer stack, out of the way, without killing the buffer)
 ; ACL2 proof-tree support
+  ; NOTE: This works by default if you install the ACL2 community books, as
+  ;       most ACL2 users do, in the books/ directory of your ACL2
+  ;       distribution.  Otherwise, you will need to set the variable
+  ;       *acl2-interface-dir* to a directory string containing a file
+  ;       top-start-shell-acl2.el that defines the functions start-proof-tree
+  ;       and start-proof-tree-noninteractive in emacs.  For user-level
+  ;       documentation provided in the ACL2 community books implementation,
+  ;       see the following file included there:
+  ;       books/interface/emacs/PROOF-TREE-EMACS.txt
   ; "meta-x start-proof-tree" starts proof-tree tracking in the current buffer
   ;      (where ACL2 is running).  See ACL2 documentation for PROOF-TREE for
   ;      more information.
   ; Function start-proof-tree-noninteractive (see below) can be used to start
   ;      proof-trees when emacs starts up; see below.
 ; Run ACL2 as inferior process
+  ; NOTE: This works by default if you install the ACL2 community books.
+  ;       Otherwise, see the NOTE above on "ACL2 proof-tree support".
   ; "meta-x run-acl2" starts up acl2 as an inferior process in emacs.  You may
   ;      have better luck simply issuing your ACL2 command in an ordinary
   ;      (emacs) shell.
@@ -606,6 +618,7 @@ then also ignore case if that argument is positive, else do not ignore case."
   (setq indent-tabs-mode nil)   
   (setq comment-column 0)
   (turn-on-auto-fill)
+  (setq save-buffer-coding-system 'iso-8859-1)
   )
 
 (if (not (boundp 'lisp-mode-hook)) (setq lisp-mode-hook nil))
@@ -745,7 +758,8 @@ beginning of the string that was processed."
   (if (boundp '*acl2-interface-dir*)
       *acl2-interface-dir*
     (setq *acl2-interface-dir*
-	  (concat (acl2-sources-dir) "interface/emacs/"))))
+; common location (i.e., for those who install ACL2 community books in books/):
+	  (concat (acl2-sources-dir) "books/interface/emacs/"))))
 
 (autoload 'start-proof-tree
   (concat (acl2-interface-dir) "top-start-shell-acl2")
@@ -938,9 +952,9 @@ of the current s-expression in the enclosing list"
 ; (setq auto-save-timeout 0)
 
 ; Abbrevs are great!  For example, if you type
-; control-x '
-; followed by
 ; ac
+; followed by
+; control-x '
 ; then (assuming the following form has been evaluated), the "ac" will be
 ; replaced by the value of *acl2-sources-dir*.
 ; (define-abbrev-table 'global-abbrev-table
