@@ -1,23 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
-; (C) 1997 Computational Logic, Inc.  See the documentation topic
-; NOTE-2-0.
+; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -117,8 +110,8 @@ supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
 In the logic, ~c[hons-copy] is just the identity function; we leave it enabled
 and would think it odd to ever prove a theorem about it.
 
-Under the hood, ~c[hons-copy] does whatever is necessary to produce a normed
-version of X.~/
+Under the hood, ~c[hons-copy] does whatever is necessary to produce a
+~il[normed] version of X.~/
 
 What might this involve?
 
@@ -180,7 +173,7 @@ In the logic, ~c[hons] is just ~ilc[cons]; we leave it enabled and would think
 it odd to ever prove a theorem about it.
 
 Under the hood, ~c[hons] does whatever is necessary to ensure that its result
-is normed.~/
+is ~il[normed].~/
 
 What might this involve?
 
@@ -219,9 +212,8 @@ recursively checking large cons structures for equality.~/
 Note.  If ~c[hons-equal] is given arguments that do not contain many normed
 objects, it can actually be much slower than ~ilc[equal]!  This is because it
 checks to see whether its arguments are normed at each recursive step, and so
-you are repeatedly paying the price of such checks.  See also
-~ilc[hons-equal-lite], which only checks at the top level whether its arguments
-are normed."
+you are repeatedly paying the price of such checks.  Also ~pl[hons-equal-lite],
+which only checks at the top level whether its arguments are normed."
 
   ;; Has an under-the-hood implementation
   (equal x y))
@@ -268,7 +260,7 @@ it odd to ever prove a theorem about it.
 Under the hood, ~c[hons-clear] brutally (1) clears all the tables that govern
 which conses are ~il[normed], then (2) optionally garbage collects, per the
 ~c[gc] argument, then finally (3) re-norms the keys of ~ilc[fast-alists] and
-persistently normed conses; see ~ilc[hons-copy-persistent].~/
+persistently normed conses; ~pl[hons-copy-persistent].~/
 
 Note.  The hash tables making up the Hons Space retain their sizes after being
 cleared.  If you want to shrink them, see ~ilc[hons-resize].
@@ -339,7 +331,7 @@ the sizes of the tables in the Hons Space.~/
 This information may be useful for deciding if you want to garbage collect
 using functions such as ~ilc[hons-clear] or ~ilc[hons-wash].  It may also be
 useful for determining good initial sizes for the Hons Space hash tables for
-your particular computation; see ~ilc[hons-resize]."
+your particular computation; ~pl[hons-resize]."
 
   ;; Has an under-the-hood implementation
   nil)
@@ -398,7 +390,7 @@ more appropriate initial size may help to reduce this overhead.
 2.  Reducing Memory Usage by Resizing Down
 
 Resizing can also be used to shrink the hash tables.  This might possibly be
-useful immediately after a ~c[hons-clear] to free up memory taken by the hash
+useful immediately after a ~ilc[hons-clear] to free up memory taken by the hash
 tables themselves.  (Of course, the tables will grow again as you create new
 normed objects.)
 
@@ -432,6 +424,70 @@ soon after a ~ilc[hons-clear]."
   ;; Has an under-the-hood implementation
   nil)
 
+(defdoc hons-note
+  ":Doc-Section Hons-and-Memoization
+
+notes about ~il[HONS], especially pertaining to expensive resizing operations~/
+
+This ~il[documentation] topic relates to the experimental extension of ACL2
+supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+
+Certain ``Hons-Notes'' are printed to the terminal to report
+implementation-level information about the management of ~il[HONS] structures.
+Some of these may be low-level and of interest mainly to system developers.
+But below we discuss what users can learn from a particular hons-note,
+``ADDR-LIMIT reached''.  In short: If you are seeing a lot of such hons-notes,
+then you may be using a lot of memory.  (Maybe you can reduce that memory; for
+certain BDD operations, for example (as defined in community books
+~c[books/centaur/ubdds/]), variable ordering can make a big difference.)~/
+
+By way of background:
+
+~bq[]
+The ADDR-HT is the main hash table that lets us look up ~il[normed] conses,
+i.e., ~il[hons]es.  It is an ordinary Common Lisp hash table, so when it starts
+to get too full the Lisp will grow it.  It can get really big.  (It has been
+seen to take more than 10 GB of memory just for the hash table's array, not to
+mention the actual conses!)
+
+Hons-Notes may be printed when the ADDR-HT is getting really full.  This growth
+can be expensive and can lead to memory problems.  Think about what it takes to
+resize a hash table:
+
+  (1) allocate a new, bigger array~nl[]
+  (2) rehash elements, copying them into the new array~nl[]
+  (3) free the old array
+
+Until you reach step (3) and a garbage collection takes place, you have to have
+enough memory to have both the old and new arrays allocated.  If the old array
+was 10 GB and we want to allocate a new one that's 15 GB, this can get pretty
+bad.  Also, rehashing the elements is expensive time-wise when there are lots
+of elements.
+
+Since resizing a hash table is expensive, we want to avoid it.  There's a
+~ilc[hons-resize] command for this.  You may want your books to start with
+something like the following.
+
+~bv[]
+  (value-triple (set-max-mem (* 30 (expt 2 30))))      ; 30 GB heap
+  (value-triple (hons-resize :addr-ht #u_100_000_000)) ; 100M honses
+~ev[]
+
+Basically, if you roughly know how many honses your book will need, you can
+just get them up front and then never to resize.~eq[]
+
+The Hons-Notes about ``ADDR-LIMIT reached'' are basically there to warn you
+that the ADDR-HT is being resized.  This can help you realize that your
+~ilc[hons-resize] command had too small of an ADDR-HT size, or might suggest
+that your book should have a ~ilc[hons-resize] command.  There are also
+commands like ~c[(]~ilc[hons-summary]~c[)] and, defined in community book
+~c[books/centaur/misc/memory-mgmt-logic.lisp], ~c[(hons-analyze-memory nil)].
+These can show you how many honses you currently have, how much space they are
+taking, and that sort of thing.  (A nice trick is to call ~ilc[hons-summary] at
+the end of a book, to get an idea of how many honses you should ask for in your
+~ilc[hons-resize] command).
+")
+
 (defdoc fast-alists
   ":Doc-Section Hons-and-Memoization
 
@@ -528,8 +584,8 @@ break (an error from which you can continue) ensues.  For instance:
  (set-slow-alist-action nil)       ; do not warn or break
 ~ev[]
 
-The above forms expand to table events, so they can be embedded in encapsulates
-and books, wrapped in ~ilc[local], and so on.")
+The above forms expand to ~ilc[table] ~il[events], so they can be embedded in
+~ilc[encapsulate]s and ~il[books], wrapped in ~ilc[local], and so on.")
 
 (table hons 'slow-alist-warning :warning)
 
@@ -698,6 +754,31 @@ and this new object would not be associated with the fast alist's hash table."
   ;; Has an under-the-hood implementation
   (cons (cons key val) alist))
 
+#+(or acl2-loop-only (not hons))
+(defn make-fast-alist (alist)
+  ":Doc-Section Hons-and-Memoization
+
+~c[(make-fast-alist alist)] creates a fast-alist from the input alist,
+returning ~c[alist] itself or, in some cases, a new object equal to it.~/
+
+Note: it is often better to use ~c[with-fast-alist]; ~pl[with-fast-alist].
+
+Logically, ~c[make-fast-alist] is the identity function.
+
+Under the hood, we construct and return an object that is ~c[equal] to
+~c[alist] and which is a fast alist.  If ~c[alist] is already a fast alist,
+almost no work is required: we simply return it unchanged.
+
+When ~c[alist] is not fast, we must minimally construct a hash table for its
+bindings.  It is often possible to bind this new hash table to ~c[alist]
+itself.  But in certain cases when the alist keys are not ~il[normed], a new
+alist must be constructed, also, and so we may return an ~c[equal] but not
+~c[eq] alist.  (In these cases, we still try to avoid at least some consing by
+reusing the \"longest normed tail\" of the alist.)~/~/"
+
+  ;; Has an under-the-hood implementation
+  alist)
+
 #+(or acl2-loop-only (not hons))
 (defn hons-shrink-alist (alist ans)
   ":Doc-Section Hons-and-Memoization
@@ -721,8 +802,7 @@ Logically, ~c[hons-shrink-alist] is defined as follows:
         (hons-shrink-alist (cdr alist) (cons (car alist) ans))))
 ~ev[]
 
-The alist argument need not be a fast alist, and it is not modified by the
-shrink.
+The alist argument need not be a fast alist.
 
 Typically ans is set to nil or some other atom.  In this case, shrinking
 produces a new, fast alist which is like alist except that (1) any
@@ -832,11 +912,164 @@ would think it odd to ever prove a theorem about it.
 
 Under the hood, this function gathers and prints some basic statistics about
 the current fast alists.  It may be useful for identifying fast alists that
-should have been freed with ~ilc[fast-alist-free]~/~/"
+should have been freed with ~ilc[fast-alist-free].~/~/"
 
   ;; Has an under-the-hood implementation
   nil)
 
+(defdoc with-fast-alist
+  ":Doc-Section Hons-and-Memoization
+
+~c[(with-fast-alist name form)] causes ~c[name] to be a fast alist for the
+execution of ~c[form].~/
+
+Logically, ~c[with-fast-alist] just returns ~c[form].
+
+Under the hood, we cause ~c[alist] to become a fast alist before executing
+~c[form].  If doing so caused us to introduce a new hash table, the hash table
+is automatically freed after ~c[form] completes.
+
+More accurately, under the hood ~c[(with-fast-alist name form)] essentially
+expands to something like:
+
+~bv[]
+ (if (already-fast-alist-p name)
+     form
+   (let ((<temp> (make-fast-alist name)))
+     (prog1 form
+            (fast-alist-free <temp>))))
+~ev[]
+
+Practically speaking, ~c[with-fast-alist] is frequently a better choice then
+just using ~ilc[make-fast-alist], and is particularly useful for writing
+functions that can take either fast or slow alists as arguments.  That is,
+consider the difference between:
+
+~bv[]
+ (defun bad (alist ...)
+   (let* ((fast-alist (make-fast-alist alist))
+          (answer     (expensive-computation fast-alist ...)))
+    (prog2$ (fast-alist-free fast-alist)
+            answer)))
+
+ (defun good (alist ...)
+    (with-fast-alist alist
+      (expensive-computation alist ...)))
+~ev[]
+
+Either approach is fine if the caller provides a slow ~c[alist].  But if the
+input ~c[alist] is already fast, ~c[bad] will (perhaps unexpectedly) free it!
+On the other hand, ~c[good] is able to take advantage of an already-fast
+argument and will not cause it to be inadvertently freed.
+
+See also the macro ~c[with-fast-alists] defined in the community book
+~c[\"books/centaur/misc/hons-extra.lisp\"], which allows you to call
+~ilc[with-fast-alist] on several alists simultaneously.
+
+The community book ~c[\"books/centaur/misc/hons-extra.lisp\"] extends the
+~c[b*] macro (defined in the community book ~c[\"books/tools/bstar.lisp\"])
+with the ~c[with-fast] pattern binder.  That is, after executing
+~c[(include-book \"centaur/misc/hons-extra.lisp\" :dir :system)] you may write
+something like this:
+
+~bv[]
+ (b* (...
+      ((with-fast a b c ...))
+      ...)
+   ...)
+~ev[]
+
+which causes ~c[a], ~c[b], and ~c[c] to become fast alists until the completion
+of the ~c[b*] form.
+
+Note that ~c[with-fast-alist] will cause logically tail-recursive functions not
+to execute tail-recursively if its cleanup phase happens after the
+tail-recursive call returns.~/~/")
+
+(defdoc with-stolen-alist
+  ":Doc-Section Hons-and-Memoization
+
+~c[(with-stolen-alist name form)] ensures that ~c[name] is a fast alist at the
+start of the execution of ~c[form].  At the end of execution, it ensures that
+~c[name] is a fast alist if and only if it was originally.  That is, if
+~c[name] was not a fast alist originally, its hash table link is freed, and if
+it was a fast alist originally but its table was modified during the execution
+of ~c[form], that table is restored.  Note that any extended table created from
+the original fast alist during ~c[form] must be manually freed.~/
+
+Logically, ~c[with-stolen-alist] just returns ~c[form].
+
+Under the hood, we cause ~c[alist] to become a fast alist before executing
+~c[form], and we check the various conditions outlined above before returning
+the final value.
+
+Note that ~c[with-stolen-alist] will cause logically tail-recursive functions
+not to execute tail-recursively if its cleanup phase happens after the
+tail-recursive call returns.~/~/")
+
+(defdoc fast-alist-free-on-exit
+  ":Doc-Section Hons-and-Memoization
+Free a fast alist after the completion of some form.~/
+
+Logically, ~c[(fast-alist-free-on-exit name form)] is the identity and returns
+~c[form].
+
+Under the hood, this essentially expands to:
+~bv[]
+ (prog1 form
+        (fast-alist-free name))
+~ev[]
+
+In other words, ~c[name] is not freed immediately, but instead remains a fast
+alist until the form completes.  This may be useful when you are writing code
+that uses a fast alist but has many return points.
+
+See also the macro ~c[fast-alists-free-on-exit] defined in the community book
+~c[\"books/centaur/misc/hons-extra.lisp\"], which can be used to free several
+alists.
+
+The community book ~c[\"books/centaur/misc/hons-extra.lisp\"] extends the ~c[b*] macro
+(defined in the community book ~c[\"books/tools/bstar.lisp\"]) with the
+~c[free-on-exit] pattern binder.  That is, after executing
+~c[(include-book \"centaur/misc/hons-extra.lisp\" :dir :system)], the form
+
+~bv[]
+ (b* (...
+      ((free-on-exit a b c))
+      ...)
+   ...)
+~ev[]
+
+causes ~c[a], ~c[b], and ~c[c] to be freed when the ~c[b*] completes, but they
+remain fast alists until then.~/~/")
+
+#+(or acl2-loop-only (not hons))
+(defmacro with-fast-alist-raw (alist form)
+  ;; Has an under-the-hood implementation
+  (declare (ignore alist))
+  form)
+
+(defmacro with-fast-alist (alist form)
+  `(return-last 'with-fast-alist-raw ,alist ,form))
+
+#+(or acl2-loop-only (not hons))
+(defmacro with-stolen-alist-raw (alist form)
+  ;; Has an under-the-hood implementation
+  (declare (ignore alist))
+  form)
+
+(defmacro with-stolen-alist (alist form)
+  `(return-last 'with-stolen-alist-raw ,alist ,form))
+
+#+(or acl2-loop-only (not hons))
+(defmacro fast-alist-free-on-exit-raw (alist form)
+  ;; Has an under-the-hood implementation
+  (declare (ignore alist))
+  form)
+
+(defmacro fast-alist-free-on-exit (alist form)
+  `(return-last 'fast-alist-free-on-exit-raw ,alist ,form))
+
 (defn cons-subtrees (x al)
   ":Doc-Section Hons-and-Memoization
 
@@ -898,7 +1131,7 @@ deprecated feature~/
 This ~il[documentation] topic relates to the experimental extension of ACL2
 supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
 
-Deprecated.  Alias for ~ilc[fast-alist-free]~/~/"
+Deprecated.  Alias for ~ilc[fast-alist-free].~/~/"
 
   (fast-alist-free alist))
 
@@ -933,5 +1166,5 @@ Deprecated.  Alias for ~ilc[fast-alist-free]~/~/"
             (:executable-counterpart flush-hons-get-hash-table-link)
             ))
 
-; For some additional helper functions and lemmas, see the files
+; For some additional helper functions and lemmas, see the community books
 ; books/misc/hons-help.lisp and books/misc/hons-help2.lisp.
