@@ -1,23 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
-; (C) 1997 Computational Logic, Inc.  See the documentation topic
-; NOTE-2-0.
+; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -37,13 +30,22 @@
 
 (defmacro defn (f a &rest r)
 
-  ":Doc-Section Programming
+  ":Doc-Section Events
    definition with ~il[guard] ~c[t]~/
 
    ~c[Defn] is ~ilc[defun] with ~il[guard] ~c[t].~/~/"
 
   `(defun ,f ,a (declare (xargs :guard t)) ,@r))
 
+(defmacro defnd (f a &rest r)
+
+  ":Doc-Section Events
+   ~il[disable]d definition with ~il[guard] ~c[t]~/
+
+   ~c[Defnd] is ~ilc[defund] with ~il[guard] ~c[t].~/~/"
+
+  `(defund ,f ,a (declare (xargs :guard t)) ,@r))
+
 (defdoc hons-and-memoization
   ":Doc-Section Hons-and-Memoization
 
@@ -175,42 +177,31 @@
   will execute without error.
   ~bv[]
   ACL2 !>(time$ (fib 40))
-
-  ... took 2,641 milliseconds (2.641 seconds) to run with 8 available
-  CPU cores.  During that period, 2,644 milliseconds (2.644 seconds)
-  were spent in user mode 0 milliseconds (0.000 seconds) were spent in
-  system mode 16 bytes of memory allocated.  102334155 ACL2 !>(memoize
-  'fib)
+  ; (EV-REC *RETURN-LAST-ARG3* ...) took 
+  ; 0.99 seconds realtime, 0.98 seconds runtime
+  ; (1,296 bytes allocated).
+  102334155
+  ACL2 !>(memoize 'fib)
 
   Summary
   Form:  ( TABLE MEMOIZE-TABLE ...)
   Rules: NIL
-  Warnings:  None
-  Time:  0.04 seconds (prove: 0.00, print: 0.00, other: 0.04)
+  Time:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)
 
   Summary
   Form:  ( PROGN (TABLE MEMOIZE-TABLE ...) ...)
   Rules: NIL
-  Warnings:  None
-  Time:  0.04 seconds (prove: 0.00, print: 0.00, other: 0.04)
+  Time:  0.01 seconds (prove: 0.00, print: 0.00, other: 0.01)
    FIB
   ACL2 !>(time$ (fib 40))
-
-  ... took 19 milliseconds (0.019 seconds) to run with 8 available CPU
-  cores.  During that period, 20 milliseconds (0.020 seconds) were
-  spent in user mode 0 milliseconds (0.000 seconds) were spent in
-  system mode
-   539,088 bytes of memory allocated.
-   193 minor page faults, 0 major page faults, 0 swaps.
+  ; (EV-REC *RETURN-LAST-ARG3* ...) took 
+  ; 0.00 seconds realtime, 0.00 seconds runtime
+  ; (2,864 bytes allocated).
   102334155
   ACL2 !>(time$ (fib 100))
-
-  ... took 0 milliseconds (0.000 seconds) to run with 8 available CPU
-  cores.  During that period, 0 milliseconds (0.000 seconds) were
-  spent in user mode 0 milliseconds (0.000 seconds) were spent in
-  system mode
-   5,760 bytes of memory allocated.
-   3 minor page faults, 0 major page faults, 0 swaps.
+  ; (EV-REC *RETURN-LAST-ARG3* ...) took 
+  ; 0.00 seconds realtime, 0.00 seconds runtime
+  ; (7,024 bytes allocated).
   354224848179261915075
   ACL2 !>(unmemoize 'fib)
   ~ev[]
@@ -305,19 +296,19 @@
 
   We next discuss the fast lookup operation for association lists.
   When a pair is added to an association list using the functions
-  ~c[hons-acons] or ~c[hons-acons!], the system also records the
+  ~ilc[hons-acons] or ~ilc[hons-acons!], the system also records the
   key-value pair in an associated hash table.  As long as a user only
   used these two functions when placing key-value pairs on an
   association list, the key-value pairs in the corresponding hash
   table will be synchronized with the key-value pairs in the
-  association list.  Later, if ~c[hons-get] is used to look up a key,
+  association list.  Later, if ~ilc[hons-get] is used to look up a key,
   then instead of performing a linear search of the association list
   we consult the associated hash table.  If a user does not strictly
   follow this discipline, then a linear search may be required.  In
   some sense, these association lists are much like ACL2 arrays, but
   without the burden of explicitly naming the arrays.  The ACL2 array
   ~ilc[compress1] function is analogous to the functions
-  ~c[hons-shrink-alist] and ~c[hons-shrink-alist!].  There are
+  ~ilc[hons-shrink-alist] and ~ilc[hons-shrink-alist!].  There are
   user-level ACL2 functions that allow the associated hash tables to
   be cleared and removed.
 
@@ -342,6 +333,16 @@
      cd ccl
      svn update
 
+   Whether obtaining a fresh CCL or just updating, finally issue these
+   commands.
+
+     ./lx86cl64
+     (rebuild-ccl :full t)
+     (quit)
+     ./lx86cl64
+     (rebuild-ccl :full t)
+     (quit)
+
   ~ev[]
 
   ~sc[References]
@@ -375,10 +376,6 @@
   Volumne 218, 1968, pages 19-22.
   ~/")
 
-(defun hons-enabledp (state)
-  (declare (xargs :guard (state-p state)))
-  (global-val 'hons-enabled (w state)))
-
 #+(or acl2-loop-only (not hons))
 (defn clear-memoize-table (fn)
 
@@ -447,7 +444,7 @@
   This ~il[documentation] topic relates to the experimental extension of ACL2
   supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
 
-  This macro is abbreviation for ~ilc[memoize-summary].  Logically, it just
+  This macro is an abbreviation for ~ilc[memoize-summary].  Logically, it just
   returns ~c[nil].~/~/"
 
   '(memoize-summary))
@@ -508,133 +505,133 @@
                         trace commutative forget memo-table-init-size aokp
                         ideal-okp)
   (declare (xargs :guard t))
-  (cond
-   ((and condition-fn (null condition-p))
-    `(progn (table memoize-table
-                   (deref-macro-name ,fn (macro-aliases world))
-                   (list* (cons :condition-fn ,condition-fn)
-                          (cons :inline ,inline)
-                          (cons :trace ,trace)
-                          (cons :commutative ,commutative)
-                          (cons :forget ,forget)
-                          (cons :memo-table-init-size
-                                ,(or memo-table-init-size
-                                     *mht-default-size*))
-                          (cons :aokp ,aokp)
-                          (and (not (eq ,ideal-okp :default))
-                               (list (cons :ideal-okp ,ideal-okp)))))
-            (value-triple (deref-macro-name
-                           ,fn
-                           (macro-aliases (w state))))))
-   ((and condition-p
-         (not (eq condition t))
-         (not (equal condition ''t)))
-    `(make-event
-      (let* ((wrld (w state))
-             (fn (deref-macro-name ,fn (macro-aliases wrld)))
-             (condition ,condition)
-             (formals
-              (and (symbolp fn) ; guard for getprop
-                   (getprop fn 'formals t
-                            'current-acl2-world wrld)))
-             (condition-fn (or ,condition-fn
-                               (intern-in-package-of-symbol
-                                (concatenate
-                                 'string
-                                 (symbol-name fn)
-                                 "-MEMOIZE-CONDITION")
-                                fn)))
-             (hints ,hints)
-             (otf-flg ,otf-flg)
-             (inline ,inline)
-             (trace ,trace)
-             (commutative ,commutative)
-             (forget ,forget)
-             (memo-table-init-size ,memo-table-init-size)
-             (aokp ,aokp)
-             (ideal-okp ,ideal-okp))
-        (cond ((not (and
-                     (symbolp fn)
-                     (not (eq t formals))
-                     (not (eq t (getprop fn 'stobjs-in t
-                                 'current-acl2-world wrld)))
-                     (not (eq t (getprop fn 'stobjs-out t
+  (let ((condition (cond ((equal condition ''t) t)
+                         ((equal condition ''nil) nil)
+                         (t condition))))
+    (cond
+     ((and condition-fn (null condition-p))
+      `(progn (table memoize-table
+                     (deref-macro-name ,fn (macro-aliases world))
+                     (list* (cons :condition-fn ,condition-fn)
+                            (cons :inline ,inline)
+                            (cons :trace ,trace)
+                            (cons :commutative ,commutative)
+                            (cons :forget ,forget)
+                            (cons :memo-table-init-size
+                                  ,(or memo-table-init-size
+                                       *mht-default-size*))
+                            (cons :aokp ,aokp)
+                            (and (not (eq ,ideal-okp :default))
+                                 (list (cons :ideal-okp ,ideal-okp)))))
+              (value-triple (deref-macro-name
+                             ,fn
+                             (macro-aliases (w state))))))
+     ((and condition-p
+           (not (eq condition t))
+           (not (eq condition nil)))
+      `(make-event
+        (let* ((wrld (w state))
+               (fn (deref-macro-name ,fn (macro-aliases wrld)))
+               (condition ,condition)
+               (formals
+                (and (symbolp fn) ; guard for getprop
+                     (getprop fn 'formals t
+                              'current-acl2-world wrld)))
+               (stobjs-in (getprop fn 'stobjs-in t
+                                   'current-acl2-world wrld))
+               (condition-fn (or ,condition-fn
+                                 (intern-in-package-of-symbol
+                                  (concatenate
+                                   'string
+                                   (symbol-name fn)
+                                   "-MEMOIZE-CONDITION")
+                                  fn)))
+               (hints ,hints)
+               (otf-flg ,otf-flg)
+               (inline ,inline)
+               (trace ,trace)
+               (commutative ,commutative)
+               (forget ,forget)
+               (memo-table-init-size ,memo-table-init-size)
+               (aokp ,aokp)
+               (ideal-okp ,ideal-okp))
+          (cond ((not (and
+                       (symbolp fn)
+                       (not (eq t formals))
+                       (not (eq t stobjs-in))
+                       (not (eq t (getprop fn 'stobjs-out t
 
 ; Normally we would avoid getting the stobjs-out of return-last.  But
 ; return-last will eventually be rejected for mamoization anyhow (by
 ; memoize-table-chk).
 
-                                 'current-acl2-world wrld)))
-                     (cltl-def-from-name fn nil wrld)))
-               (er hard 'memoize
-                   "The symbol ~x0 is not a known function symbol, and thus ~
-                    it cannot be memoized."
-                   fn))
-              ((not (eq :common-lisp-compliant
-                        (symbol-class fn wrld)))
-               (er hard 'memoize
-                   "~x0 is not Common Lisp compliant, so is best memoized and ~
-                    called from raw Lisp (but raw Lisp should be avoiding ~
-                    unless you are hacking)."
-                   fn))
+                                           'current-acl2-world wrld)))
+                       (cltl-def-from-name fn wrld)))
+                 (er hard 'memoize
+                     "The symbol ~x0 is not a known function symbol, and thus ~
+                      it cannot be memoized."
+                     fn))
 
 ; Certify-book seems to do things twice, so the following is commented out.
 
 ;             ((cdr (assoc-eq fn (table-alist 'memoize-table wrld)))
 ;              (er hard 'memoize "~x0 is already memoized." fn))
 
-              ((not (member-eq inline '(t nil)))
-               (er hard 'memoize
-                   "The value ~x0 for inline is illegal (must be ~x1 or ~x2)."
-                   inline t nil))
-              ((not (member-eq trace '(t nil)))
-               (er hard 'memoize
-                   "The value ~x0 for trace is illegal (must be ~x1 or ~x2)."
-                   trace t nil))
-              (t
-               `(progn
-                  (defun ,condition-fn ,formals
-                    (declare
-                     (ignorable ,@formals)
-                     (xargs :guard
-                            ,(getprop fn 'guard *t*
-                                      'current-acl2-world wrld)
-                            :verify-guards nil))
-                    ,condition)
-                  (verify-guards ,condition-fn
-                                 ,@(and hints `(:hints ,hints))
-                                 ,@(and otf-flg
-                                        `(:otf-flg ,otf-flg)))
-                  (table memoize-table
-                         ',fn
-                         (list* (cons :condition-fn ',condition-fn)
-                                (cons :inline ',inline)
-                                (cons :trace ',trace)
-                                (cons :commutative ',commutative)
-                                (cons :forget ',forget)
-                                (cons :memo-table-init-size
-                                      (or ,memo-table-init-size
-                                          *mht-default-size*))
-                                (cons :aokp ',aokp)
-                                (and (not (eq ',ideal-okp :default))
-                                     (list (cons :ideal-okp ',ideal-okp)))))
-                  (value-triple ',fn)))))))
-   (t `(progn (table memoize-table
-                     (deref-macro-name ,fn (macro-aliases world))
-                     (list* (cons :condition-fn t)
-                            (cons :inline ,inline)
-                            (cons :trace ,trace)
-                            (cons :commutative ,commutative)
-                            (cons :forget ,forget)
-                            (cons :memo-table-init-size
-                                  (or ,memo-table-init-size
-                                      *mht-default-size*))
-                            (cons :aokp ',aokp)
-                            (and (not (eq ',ideal-okp :default))
-                                 (list (cons :ideal-okp ',ideal-okp)))))
-              (value-triple (deref-macro-name
-                             ,fn
-                             (macro-aliases (w state))))))))
+                ((not (member-eq inline '(t nil)))
+                 (er hard 'memoize
+                     "The value ~x0 for inline is illegal (must be ~x1 or ~x2)."
+                     inline t nil))
+                ((not (member-eq trace '(t nil)))
+                 (er hard 'memoize
+                     "The value ~x0 for trace is illegal (must be ~x1 or ~x2)."
+                     trace t nil))
+                (t
+                 `(progn
+                    (defun ,condition-fn ,formals
+                      (declare
+                       (ignorable ,@formals)
+                       (xargs :guard
+                              ,(getprop fn 'guard *t*
+                                        'current-acl2-world wrld)
+                              :verify-guards nil
+                              ,@(let ((stobjs (remove nil stobjs-in)))
+                                  (and stobjs
+                                       `(:stobjs ,stobjs)))))
+                      ,condition)
+                    (verify-guards ,condition-fn
+                                   ,@(and hints `(:hints ,hints))
+                                   ,@(and otf-flg
+                                          `(:otf-flg ,otf-flg)))
+                    (table memoize-table
+                           ',fn
+                           (list* (cons :condition-fn ',condition-fn)
+                                  (cons :inline ',inline)
+                                  (cons :trace ',trace)
+                                  (cons :commutative ',commutative)
+                                  (cons :forget ',forget)
+                                  (cons :memo-table-init-size
+                                        (or ,memo-table-init-size
+                                            *mht-default-size*))
+                                  (cons :aokp ',aokp)
+                                  (and (not (eq ',ideal-okp :default))
+                                       (list (cons :ideal-okp ',ideal-okp)))))
+                    (value-triple ',fn)))))))
+     (t `(progn (table memoize-table
+                       (deref-macro-name ,fn (macro-aliases world))
+                       (list* (cons :condition-fn ,condition) ; t or nil
+                              (cons :inline ,inline)
+                              (cons :trace ,trace)
+                              (cons :commutative ,commutative)
+                              (cons :forget ,forget)
+                              (cons :memo-table-init-size
+                                    (or ,memo-table-init-size
+                                        *mht-default-size*))
+                              (cons :aokp ',aokp)
+                              (and (not (eq ',ideal-okp :default))
+                                   (list (cons :ideal-okp ',ideal-okp)))))
+                (value-triple (deref-macro-name
+                               ,fn
+                               (macro-aliases (w state)))))))))
 
 (defmacro memoize (fn &key
                       (condition 't condition-p)
@@ -658,9 +655,14 @@
 ; that we do exactly what we would normally do for condition except
 ; that we use the name condition-fn.
 
+; Parallelism blemish: when waterfall parallelism is enabled (detected by
+; seeing whether ACL2 global 'waterfall-parallelism is non-nil), memoize and
+; unmemoize should be changed to modify the 'saved-memoize-table instead of
+; 'memoize-table.
+
   ":Doc-Section Events
 
-  turn on memoization for one function~/
+  turn on memoization for a specified function~/
 
   This ~il[documentation] topic relates to an experimental extension of ACL2
   under development by Bob Boyer and Warren Hunt.  ~l[hons-and-memoization] for
@@ -702,12 +704,13 @@
            )
   ~ev[]
   where ~c[fn] evaluates to a user-defined function symbol; ~c[condition] is
-  either ~c[t] (the default) or ~c['t] or else evaluates to an expression whose
-  free variables are among the formal parameters of ~c[fn]; and
-  ~c[condition-fn] is either ~c[nil] (the default) or else evaluates to a legal
-  function symbol.  Further restrictions and options are discussed below.  Note
-  that all arguments are evaluated (but for the special handling of value ~c[t]
-  for ~c[:commutative], the argument must literally be ~c[t]; see below).
+  either ~c[t] (the default), ~c['t], ~c[nil], or ~c['nil], or else evaluates
+  to an expression whose free variables are among the formal parameters of
+  ~c[fn]; and ~c[condition-fn] is either ~c[nil] (the default) or else
+  evaluates to a legal function symbol.  Further restrictions and options are
+  discussed below.  Note that all arguments are evaluated (but for the special
+  handling of value ~c[t] for ~c[:commutative], the argument must literally be
+  ~c[t]; see below).
 
   Generally ~c[fn] must evaluate to a defined function symbol.  However, this
   value can be the name of a macro that is associated with such a function
@@ -735,7 +738,11 @@
   on the same parameters, and the lookup or table store described above are
   only performed if the result from the condition function call is non-~c[nil].
 
-  If however ~c[:condition] is supplied, then an attempt will be made to define
+  Suppose however that ~c[:condition] is supplied.  If the value supplied is
+  ~c[t] or ~c['t], then the lookup and table store described above are always
+  done.  If the value is ~c[nil] or ~c['nil], then this lookup and table store
+  are never done, although statistics may be gathered; ~pl[profile].  Now
+  consider other values for ~c[:condition].  An attempt will be made to define
   a condition function whose ~il[guard] and formal parameters list are the same
   as those of the memoized function, and whose body is the result, ~c[r], of
   evaluating the given ~c[condition].  The name of that condition function is
@@ -753,7 +760,7 @@
 
   Note that ~c[fn] can be either a ~c[:]~ilc[logic] mode function or a
   ~c[:]~ilc[program] mode function.  However, only the corresponding raw Lisp
-  function is actually memoized, so ~il[guard] violatations can defeat
+  function is actually memoized, so ~il[guard] violations can defeat
   memoization, and ~c[:]~ilc[logic] mode functions without their ~il[guard]s
   verified will only be memoized when called by ~c[:]~ilc[program] mode
   functions.  (~l[guards-and-evaluation] for more information about guards and
@@ -773,8 +780,8 @@
   illegal to convert the function from ~c[:]~ilc[program] to ~c[:]~ilc[logic]
   mode (~pl[verify-termination]).  To turn off memoization, ~pl[unmemoize].
 
-  ~c[Memoize] is illegal for a function whose arguments include ~c[state] or,
-  more generally, any ~il[stobj].  Also, ~c[memoize] never allows attachments
+  ~c[Memoize] is illegal for a function if its arguments include ~c[state] or
+  if it returns any ~il[stobj]s.  Also, ~c[memoize] never allows attachments
   to be used (~pl[defattach]); if an attachment is used during evaluation, then
   the evaluation result will not be stored.
 
@@ -862,7 +869,8 @@
   ~c[nil], then it must be ~c[t], which causes all memoization done for a
   top-level call of ~c[fn] to be forgotten when that top-level call exits.~/
 
-  :cite hons-and-memoization"
+  :cite hons-and-memoization
+  :cited-by hons-and-memoization"
 
   (declare (xargs :guard t)
            (ignorable condition-p condition condition-fn hints otf-flg inline
@@ -957,7 +965,15 @@
 
   Calls of this macro generate events of the form
   ~c[(table memoize-table fn nil)].  When successful, the returned
-  value is of the form ~c[(mv nil function-symbol state)]."
+  value is of the form ~c[(mv nil function-symbol state)].
+
+  To remove the effects of all ~ilc[memoize] ~il[events], evaluate:
+  ~c[(clear-memo-table)].  To save and restore memoization,
+  ~pl[save-and-clear-memoization-settings] and
+  ~pl[restore-memoization-settings].~/
+
+  :cite hons-and-memoization
+  :cited-by hons-and-memoization"
 
   (declare (xargs :guard t))
   #-acl2-loop-only
@@ -967,9 +983,8 @@
 ; example).
 
             (warning$ 'unmemoize nil
-                      "No change for function ~x0: Unmemoization ~
-                       requests are ignored in raw Lisp.  In raw ~
-                       Lisp, use unmemoize-fn."
+                      "No change for function ~x0: Unmemoization requests are ~
+                       ignored in raw Lisp.  In raw Lisp, use unmemoize-fn."
                       ',fn))
           (value-triple nil))
   #+acl2-loop-only
@@ -989,7 +1004,7 @@
 
   NOTE: An alternative to this utility, which has a lot less functionality but
   doesn't depend on the experimental extension of ACL2 mentioned just below,
-  may be found in distributed book ~c[books/misc/profiling.lisp].
+  may be found in community book ~c[books/misc/profiling.lisp].
 
   This documentation topic relates to an experimental extension of ACL2 under
   development by Bob Boyer and Warren Hunt.  ~l[hons-and-memoization] for a
@@ -1018,7 +1033,7 @@
   ~c[Profile] is just a macro that calls ~ilc[memoize] to do its work.
   ~c[Profile] gives the two keyword parameters ~c[:CONDITION] and ~c[:INLINE]
   of ~ilc[memoize] the value ~c[nil].  Other keyword parameters for
-  ~c[memoize], which must not include ~c[:CONDITION], ~c[:CONDITION-fn], or
+  ~c[memoize], which must not include ~c[:CONDITION], ~c[:CONDITION-FN], or
   ~c[:INLINE], are passed through.  To eliminate profiling, use
   ~ilc[unmemoize]; for example, to eliminate profiling for function ~c[fn],
   evaluate ~c[(unmemoize 'fn)].
@@ -1040,7 +1055,7 @@
 
 (defmacro memoize-on (fn form)
 
-; MEMOIZE-ON evaluates x.  During the evaluation the symbol fn has as
+; MEMOIZE-ON evaluates form.  During the evaluation the symbol fn has as
 ; its symbol-function what it had immediately AFTER the memoization of
 ; fn.  Hence, the values of calls of fn may be remembered during the
 ; evaluation and later.  Warning: to use MEMOIZE-ON, fn must already
@@ -1055,7 +1070,7 @@
 
 (defmacro memoize-off (fn form)
 
-; MEMOIZE-OFF evaluates x.  During the evaluation the symbol fn has as
+; MEMOIZE-OFF evaluates form.  During the evaluation the symbol fn has as
 ; its symbol-function what it had immediately BEFORE the memoization
 ; of fn.  Hence the values of calls of fn may not be remembered during
 ; the evaluation.  Warning: to use MEMOIZE-OFF, fn must already be
@@ -1112,14 +1127,10 @@
 ; NIL
 ; ? 
 
-(defconst *untroublesome-characters*
+; Some centaur/ books put entries in *never-profile-ht*.  In order to allow
+; those books to certify in vanilla ACL2, we define a default value for that
+; variable here.
 
-  (coerce "~/:{}=+-_.,!?@#%^%<>|&$*`\\/\";()" 'list)
-
-  ":Doc-Section Hons-and-Memoization
-
-  some not so weird characters~/
-
-  ~c[*UNTROUBLESOME-CHARACTERS*] should not include single quote, whitespace
-  characters, control characters, or anything else weird for a file name.  Such
-  characters should only be used in file names by experts at quoting.~/~/")
+#+(and (not hons) (not acl2-loop-only))
+(defparameter *never-profile-ht*
+  (make-hash-table :test 'eq))
