@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -42,6 +36,22 @@
 #+(and (not sbcl) sb-thread)
 (error "Feature sb-thread not supported on Lisps other than SBCL")
 
+; We would like to use the semaphore notification objects supported in SBCL
+; 1.0.54+, but version 1.0.54 has a heap exhaustion error that we wish to
+; avoid.  As such, for now, we remain compatible with 1.0.53 and below.  While
+; we can consider uncommenting the following feature push once SBCL's support
+; for semaphore notification objects is stable, the following questions seem
+; difficult to answer: How can we determine when that is the case, and how
+; confident are we that Rager's implementation using semaphore notification
+; objects is correct?  Rager determined that 1.0.54 was unstable by making all
+; of the books with parallelism enabled (and experiencing the "heap exhausted"
+; error in one of the ordinal books [at the moment of this writing, he does not
+; recall which one]).  Thus, Rager's code has not been completely tested using
+; semaphore notification objects.
+
+; #+sbcl
+; (push :sbcl-sno-patch *features*)
+
 ;---------------------------------------------------------------------
 ; Section:  Enabling and Disabling Interrupts
 
@@ -57,11 +67,11 @@
 
 (defmacro without-interrupts (&rest forms)
 
-; This macro prevents interrupting evaluation of any of the indicated forms in
-; a parallel lisp.  In a non-parallel environment (#-(or ccl sb-thread)),
-; we simply evaluate the forms.  This behavior takes priority over any
-; enclosing call of with-interrupts.  Since we do not have a good use case for
-; providing with-interrupts, we omit it from this interface.
+; This macro prevents interrupting evaluation (including throws, e.g., as done
+; in support of por) of any of the indicated forms in a parallel lisp.  In a
+; non-parallel environment, we simply evaluate the forms.  This behavior takes
+; priority over any enclosing call of with-interrupts.  Since we do not have a
+; good use case for providing with-interrupts, we omit it from this interface.
 
   #+ccl
   `(ccl:without-interrupts ,@forms)
@@ -172,13 +182,23 @@
   #+ccl
   `(ccl::atomic-incf ,x)
   #+sb-thread
+
+; Parallelism blemish: we (David Rager) performed experiments in either 2009 or
+; 2010 that indicated that there was a problem with using SBCL's built-in
+; atomic increment and decrements.  Because of this observation, we roll our
+; own atomic increments and decrements based on locking.  For future reference,
+; we leave the version of the code that used SBCL's built-in primitives as a
+; comment.
+
 ;  `(progn ; (sb-debug:backtrace) 
 ;          (1+ (sb-ext:atomic-incf
 ;               (atomically-modifiable-counter-val ,x))))
 
-; Parallelism wart: why is nil used in the call to with-recursive-lock?
+; Parallelism blemish: we do not recall why nil appears in the call to
+; with-recursive-lock below.  Probably it can be omitted.
 
-  `(sb-thread:with-recursive-lock ((atomically-modifiable-counter-lock ,x)) nil
+  `(sb-thread:with-recursive-lock ((atomically-modifiable-counter-lock ,x))
+                                  nil ; see comment above
                                   (incf (atomically-modifiable-counter-val ,x)))
   #+lispworks
   `(system:atomic-incf ,x)
@@ -196,7 +216,13 @@
 ; documentation, Lispworks returns the new value.
 
   #+ccl
-  `(without-interrupts (dotimes (i ,count) (ccl::atomic-incf ,counter)))
+  `(without-interrupts
+
+; Admittedly, the following loop can be viewed as not being "atomic".  But the
+; important thing is that two threads don't increment from the same value, and
+; the use of atomic-incf guarantees that.
+
+    (dotimes (i ,count) (ccl::atomic-incf ,counter)))
   #+sb-thread
 ;  `(+ (sb-ext:atomic-incf 
 ;       (atomically-modifiable-counter-val ,counter) ,count)
@@ -248,6 +274,8 @@
 
 ; See also deflock.
 
+; If lock-name is supplied, it must be nil or a string.
+
 ; Even though CCL nearly always uses a FIFO for threads blocking on a lock,
 ; it does not guarantee so: no such promise is made by the CCL
 ; documentation or implementor (in fact, we are aware of a race condition that
@@ -271,31 +299,6 @@
 
   nil)
 
-(defmacro deflock (lock-symbol)
-
-; Deflock defines what some Lisps call a "recursive lock", namely a lock that
-; can be grabbed more than once by the same thread, but such that if a thread
-; outside the owner tries to grab it, that thread will block.  In addition to
-; defining a lock, this macro also defines a macro that uses the lock to
-; provide mutual-exclusion for a given list of operations.  This macro has the
-; name with-<modified-lock-name>, where <modified-lock-name> is the given
-; lock-symbol without the leading and trailing * characters.
-
-; Note that if lock-symbol is already bound, then deflock will not re-bind
-; lock-symbol.
-
-  (let* ((name (symbol-name lock-symbol))
-         (macro-symbol (intern 
-                        (concatenate 'string
-                                     "WITH-"
-                                     (subseq name 1 (1- (length name))))
-                        "ACL2")))
-    `(progn
-       (defvar ,lock-symbol
-         (make-lock (symbol-name ',lock-symbol)))
-       (defmacro ,macro-symbol (&rest args)
-         (list* 'with-lock ',lock-symbol args)))))
-
 (defmacro reset-lock (bound-symbol)
 
 ; This macro binds the given global (but not necessarily special) variable to a
@@ -437,6 +440,8 @@
 ; sb-sys:with-deadline, but we figure the simplicity is more valuable than
 ; randomly passing in a :timeout value.
 
+; Note that if there is a timeout, the return value is unspecified.
+
   #+sb-thread
   `(if ,timeout
        (handler-case
@@ -541,12 +546,7 @@
   #+lispworks
   (mp:make-condition-variable)
   #-(or ccl sb-thread lispworks)
-
-; We may wish to have assertions that evaluation of (make-condition-variable)
-; is non-nil.  So we return t, even though as of this writing there are no such
-; assertions.
-
-  t)
+  nil)
 
 (defmacro signal-condition-variable (cv)
   #-(or ccl sb-thread lispworks)
@@ -564,8 +564,18 @@
   `(sb-thread:condition-notify ,cv)
   #+lispworks
   `(mp:condition-variable-signal ,cv)
-  #-(or ccl sb-thread lispworks)
-  t)
+
+; Parallelism wart: we would like for signal-condition-variable to return nil
+; to signify that this macro is only evaluated for side-effects.  However, it
+; appears that this macro's return value is actually needed, because the
+; theorem prover hangs consistently in SBCL and LispWorks when we do return nil
+; here.  As such, we currently remove the returning of nil.  But, this is a
+; hotfix.  A much better solution is to change the reliance upon this macro's
+; return value and then uncomment the following nil.
+
+; nil
+
+)
 
 (defmacro broadcast-condition-variable (cv)
   #-(or sb-thread lispworks)
@@ -576,28 +586,42 @@
   `(sb-thread:condition-broadcast ,cv)
   #+lispworks
   `(mp:condition-variable-broadcast ,cv)
-  #-(or ccl sb-thread lispworks)
-  t)
+
+  nil)
 
 (defun wait-on-condition-variable (cv lock &key timeout)
 
+; This is only executed for side-effect.
+
 ; A precondition to this function is that the current thread "owns" lock.  This
 ; is a well-known part of how condition variables work.  This is also
 ; documented in the SBCL manual in section 12.5 entitled "Waitqueue/condition
 ; variables."
 
+; Parallelism blemish: when timeout is provided, the return value of t can be
+; misleading.  This isn't a huge problem, because condition variables are only
+; used for signaling, and so, assuming the programmer is following the
+; discipline associated with using condition variables, any thread waiting on a
+; condition variable should check the state of the program for the property it
+; needs anyway.  And, if that property is not provided, the thread will just
+; rewait on the condition variable.  So, the only real penalty should be a
+; slight loss of performance.  Of course, this small penalty depends on the
+; following the discipline of using condition variables correctly.  We may not
+; wish to rely upon the programmer's ability to follow this discipline, so we
+; leave it as a parallelism blemish.
+
   #-(or sb-thread lispworks)
   (declare (ignore cv lock timeout))
   #+ccl
-  (error "Waiting on condition variables with locks is unsupported in CCL")
+  (error "Waiting on a condition variable is unsupported in CCL")
   #+sb-thread
   (with-potential-timeout
    (sb-thread:condition-wait cv lock)
    :timeout timeout)
   #+lispworks
   (mp:condition-variable-wait cv lock :timeout timeout)
-  #-(or ccl sb-thread lispworks)
-  nil) ; the default is to never receive a signal
+
+  t)
 
 #+sb-thread
 (defstruct acl2-semaphore
@@ -657,8 +681,10 @@
   (declare (ignore name))
   #+ccl
   (ccl:make-semaphore)
-  #+(or sb-thread lispworks)
+  #+(or (and sb-thread (not sbcl-sno-patch)) lispworks)
   (make-acl2-semaphore)
+  #+(and sb-thread sbcl-sno-patch)
+  (sb-thread:make-semaphore)
   #-(or ccl sb-thread lispworks)
 
 ; We return nil in the uni-threaded case in order to stay in sync with
@@ -701,8 +727,10 @@
 
   #+ccl
   (ccl:make-semaphore-notification)
-  #+(or sb-thread lispworks)
+  #+(or (and sb-thread (not sbcl-sno-patch)) lispworks)
   (make-array 1 :initial-element nil)
+  #+(and sb-thread sbcl-sno-patch)
+  (sb-thread:make-semaphore-notification)
   #-(or ccl sb-thread lispworks)
   nil)
 
@@ -711,8 +739,10 @@
   (declare (ignore semaphore-notification-object))
   #+ccl
   (ccl:semaphore-notification-status semaphore-notification-object)
-  #+(or sb-thread lispworks)
+  #+(or (and sb-thread (not sbcl-sno-patch)) lispworks)
   (aref semaphore-notification-object 0)
+  #+(and sb-thread sbcl-sno-patch)
+  (sb-thread:semaphore-notification-status semaphore-notification-object)
   #-(or ccl sb-thread lispworks)
 
 ; t may be the wrong default, but we don't have a use case for this return
@@ -726,8 +756,10 @@
   (declare (ignore semaphore-notification-object))
   #+ccl
   (ccl:clear-semaphore-notification-status semaphore-notification-object)
-  #+(or sb-thread lispworks)
+  #+(or (and sb-thread (not sbcl-sno-patch)) lispworks)
   (setf (aref semaphore-notification-object 0) nil)
+  #+(and sb-thread sbcl-sno-patch)
+  (sb-thread:clear-semaphore-notification semaphore-notification-object)
   #-(or ccl sb-thread lispworks)
   nil)
 
@@ -756,13 +788,15 @@
   (declare (ignore semaphore))
   #+ccl
   (ccl:signal-semaphore semaphore)
-  #+(or sb-thread lispworks)
+  #+(or (and sb-thread (not sbcl-sno-patch)) lispworks)
   (with-lock
    (acl2-semaphore-lock semaphore)
    (without-interrupts
     (incf (acl2-semaphore-count semaphore))
     (signal-condition-variable (acl2-semaphore-cv semaphore))))
-  #-(or ccl sb-thread lispworks)
+  #+(and sb-thread sbcl-sno-patch)
+  (sb-thread:signal-semaphore semaphore)
+
   nil)
 
 ; Once upon a time, we optimized the manual allocation and deallocation of
@@ -772,10 +806,16 @@
 
 (defun wait-on-semaphore (semaphore &key notification timeout)
 
-; This function is guaranteed to return t when it has received the signal.  Its
-; return value when the signal has not been received is unspecified.  As such,
-; we provide the semaphore notification object as a means for determining
-; whether a signal was actually received.
+; This function is guaranteed to return t when there is no timeout provided and
+; the function call received the signal.  Its return value when the signal has
+; not been received is unspecified.  As such, we provide the semaphore
+; notification object as a means for determining whether a signal was actually
+; received.
+
+; Parallelism blemish: if a timeout is included, then it is our intent that a
+; call of this function that times-out should return nil.  This is known to be
+; the case in CCL, but while we have written the code for SBCL and LispWorks,
+; we have not actually tested the code for this property.
 
 ; This function only returns normally after receiving a signal for the given
 ; semaphore, setting the notification status of notification (if supplied and
@@ -798,9 +838,10 @@
   #+ccl
   (if timeout
       (ccl:timed-wait-on-semaphore semaphore timeout notification)
-    (ccl:wait-on-semaphore semaphore notification))
+    (progn (ccl:wait-on-semaphore semaphore notification)
+           t))
 
-  #+(or sb-thread lispworks)
+  #+(or (and sb-thread (not sbcl-sno-patch)) lispworks)
   (let ((received-signal nil))
 
 ; If we did not use a variable like "received-signal", we could have the
@@ -864,7 +905,17 @@
 ; variable portion of the semaphore every time a timeout occurs.  However, that
 ; is the cost of a "live" implementation ("live" loosely means "makes progress").
 
-          (signal-condition-variable (acl2-semaphore-cv semaphore))))))
+        (signal-condition-variable (acl2-semaphore-cv semaphore)))))
+    (if timeout received-signal t))
+  #+(and sb-thread sbcl-sno-patch)
+  (if timeout
+      (sb-thread:wait-on-semaphore semaphore
+                                   :timeout timeout
+                                   :notification notification)
+    (progn (sb-thread:wait-on-semaphore semaphore
+                                        :timeout timeout
+                                        :notification notification)
+           t))
   #-(or ccl sb-thread lispworks)
   t) ; default is to receive a semaphore/lock
 
@@ -902,10 +953,39 @@
   (if (endp thread-list)
       nil
     (progn
+      #-sbcl
       (interrupt-thread
        (car thread-list)
        #'(lambda () (when *throwable-worker-thread*
                       (throw :worker-thread-no-longer-needed nil))))
+      #+sbcl
+      (handler-case
+       (interrupt-thread
+        (car thread-list)
+        #'(lambda () (when *throwable-worker-thread*
+                       (throw :worker-thread-no-longer-needed nil))))
+       (sb-thread:interrupt-thread-error
+        ()
+
+; Parallelism no-fix: it turns out that this error is common.  And since we
+; think that the error is caused by the thread being finished by the time we
+; get around to interrupting it, we do not print the following warning.  The
+; error message we are trying to avoid is:
+
+; debugger invoked on a SB-THREAD:INTERRUPT-THREAD-ERROR in thread #<THREAD
+;                                                                    "initial thread" RUNNING
+
+;                                                                    {100E761FB1}>:
+;   Interrupt thread failed: thread #<THREAD "Worker thread" FINISHED values: 17
+;                                     {100F1AD651}> has exited.
+
+;        (format t "Warning: We tried to interrupt a thread and throw it with ~%~
+;                   the :worker-thread-no-longer-needed tag, but we ~%~
+;                   encountered an error. Since the error is likely benign, ~%~
+;                   this is only a warning.  Unless you are an ACL2(p) ~%~
+;                   maintainer, you can ignore this warning.~%")
+
+        ))
       (throw-all-threads-in-list (cdr thread-list)))))
 
 (defun kill-all-threads-in-list (thread-list)
@@ -920,94 +1000,68 @@
       (kill-thread (car thread-list))
       (kill-all-threads-in-list (cdr thread-list)))))
 
-#+(or ccl lispworks)
-(defun initial-threads1 (threads)
+(defun thread-name (thread)
+  #+ccl
+  (ccl:process-name thread)
+  #+sbcl
+  (sb-thread:thread-name thread)
+  #+lispworks
+  (mp:process-name thread)
+  #-(or ccl sbcl lispworks)
+  (error "Function thread-name is unimplemented in this Lisp."))
+
+(defconstant *worker-thread-name* "Worker thread")
+
+(defun worker-threads1 (threads)
   (cond ((endp threads)
          nil)
-        (#+ccl
-         (member-equal (ccl:process-name (car threads))
-                       '("listener" "Initial"))
-         #+lispworks
-         (member-equal (mp:process-name (car threads))
-                       '("TTY Listener" "The idle process" 
-                         "Restart Function Process"))
+        ((equal (thread-name (car threads)) *worker-thread-name*)
          (cons (car threads)
-               (initial-threads1 (cdr threads))))
-        (t (initial-threads1 (cdr threads)))))
+               (worker-threads1 (cdr threads))))
+        (t (worker-threads1 (cdr threads)))))
 
-(defvar *initial-threads* 
+(defun worker-threads ()
 
-; *Intial-threads* stores a list of threads that are considered to be part of
-; the non-threaded part of ACL2.  When terminating parallelism threads, only
-; those not appearing in this list will be terminated.  Warning: If ACL2 uses
-; parallelism during the build process, this variable could incorrectly record
-; parallelism threads as initial threads.
+; Returns the subset of the current list of threads that are worker threads
+; spawned by ACL2(p).
 
-  (all-threads))
+  (worker-threads1 (all-threads)))
 
-(defun initial-threads ()
-
-; We know how to set the *initial-threads* variable reliably in all Lisps
-; except Lispworks.  Due to Lispworks' multiprocessing model (where we start a
-; tty-listener when lp exits), accurately updating this variable is difficult
-; (perhaps infeasiable).  Rather than spend even more time on this problem, we
-; simply return the threads that currently exist that match the names of
-; threads that we associate with "initial" threads.
-
-; Parallelism wart: we might want to adapt a similar name-based strategy for
-; SBCL.  This would allow us to get rid of *initial-threads* altogether.
-
-  #-(or ccl lispworks)
-  *initial-threads* 
-  #+(or ccl lispworks)
-  (initial-threads1 (all-threads)))
-
-(defun all-threads-except-initial-threads-are-dead ()
+(defun all-worker-threads-are-dead ()
   #+sbcl
   (<= (length (all-threads)) 1)
   #-sbcl
-  (null (set-difference (all-threads) (initial-threads))))
+  (null (worker-threads)))
 
-; Parallelism wart: consider deleting the following comment and associated
-; code.
+#+ccl
+(defun all-given-threads-are-reset (threads)
+  (if (endp threads)
+      t
+    (and (equal (ccl:process-whostate (car threads))
+                "Reset")
+         (all-given-threads-are-reset (cdr threads)))))
 
-; The below commented code (functions all-given-threads-are-reset and
-; all-threads-except-initial-threads-are-dead-or-reset) implements a scheme for
-; CCL that, for the most part, would prevent the need to call sleep inside
-; send-die-to-all-except-initial-threads.  That being said, the solution to
-; call sleep is simpler (and ports more easily to other Lisps), so we use that
-; solution instead.  We preserve this code as an example alternative to calling
-; sleep.
+(defun all-worker-threads-are-dead-or-reset ()
 
-;; (defun all-given-threads-are-reset (threads)
-;;   (if (endp threads)
-;;       t
-;;     (and (equal (ccl:process-whostate (car threads))
-;;                 "Reset")
-;;          (all-given-threads-are-reset (cdr threads)))))
+; CCL has a bug that if you try to throw a thread that is "reset" ("reset" is a
+; CCL term), it ignores the throw (and the thread never expires).  As such, we
+; do not let threads in the "reset" state prevent us from finishing the
+; resetting of parallelism variables.
 
-;; (defun all-threads-except-initial-threads-are-dead-or-reset ()
-;;   (format t "All-threads is ~s~%" (all-threads))
-;;   #+ccl
-;;   (all-given-threads-are-reset 
-;;    (set-difference (all-threads) (initial-threads)))
-;;   #+sbcl
-;;   (<= (length (all-threads)) 1)
-;;   #-(or ccl sbcl)
-;;   (null (set-difference (all-threads) (initial-threads))))
+  #+ccl
+  (all-given-threads-are-reset (worker-threads))
+  #-ccl
+  (null (worker-threads)))
 
-
-(defun send-die-to-all-except-initial-threads ()
+(defun send-die-to-worker-threads ()
 
 ; This function is evaluated only for side effect.
 
-  (let ((target-threads (set-difference (all-threads)
-                                        (initial-threads))))
-    (throw-all-threads-in-list target-threads)
-    (let ((round 0))
-      (when (not (all-threads-except-initial-threads-are-dead))
+  (throw-all-threads-in-list (worker-threads))
+  (let ((round 0))
+    (loop do
       
-; We used to call "(thread-wait 'all-threads-except-initial-threads-are-dead)".
+; We used to call "(thread-wait 'all-worker-threads-are-dead)".
 ; However, we noticed a synchronization problem between what we might prefer
 ; the underlying Lisp to do (in this one case) and what the Lisp actually does.
 ; In particular, when we call run-thread, the call to run-thread returns,
@@ -1018,19 +1072,25 @@
 ; this problem, we punt, and simply wait for one second before issuing a new
 ; throw.  In practice, this works just fine.
 
-        (when (equal (mod round 10) 0)
-          (throw-all-threads-in-list target-threads))
-        (sleep 0.1)
-        (incf round)))))
+          (when (equal (mod round 10) 0)
+            (throw-all-threads-in-list (worker-threads))
 
-(defun kill-all-except-initial-threads ()
+; The following commented code is only for debugging
+
+;              (format t "Waiting for parallelism worker threads to halt.~%")
+;              (format t "Current threads are ~%~s~%~%" (all-threads))
+
+            )
+          (sleep 0.03)
+          (incf round)
+          while (not (all-worker-threads-are-dead-or-reset)))))
+
+(defun kill-all-worker-threads ()
 
 ; This function is evaluated only for side effect.
 
-  (let ((target-threads (set-difference (all-threads)
-                                        (initial-threads))))
-    (kill-all-threads-in-list target-threads))
-  (thread-wait 'all-threads-except-initial-threads-are-dead))
+  (kill-all-threads-in-list (worker-threads))
+  (thread-wait 'all-worker-threads-are-dead))
 
 
 ;---------------------------------------------------------------------
@@ -1061,16 +1121,20 @@
     (or default 16)))
 
 (defvar *core-count*
+
+; *core-count* is a variable so that we can redefine it dynamically (for
+; performance testing) and not have to redefine everything that uses it.
+
   (core-count-raw)
   "The total number of CPU cores in the system.")
 
-(defconstant *unassigned-and-active-work-count-limit*
+(defvar *unassigned-and-active-work-count-limit*
 
 ; The *unassigned-and-active-work-count-limit* limits work on the *work-queue*
 ; to what we think the system will be able to process in a reasonable amount of
 ; time.  Suppose we have 8 CPU cores.  This means that there can be 8 active
 ; work consumers, and that generally not many more than 24 pieces of
-; paralellism work are stored in the *work-queue* to be processed.  This
+; parallelism work are stored in the *work-queue* to be processed.  This
 ; provides us the guarantee that if all worker threads were immediately to
 ; finish their piece of parallelism work, that each of them would immediately
 ; be able to grab another piece from the work queue.
@@ -1082,43 +1146,15 @@
 
   (* 4 *core-count*))
 
-(defconstant *total-work-limit* ; unassigned, started, resumed AND pending
-
-; The number of pieces of work in the system, *parallelism-work-count*, must be
-; less than *total-work-limit* in order to enable creation of new pieces of
-; work.  (However, we could go from 49 to 69 pieces of work when encountering a
-; pand; just not from 50 to 52.)
-
-; Why limit the amount of work in the system?  :Doc parallelism-how-to
-; (subtopic "Another Granularity Issue Related to Thread Limitations") provides
-; an example showing how cdr recursion can rapidly create threads.  That
-; example shows that if there is no limit on the amount of work we may create,
-; then eventually, many successive cdrs starting at the top will correspond to
-; waiting threads.  If we do not limit the amount of work that can be created,
-; this can exhaust the supply of Lisp threads available to process the elements
-; of the list.
-
-  (let ((val
-
-; Warning: It is possible, in principle to create (+ val
-; *max-idle-thread-count*) threads.  Presumably you'll get a hard Lisp error
-; (or seg fault!) if your Lisp cannot create that many threads.
-
-         50)
-        (bound (* 2 *core-count*)))
-    (when (< val bound)
-      (error "The variable *total-work-limit* needs to be at least ~s, i.e., ~%~
-              at least double the *core-count*.  Please redefine ~%~
-              *total-work-limit* so that it is not ~s."
-             bound
-             val))
-    val))
-
 (defconstant *max-idle-thread-count* 
 
+; Note that although this is a defconstant and
+; *unassigned-and-active-work-count-limit* is a defvar, David Rager says that
+; there's no particular reason he recalls for this distinction.
+
 ; We don't want to spawn more worker threads (which are initially idle) when we
 ; already have sufficiently many idle worker threads.  We use
 ; *max-idle-thread-count* to limit this spawning in function
 ; spawn-worker-threads-if-needed.
 
-(* 2 *core-count*))
+  (* 2 *core-count*))
