@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -353,6 +347,16 @@
 
 (defparameter *reset-parallelism-variables* nil)
 
+(defparameter *reset-core-count-too*
+
+; This variable has a relatively unsophisticated use: When Rager runs his
+; dissertation performance test scripts, sometimes he adjusts the number of
+; cpu-cores to be a factor of the actual cpu-core count.  In this case we are
+; just testing, and, to avoid resetting the core count variable every time we
+; reset the parallelism system, we will want to set this variable to nil.
+
+  t)
+
 (defun reset-parallelism-variables ()
 
 ; We use this function (a) to kill all worker threads, (b) to reset "most" of
@@ -368,12 +372,23 @@
 ; prompt after entering a raw Lisp break (using
 ; *reset-parallelism-variables*).
 
+; This function is not to be confused with reset-future-parallelism-variables
+; (although it is similar in nature).
+
 ; (a) Kill all worker threads.
 
-  (send-die-to-all-except-initial-threads)
+  (send-die-to-worker-threads)
 
 ; (b) Reset "most" of the parallelism variables.
 
+  (when *reset-core-count-too*
+
+; We reset *core-count* and related variable(s) in case the current platform
+; has a different number of CPU cores than the compilation platform had.
+
+    (setf *core-count* (core-count-raw))
+    (setf *unassigned-and-active-work-count-limit* (* 4 *core-count*)))
+
   (setf *idle-thread-count* (make-atomically-modifiable-counter 0))
 
   (setf *idle-core-count*
@@ -394,24 +409,11 @@
 
   (setf *total-parallelism-piece-historical-count* 0)
 
-  (setf *initial-threads* (all-threads))
-
-  (setf *core-count* (core-count-raw))
-
   (setf *reset-parallelism-variables* nil)
 
   t ; return t
 )
 
-; Parallelism wart: consider deleting the following comment and commented 
-; code.
-
-; Once upon a time, we reset parallelism variables as a standard part of 
-; compilation to ensure that the code behind declaring variables and resetting
-; variables is consistent and that we are in a stable state.
-
-; (reset-parallelism-variables)
-
 ;---------------------------------------------------------------------
 ; Section:  Work Consumer Code
 
@@ -654,7 +656,7 @@
 ; CPU cores busy, while limiting the total amount of work so that the number of
 ; threads necessary to evaluate that work does not execede the number of
 ; threads that the underlying Lisp supports creating.  (See also comments in
-; *total-work-limit*.)
+; default-total-parallelism-work-limit.)
 
 (defun add-work-list-to-queue (work-list)
 
@@ -794,6 +796,21 @@
           result-array
           children-done-semaphore))))
 
+(defun pargs-parallelism-buffer-has-space-available ()
+  (< (atomically-modifiable-counter-read *unassigned-and-active-work-count*)
+     *unassigned-and-active-work-count-limit*))
+
+(defun not-too-many-pieces-of-parallelism-work-already-in-existence ()
+
+; Parallelism no-fix: we could fix the plet, pargs, pand, and por parallel
+; execution system to cause an error when this limit is exceeded.  However,
+; since there is no notion of ":full" parallel execution (like in the ACL2
+; waterfall) for these primitives (because these primitives only parallelize
+; when resources are avaiable), such an error would be meaningless.
+
+  (< (atomically-modifiable-counter-read *total-work-count*)
+     (f-get-global 'total-parallelism-work-limit *the-live-state*)))
+
 (defun parallelism-resources-available ()
 
 ; This function is our attempt to guess when resources are available.  When
@@ -808,11 +825,9 @@
 ; In summary, it is unneccessary to acquire a lock, because we just don't care
 ; if we miss a few chances to parallelize, or parallelize a few extra times.
 
-  (and (f-get-global 'parallel-evaluation-enabled *the-live-state*)
-       (< (atomically-modifiable-counter-read *unassigned-and-active-work-count*)
-          *unassigned-and-active-work-count-limit*)
-       (< (atomically-modifiable-counter-read *total-work-count*)
-          *total-work-limit*)))
+  (and (f-get-global 'parallel-execution-enabled *the-live-state*)
+       (pargs-parallelism-buffer-has-space-available)
+       (not-too-many-pieces-of-parallelism-work-already-in-existence)))
 
 (defun throw-threads-in-array (thread-array current-position)
 
@@ -1171,13 +1186,6 @@
                           ,@body)))
                    `(let ,bindings ,@declare-forms ,@body)))))))
 
-(defun and-list (x)
-  (declare (xargs :guard (true-listp x)))
-  (if (endp x)
-      t
-    (and (car x)
-         (and-list (cdr x)))))
-
 (defmacro pand (&rest forms)
 
 ; This is the raw Lisp version for threaded Lisps.
@@ -1199,14 +1207,6 @@
                    and-early-termination-function)
              (list 'if (cons 'and remainder-forms) t nil))))))
 
-(defun or-list (x)
-  (declare (xargs :guard (true-listp x)))
-  (if (endp x)
-      nil
-    (if (car x)
-        t
-        (or-list (cdr x)))))
-
 (defmacro por (&rest forms)
 
 ; This is the raw Lisp version for threaded Lisps.
@@ -1230,9 +1230,9 @@
              (list 'if (cons 'or remainder-forms) t nil))))))
 
 (defun signal-semaphores (sems)
-  (if (atom sems)
-      nil
-    (progn (signal-semaphore (car sems))
+  (cond ((endp sems)
+         nil)
+        (t (signal-semaphore (car sems))
            (signal-semaphores (cdr sems)))))
 
 (defmacro spec-mv-let (bindings computation body)
@@ -1246,7 +1246,8 @@
 ; of work.
 
         (or (equal (car body) 'mv-let@par)
-            (equal (car body) 'mv-let))))
+            (equal (car body) 'mv-let)
+            (equal (car body) 'mv?-let))))
    
   (let* ((inner-let (car body))
          (inner-bindings (cadr body))
@@ -1273,73 +1274,148 @@
             (progn (future-abort the-very-obscure-feature)
                    ,false-branch)))))))
 
-; Parallelism wart: should we make a function called "set-gc-threshold" and
-; document it in a performance section?
-
-; Parallelism wart: should there be a documentation section called performance,
-; with links to sections "Parallelism", "Hons", and "Set-gc-threshold"?
-
-#+ccl
-(ccl:egc nil)
-
-#+ccl
-(ccl:set-lisp-heap-gc-threshold (expt 2 30))
-
-#+sbcl
-(setf (sb-ext:bytes-consed-between-gcs) (1- (expt 2 30)))
-
-(defvar *resource-and-timing-based-parallelizations*
+(defun number-of-active-threads-aux (threads acc)
+  #-ccl
+  (declare (ignore threads acc))
+  #-ccl
   0
-  "Tracks the number of times that we parallelize execution when
-  waterfall-parallelism is set to :resource-and-timing-based")
+  #+ccl
+  (cond ((atom threads)
+         acc)
+        ((equal (ccl:process-whostate (car threads)) "Active")
+         (number-of-active-threads-aux (cdr threads) (1+ acc)))
+        (t (number-of-active-threads-aux (cdr threads) acc))))
 
-(defvar *resource-and-timing-based-serializations*
+(defun number-of-active-threads ()
+  (number-of-active-threads-aux (all-threads) 0))
+
+(defun number-of-threads-waiting-on-a-child-aux (threads acc)
+  #-ccl
+  (declare (ignore threads acc))
+  #-ccl
   0
-  "Tracks the number of times that we do not parallize execution when
-  waterfall-parallelism is set to :resource-and-timing-based")
+  #+ccl
+  (cond ((atom threads)
+         acc)
+        ((equal (ccl:process-whostate (car threads)) "semaphore wait")
+         (number-of-threads-waiting-on-a-child-aux (cdr threads) (1+ acc)))
+        (t (number-of-threads-waiting-on-a-child-aux (cdr threads) acc))))
 
-(defvar *resource-based-parallelizations*
-  0
-  "Tracks the number of times that we parallelize execution when
-  waterfall-parallelism is set to :resource-based")
+(defun number-of-threads-waiting-on-a-child ()
+  (number-of-threads-waiting-on-a-child-aux (all-threads) 0))
 
-(defvar *resource-based-serializations*
-  0
-  "Tracks the number of times that we do not parallize execution when
-  waterfall-parallelism is set to :resource-based")
+(defun future-queue-length ()
 
-(defun print-interesting-parallelism-variables()
-  (format t "Printing vars related to executing in parallel.~%")
-  (format t "*idle-future-core-count* is ~s~%"
-          (atomically-modifiable-counter-read *idle-future-core-count*))
-  (format t "*idle-future-resumptive-core-count* is ~s~%"
-          (atomically-modifiable-counter-read
-           *idle-future-resumptive-core-count*))
-  (format t "*idle-future-thread-count* is ~s~%"
-          (atomically-modifiable-counter-read *idle-future-thread-count*))
-  (format t "*unassigned-and-active-future-count* is ~s~%"
-          (atomically-modifiable-counter-read
-           *unassigned-and-active-future-count*))
-  (format t "*total-future-count* is ~s~%"
-          (atomically-modifiable-counter-read *total-future-count*))  
-  (format t "*futures-resources-available-count* is ~s~%"
-          *futures-resources-available-count*)
-  (format t "*futures-resources-unavailable-count* is ~s~%"
-          *futures-resources-unavailable-count*)
+; At one point this was simply the difference between the *last-slot-saved* and
+; the *last-slot-taken*.  However, since we grab work from the work queue
+; before actually processing it with an idle cpu core, it is also necessary to
+; include the number of threads taht are waiting for a starting core.
 
-  (format t "~%Printing vars related to aborting futures.~%")
-  (format t "*aborted-futures-total* is ~s~%" *aborted-futures-total*)
-  (format t "*aborted-futures-via-throw* is ~s~%" *aborted-futures-via-throw*)
-  (format t "*aborted-futures-via-flag* is ~s~%" *aborted-futures-via-flag*)
-  (format t "*almost-aborted-future-count* is ~s~%~%"
-          *almost-aborted-future-count*)
+  (+ (- *last-slot-saved* *last-slot-taken*)
+     *threads-waiting-for-starting-core*
 
-  (format t "*resource-based-parallelizations* is ~s~%"
-          *resource-based-parallelizations*)
-  (format t "*resource-based-serializations* is ~s~%"
-          *resource-based-serializations*)
+; I intentionally ignore the threads that have incremented *last-slot-taken*
+; but not yet entered claim-starting-core.  I could come up with some mechanism
+; to track these, but there should be an insignificant number of them (less
+; than the total number of hardware threads, as of 2012-07), and it's not worth
+; it right now.
 
-  (format t "*resource-and-timing-based-parallelizations* is ~s~%"
-          *resource-and-timing-based-parallelizations*)
-  (format t "*resource-and-timing-based-serializations* is ~s~%"
-          *resource-and-timing-based-serializations*))
+     ))
+
+(defun total-number-of-threads ()
+  (length (all-threads)))
+
+(defvar *refresh-rate-indicator* 0)
+
+(defmacro value-of-symbol (var)
+  (when (not (or (fboundp var)
+                 (symbolp var)))
+    (error "value-of-symbol requires a symbol or function name as its argument"))
+  (cond ((constantp var)
+         `(format nil " Constant ~s is ~s~% " ,(symbol-name var) ,var))
+        ((fboundp var)
+         `(format nil " Stat     ~s is ~s~% " ,(symbol-name var) (,var)))
+        ((boundp-global var *the-live-state*)
+         `(format nil " Stat     ~s is ~s~% " ,(symbol-name var) 
+                  ,(f-get-global var *the-live-state*)))
+        (t
+         `(format nil " Variable ~s is ~s~% " ,(symbol-name var) ,var))))
+
+(defun acl2p-sum-list1 (lst acc)
+  (cond ((endp lst)
+         acc)
+        (t (acl2p-sum-list1 (cdr lst)
+                            (+ (car lst) acc)))))
+
+(defun acl2p-sum-list (lst)
+
+; An arcane name is chosen so that we don't conflict with other implementations
+; of "sum-list".
+
+  (acl2p-sum-list1 lst 0))
+
+(defun average-future-queue-size ()
+  (* 1.0 (/ (acl2p-sum-list *future-queue-length-history*)
+            (length *future-queue-length-history*))))
+
+(defun print-interesting-parallelism-variables-str ()
+  (incf *refresh-rate-indicator*)
+  (setf *future-queue-length-history* 
+
+; Note that this setf isn't thread safe, but if we lose one entry in the
+; history, we don't really care -- it's just a debugging tool anyway.
+
+        (cons (future-queue-length)
+              *future-queue-length-history*))
+
+  (concatenate 
+   'string
+   (format nil "  Printing stats related to executing proofs in parallel.~% ")
+   (value-of-symbol *idle-future-core-count*)
+   (value-of-symbol *idle-future-resumptive-core-count*)
+   (value-of-symbol *idle-future-thread-count*)
+   (value-of-symbol *threads-waiting-for-starting-core*)
+   (value-of-symbol number-of-idle-threads-and-threads-waiting-for-a-starting-core)
+   (value-of-symbol total-number-of-threads)
+
+   (format nil "~% ")
+
+   (value-of-symbol *unassigned-and-active-future-count*)
+   (value-of-symbol *unassigned-and-active-work-count-limit*)
+
+   (value-of-symbol *total-future-count*)
+   (value-of-symbol total-parallelism-work-limit)
+
+   (format nil "~% ")
+   (value-of-symbol number-of-active-threads)
+   (value-of-symbol number-of-threads-waiting-on-a-child)
+
+   (format nil "~% ")
+   (value-of-symbol *last-slot-taken*)
+   (value-of-symbol *last-slot-saved*)
+   (value-of-symbol future-queue-length)
+   (value-of-symbol average-future-queue-size)
+
+
+   (format nil "~% ")
+   (value-of-symbol *resource-based-parallelizations*)
+   (value-of-symbol *resource-based-serializations*)
+
+   (value-of-symbol *resource-and-timing-based-parallelizations*)
+   (value-of-symbol *resource-and-timing-based-serializations*)
+
+   (value-of-symbol *futures-resources-available-count*)
+   (value-of-symbol *futures-resources-unavailable-count*)
+
+   (format nil "~% ")
+   (format nil " Printing stats related to aborting futures.~% ")
+   (value-of-symbol *aborted-futures-total*)
+   (value-of-symbol *aborted-futures-via-throw*)
+   (value-of-symbol *aborted-futures-via-flag*)
+   (value-of-symbol *almost-aborted-future-count*)
+
+   (format nil "~% ")
+   (value-of-symbol *refresh-rate-indicator*)))
+
+(defun print-interesting-parallelism-variables ()
+  (format t (print-interesting-parallelism-variables-str)))
