@@ -1,22 +1,16 @@
-; ACL2 Version 4.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2011  University of Texas at Austin
+; ACL2 Version 6.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2012, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
 
 ; This program is free software; you can redistribute it and/or modify
-; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2 of the License, or
-; (at your option) any later version.
+; it under the terms of the LICENSE file distributed with ACL2.
 
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-; GNU General Public License for more details.
-
-; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; LICENSE for more details.
 
 ; Written by:  Matt Kaufmann               and J Strother Moore
 ; email:       Kaufmann@cs.utexas.edu      and Moore@cs.utexas.edu
@@ -157,9 +151,10 @@
 ; < depends-on.
 
 (defrec goal
-  (conc (hyps . current-addr)
-        (goal-name . depends-on))
-  nil)
+  (conc depends-on
+        (hyps . current-addr)
+        goal-name)
+  t)
 
 (defconst *goal-fields*
   '(conc hyps current-addr goal-name depends-on))
@@ -179,13 +174,9 @@
 (defmacro depends-on (&optional ss-supplied-p)
   `(access goal (car (goals ,ss-supplied-p)) :depends-on))
 
-(defmacro make-official-pc-command (sym &optional pre-package)
-  (if pre-package
-      `(intern-in-package-of-symbol
-        (symbol-name ,sym)
-        (caar (table-alist 'pc-command-table (w state))))
-    `(intern-in-package-of-symbol (symbol-name ,sym)
-                                  'acl2-pc::any-symbol)))
+(defmacro make-official-pc-command (sym)
+  `(intern-in-package-of-symbol (symbol-name ,sym)
+                                'acl2-pc::acl2-pkg-witness))
 
 (defun intern-in-keyword-package (sym)
   (declare (xargs :guard (symbolp sym)))
@@ -210,7 +201,7 @@
 (defmacro change-pc-state (pc-s &rest args)
   (list* 'change 'pc-state pc-s args))
 
-(defun make-official-pc-instr (instr state)
+(defun make-official-pc-instr (instr)
 
 ; This function always returns a syntactically legal instruction, i.e., a true
 ; list whose car is a symbol in the ACL2-PC package
@@ -218,14 +209,14 @@
   (if (consp instr)
       (if (and (symbolp (car instr))
                (true-listp (cdr instr)))
-          (cons (make-official-pc-command (car instr) t) (cdr instr))
-        (list (make-official-pc-command 'illegal t) instr))
+          (cons (make-official-pc-command (car instr)) (cdr instr))
+        (list (make-official-pc-command 'illegal) instr))
     (if (symbolp instr)
-        (list (make-official-pc-command instr t))
+        (list (make-official-pc-command instr))
       (if (and (integerp instr)
                (> instr 0))
-          (list (make-official-pc-command 'dv t) instr)
-        (list (make-official-pc-command 'illegal t) instr)))))
+          (list (make-official-pc-command 'dv) instr)
+        (list (make-official-pc-command 'illegal) instr)))))
 
 (defun check-formals-length (formals args fn ctx state)
   (declare (xargs :guard (and (symbol-listp formals)
@@ -425,16 +416,44 @@
            (cons ',raw-name args)
            ,(let-form-for-pc-state-vars (car (last body)))))))))
 
+(defun pc-command-table-guard (key val wrld)
+
+; We wrap the pc-command-table guard into this function so that we can redefine
+; it when modifying the ACL2 system.
+
+  (and (function-symbolp key wrld)
+       (or (eq val 'macro)
+           (eq val 'atomic-macro)
+           (eq val 'meta)
+           (and (eq val 'primitive)
+                (global-val 'boot-strap-flg wrld)))))
+
+(table pc-command-table nil nil
+       :guard
+
+; Before adding this table guard after Version_4.3, we were able to certify the
+; following book.
+
+;   (in-package "ACL2")
+;   (program)
+;   (set-state-ok t)
+;   (define-pc-primitive foo (&rest rest-args)
+;     (declare (ignore rest-args))
+;     (mv (change-pc-state pc-state :goals (cdr goals))
+;         state))
+;   (logic)
+;   (defthm bug
+;     nil
+;     :instructions (:foo)
+;     :rule-classes nil)
+
+       (pc-command-table-guard key val world))
+
 (defmacro add-pc-command (name command-type)
   `(table pc-command-table ',name ,command-type))
 
-(defun table-get (name key wrld)
-  (assoc-equal key
-               (getprop name 'table-alist nil
-                        'current-acl2-world wrld)))
-
 (defmacro pc-command-type (name)
-  `(cdr (table-get 'pc-command-table ,name (w state))))
+  `(cdr (assoc-equal ,name (table-alist 'pc-command-table (w state)))))
 
 (defmacro print-no-change3 (&optional str alist (col '0))
   `(pprogn (print-no-change-fn ,str ,alist ,col state)
@@ -565,7 +584,7 @@
 ; Notice that unlike Lisp macros, the global Lisp state is available for the
 ; expansion.  Hence we can query the ACL2 database etc.
 
-  (let ((instr (make-official-pc-instr raw-instr state)))
+  (let ((instr (make-official-pc-instr raw-instr)))
 
 ; Notice that instr is syntactically valid, i.e. is a true-listp headed by a
 ; symbol in the acl2-pc package -- even if raw-instr isn't of this form.
@@ -749,18 +768,27 @@
   (if forced-goals
       (if goal-unproved-p
           (cons (if known-assumptions
-                    (change goal (car remaining-goals)
-                            :hyps
-                            (append (access goal (car remaining-goals) :hyps)
-                                    known-assumptions))
-                    (car remaining-goals))
+                    (if forced-goals
+                        (change goal (car remaining-goals)
+                                :hyps
+                                (append (access goal (car remaining-goals) :hyps)
+                                        known-assumptions)
+                                :depends-on (+ (access goal
+                                                       (car remaining-goals)
+                                                       :depends-on)
+                                               (length forced-goals)))
+                      (change goal (car remaining-goals)
+                              :hyps
+                              (append (access goal (car remaining-goals) :hyps)
+                                      known-assumptions)))
+                  (car remaining-goals))
                 (append forced-goals (cdr remaining-goals)))
-          (append forced-goals remaining-goals))
+        (append forced-goals remaining-goals))
 
 ; Otherwise, we assume that since forced-goals is nil, assns is nil.
 ; This saves us the cons above.
 
-      remaining-goals))
+    remaining-goals))
 
 (defun unproved-goals (pc-state)
   (let ((goals (access pc-state pc-state :goals)))
@@ -803,16 +831,17 @@
    (let* ((goals (goals))
           (wrld (w state))
           (old-tag-tree (tag-tree)))
-     (if (null goals)
-         (pprogn (print-all-goals-proved-message state)
-                 (mv nil nil state))
+     (cond
+      ((null goals)
+       (pprogn (print-all-goals-proved-message state)
+               (mv nil nil state)))
+      (t
        (mv-let
         (erp stobjs-out/vals state)
         (trans-eval (list (car instr)
                           (list 'quote (cdr instr))
                           'state)
-                    'pc-single-step
-                    state t)
+                    'pc-single-step state t)
         (let ((vals (cdr stobjs-out/vals)))
 
 ; Vals is (x replaced-state), where x is a pc-state or nil.
@@ -861,70 +890,73 @@
                               (cons #\1 (access assumnote assumnote :target)))))
                      (mv nil nil state)))
                    (t
-                    (let* ((remaining-goals (unproved-goals (car vals)))
-
-; Just what is the invariant on primitive instructions?  Can they leave a goal
-; with conclusion *t* other than the top one being operated upon?
-
-                           (goal-name (goal-name))
+                    (let* ((returned-pc-state (car vals))
+                           (remaining-goals (unproved-goals returned-pc-state))
+                           (goal-name (goal-name)) ; original goal-name
                            (goal-unproved-p
                             (and remaining-goals
                                  (equal goal-name
-                                        (access goal (car remaining-goals) :goal-name))))
-                           (hyps (hyps))
-                           (depends-on (depends-on)))
+                                        (access goal (car remaining-goals)
+                                                :goal-name))))
+                           (hyps (hyps)) ; original hyps
+                           (returned-goal
+                            (let* ((goals (access pc-state returned-pc-state
+                                                  :goals)))
+                              (and goals
+                                   (equal goal-name
+                                          (access goal (car goals) :goal-name))
+                                   (car goals))))
+                           (depends-on
+                            (cond (returned-goal (access goal returned-goal
+                                                         :depends-on))
+                                  (t ; goal has disappeared; use old depends-on
+                                   (depends-on)))))
                       (mv-let
                        (cl-set assns ttree state)
                        (pc-process-assumptions pc-ens ttree wrld state)
                        (mv-let
                         (contradictionp hyps-type-alist ttree0)
-                        (if (and assns goal-unproved-p)
-                            (type-alist-clause (dumb-negate-lit-lst hyps)
-                                               nil nil nil pc-ens wrld
-                                               nil nil)
-                          ;; else don't bother here
-                          (mv nil nil nil))
+                        (cond ((and assns goal-unproved-p)
+                               (type-alist-clause (dumb-negate-lit-lst hyps)
+                                                  nil nil nil pc-ens wrld nil
+                                                  nil))
+                              (t ; else don't bother
+                               (mv nil nil nil)))
                         (cond
                          (contradictionp
                           (er-let*
-                              ((new-pc-state
-                                (let ((local-ttree (cons-tag-trees ttree ttree0)))
-                                  (accumulate-ttree-in-pc-state
-                                   (change-pc-state
-                                    (car vals)
-                                    :goals
-                                    (cdr goals)
-                                    :tag-tree
-
-; We are concerned about the comment in note-2-8-bug-fixes about the
-; proof-checker running slowly because of tag-trees.  So we play it safe here
-; by avoiding duplication.
-
-                                    (cons-tag-trees local-ttree old-tag-tree)
-                                    :local-tag-tree
-                                    local-ttree)
-                                   state))))
-                            (pprogn (io? proof-checker nil state
-                                         (instr goal-name)
-                                         (fms0 "~|AHA!  A contradiction has been ~
-                                             discovered in the hypotheses of ~
-                                             goal ~x0 in the course of ~
-                                             executing instruction ~x1, in ~
-                                             the process of preparing to deal ~
-                                             with forced assumptions.~|"
-                                               (list (cons #\0 goal-name)
-                                                     (cons #\0 instr))
-                                               0 nil))
-                                    (io? proof-checker nil state
-                                         (goals)
-                                         (maybe-print-proved-goal-message
-                                          (car goals) goals (cdr goals) state))
-                                    (pc-assign
-                                     state-stack
-                                     (cons
-                                      new-pc-state
-                                      (state-stack)))
-                                    (value new-pc-state))))
+                           ((new-pc-state
+                             (let ((local-ttree (cons-tag-trees ttree ttree0)))
+                               (accumulate-ttree-in-pc-state
+                                (change-pc-state
+                                 (car vals)
+                                 :goals
+                                 (cdr goals)
+                                 :tag-tree
+                                 (cons-tag-trees local-ttree old-tag-tree)
+                                 :local-tag-tree
+                                 local-ttree)
+                                state))))
+                           (pprogn (io? proof-checker nil state
+                                        (instr goal-name)
+                                        (fms0 "~|AHA!  A contradiction has ~
+                                               been discovered in the ~
+                                               hypotheses of goal ~x0 in the ~
+                                               course of executing ~
+                                               instruction ~x1, in the ~
+                                               process of preparing to deal ~
+                                               with forced assumptions.~|"
+                                              (list (cons #\0 goal-name)
+                                                    (cons #\0 instr))
+                                              0 nil))
+                                   (io? proof-checker nil state
+                                        (goals)
+                                        (maybe-print-proved-goal-message
+                                         (car goals) goals (cdr goals) state))
+                                   (pc-assign state-stack
+                                              (cons new-pc-state
+                                                    (state-stack)))
+                                   (value new-pc-state))))
                          (t
                           (let* ((termlist
                                   (cl-set-to-implications cl-set))
@@ -948,39 +980,38 @@
                                        (accumulate-ttree-in-pc-state
                                         pc-state-1
                                         state)))
-                              (pprogn
-                               (cond
-                                (forced-goals
-                                 (io? proof-checker nil state
-                                      (forced-goals)
-                                      (fms0
-                                       "~|NOTE (forcing):  Creating ~n0 ~
-                                             new ~#1~[~/goal~/goals~] due to ~
-                                             forcing assumptions.~%"
-                                       (list
-                                        (cons #\0 (length forced-goals))
-                                        (cons #\1
-                                              (zero-one-or-more
-                                               (length forced-goals)))))))
-                                (t state))
-                               (io? proof-checker nil state
-                                    (new-goals goals)
-                                    (maybe-print-proved-goal-message
-                                     (car goals) goals new-goals state))
-                               (pc-assign
-                                state-stack
-                                (cons
-                                 new-pc-state
-                                 (state-stack)))
-                               (value new-pc-state))))))))))))))
+                                     (pprogn
+                                      (cond
+                                       (forced-goals
+                                        (io? proof-checker nil state
+                                             (forced-goals)
+                                             (fms0
+                                              "~|~%NOTE (forcing):  Creating ~
+                                               ~n0 new ~#1~[~/goal~/goals~] ~
+                                               due to FORCE or CASE-SPLIT ~
+                                               hypotheses of rules.~%"
+                                              (list
+                                               (cons #\0 (length forced-goals))
+                                               (cons #\1
+                                                     (zero-one-or-more
+                                                      (length forced-goals)))))))
+                                       (t state))
+                                      (io? proof-checker nil state
+                                           (new-goals goals)
+                                           (maybe-print-proved-goal-message
+                                            (car goals) goals new-goals state))
+                                      (pc-assign
+                                       state-stack
+                                       (cons new-pc-state (state-stack)))
+                                      (value new-pc-state))))))))))))))
               (t
-               (mv nil nil state))))))))))))
+               (mv nil nil state)))))))))))))
 
 (defun maybe-print-macroexpansion (instr raw-instr state)
   (let ((pc-print-macroexpansion-flg (pc-print-macroexpansion-flg)))
     (if (and pc-print-macroexpansion-flg
-             (not (eq (car instr) (make-official-pc-command 'lisp t)))
-             (not (equal instr (make-official-pc-instr raw-instr state))))
+             (not (eq (car instr) (make-official-pc-command 'lisp)))
+             (not (equal instr (make-official-pc-instr raw-instr))))
         (io? proof-checker nil state
              (pc-print-macroexpansion-flg instr)
              (fms0 ">> ~x0~|" (list (cons #\0 instr)) 0
@@ -1113,66 +1144,118 @@
 
 (defconst *pc-complete-signal* 'acl2-pc-complete)
 
+(defmacro catch-throw-to-local-top-level (form)
+
+; Form should evaluate to (mv erp val state) or else throw to
+; 'local-top-level.
+
+  #+acl2-loop-only
+  `(mv-let (cttltl-erp cttltl-val state)
+           (read-acl2-oracle state)
+           (cond ((or cttltl-erp cttltl-val)
+                  (mv 'thrown-to-local-top-level
+                      (or cttltl-erp cttltl-val)
+                      state))
+                 (t (check-vars-not-free
+                     (cttltl-erp cttltl-val)
+                     ,form))))
+  #-acl2-loop-only
+  (let ((thrown-var (gensym)))
+    `(let* ((,thrown-var t)
+            (trip (catch 'local-top-level
+                    (prog1
+                        (mv-list 3 ,form)
+                      (setq ,thrown-var nil)))))
+       (cond (,thrown-var
+              (mv 'thrown-to-local-top-level trip state))
+             (t (assert$ (eq *the-live-state*
+                             (caddr trip))
+                         (mv (car trip) (cadr trip) state)))))))
+
 (defun pc-main-loop (instr-list quit-conditions last-value
                                 pc-print-prompt-and-instr-flg state)
-  ;; Returns an error triple whose state has
-  ;; the new state-stack "installed".  Here instr-list is a (true) list of
-  ;; instructions or else is a non-NIL atom, probably *standard-oi*,
-  ;; from which the instructions are to be read.  Notice that by
-  ;; taking (append instrs <stream>), one is able to get the system to read
-  ;; from the instr-list input until there are no more instructions,
-  ;; and then to read from the stream.
-  ;;    quit-conditions indicates when we want to quit; it is a list
-  ;; of atoms.  'signal means that we quit when
-  ;; there's a signal, while 'value means that we quit when the
-  ;; value is nil.  If quit-conditions is empty (nil) then we keep
-  ;; going, no matter what.  However, a signal to quit (i.e. *pc-complete-signal*)
-  ;; is always obeyed if 'exit is a quit-condition.
-  ;;  This only returns non-nil if we exit successfully,
-  ;; or if all instructions succeed (null erp, non-nil value) without error.
+
+; Returns an error triple whose state has the new state-stack "installed".
+; Here instr-list is a (true) list of instructions or else is a non-NIL atom,
+; probably *standard-oi*, from which the instructions are to be read.  Notice
+; that by taking (append instrs <stream>), one is able to get the system to
+; read from the instr-list input until there are no more instructions, and then
+; to read from the stream.
+
+; Quit-conditions indicates when we want to quit; it is a list of atoms.
+; 'signal means that we quit when there's a signal, while 'value means that we
+; quit when the value is nil.  If quit-conditions is empty (nil) then we keep
+; going, no matter what.  However, a signal to quit (i.e. *pc-complete-signal*)
+; is always obeyed if 'exit is a quit-condition.
+
+; This only returns non-nil if we exit successfully, or if all instructions
+; succeed (null erp, non-nil value) without error.
 
   (if (null instr-list)
       (mv nil last-value state)
-    (mv-let (col state)
-            (if pc-print-prompt-and-instr-flg
-                (print-pc-prompt state)
-              (mv 0 state))
-            (mv-let (erp instr state)
-                    (if (consp instr-list)
-                        (pprogn (if pc-print-prompt-and-instr-flg
-                                    (io? proof-checker nil state
-                                         (col instr-list)
-                                         (fms0 "~y0~|"
-                                               (list (cons #\0
-                                                           (car instr-list)))
-                                               col))
-                                  state)
-                                (value (car instr-list)))
-                      (state-global-let*
-                       ((infixp nil))
-                       (read-object instr-list state)))
-                    (declare (ignore erp))
-                    (mv-let (signal val state)
-                            (pc-single-step (make-official-pc-instr instr state) state) 
-                            (cond
-                             ((and signal
-                                   (or (member-eq 'signal quit-conditions)
-                                       (and (equal signal *pc-complete-signal*)
-                                            (member-eq 'exit quit-conditions))))
-                              (mv signal val state))
-                             ((and (null val) (member-eq 'value quit-conditions))
-                              (mv signal val state))
-                             (t (pc-main-loop
-                                 (if (consp instr-list)
-                                     (cdr instr-list)
-                                   instr-list)
-                                 quit-conditions
-                                 ;; We ultimately "succeed" if and only if every instruction "succeeds".
-                                 ;; If that decision is changed, then change documentation in CATCH
-                                 ;; meta command.
-                                 (and last-value (null signal) val)
-                                 pc-print-prompt-and-instr-flg
-                                 state))))))))
+    (mv-let
+     (col state)
+     (if pc-print-prompt-and-instr-flg
+         (print-pc-prompt state)
+       (mv 0 state))
+     (mv-let
+      (erp instr state)
+      (if (consp instr-list)
+          (pprogn (if pc-print-prompt-and-instr-flg
+                      (io? proof-checker nil state
+                           (col instr-list)
+                           (fms0 "~y0~|"
+                                 (list (cons #\0
+                                             (car instr-list)))
+                                 col))
+                    state)
+                  (value (car instr-list)))
+        (state-global-let*
+         ((infixp nil))
+         (catch-throw-to-local-top-level
+          (read-object instr-list state))))
+      (cond
+       (erp ; read error
+        (pprogn
+         (io? proof-checker nil state nil
+              (fms0
+               "~|~%~
+                /----------------------------------------------------\\~%~
+                |        NOTE: Read error -- input discarded.        |~%~
+                | Submit EXIT if you want to exit the proof-checker. |~%~
+                \\----------------------------------------------------/~%"))
+         (pc-main-loop instr-list quit-conditions last-value
+                       pc-print-prompt-and-instr-flg state)))
+       (t (mv-let
+           (signal val state)
+           (catch-throw-to-local-top-level
+            (pc-single-step
+             (make-official-pc-instr instr)
+             state))
+           (cond
+            ((and signal
+                  (or (member-eq 'signal quit-conditions)
+                      (and (eq signal *pc-complete-signal*)
+                           (member-eq 'exit quit-conditions))))
+             (mv signal val state))
+            ((and (null val) (member-eq 'value quit-conditions))
+             (mv signal val state))
+            (t (let ((new-last-value
+
+; We ultimately "succeed" if and only if every instruction "succeeds".  We use
+; a let-binding here in order to avoid an Allegro CL compiler bug (found using
+; Allegro CL 8.0, but told by Franz support that it still exists in Allegro CL
+; 9.0).
+
+                      (and last-value (null signal) val)))
+                 (pc-main-loop
+                  (if (consp instr-list)
+                      (cdr instr-list)
+                    instr-list)
+                  quit-conditions
+                  new-last-value
+                  pc-print-prompt-and-instr-flg
+                  state)))))))))))
 
 (defun make-initial-goal (term)
   (make goal
@@ -1274,6 +1357,7 @@
        (state-global-let*
         ((in-verify-flg t)
          (print-base 10)
+         (print-radix nil)
          (inhibit-output-lst
           (remove1-eq 'proof-checker
                       (f-get-global 'inhibit-output-lst state))))
@@ -1302,6 +1386,7 @@
            (state-global-let*
             ((in-verify-flg t)
              (print-base 10)
+             (print-radix nil)
              (inhibit-output-lst
               (remove1-eq 'proof-checker
                           (f-get-global 'inhibit-output-lst state))))
